@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
  
-#ifdef HAVE_NCURSES
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -49,6 +48,7 @@
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
 #endif
+#include <stdarg.h>
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
@@ -59,10 +59,27 @@
 #include "askloc.h"
 #include "log.h"
 
-extern const char *monstr[];
+static char *td_getcwd(char *buf, unsigned long size)
+{
+  /* buf must non-NULL*/
+#ifdef HAVE_GETCWD
+  if(getcwd(buf, size)!=NULL)
+    return buf;
+#endif
+  buf[0]='.';
+  buf[1]='\0';
+  return buf;
+}
 
-#define INTER_DIR (LINES-25+16)
-#define Y_DIR	  8
+char *get_default_location(void)
+{
+  char dst_directory[4096];
+  td_getcwd(dst_directory, sizeof(dst_directory));
+  return strdup(dst_directory);
+}
+
+#ifdef HAVE_NCURSES
+extern const char *monstr[];
 
 #ifdef __MINGW32__
 #define SPATH_SEP "\\"
@@ -97,7 +114,7 @@ void get_dos_drive_list(struct td_list_head *list)
     new_drive=(file_info_t*)MALLOC(sizeof(*new_drive));
     new_drive->name=(char*)MALLOC(4);
     sprintf(new_drive->name, "%c:/", i);
-    new_drive->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+    new_drive->st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
     td_list_add_tail(&new_drive->list, list);
   }
 }
@@ -158,7 +175,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
   do
   {
     DIR* dir;
-    static file_info_t dir_list = {
+    file_info_t dir_list = {
       .list = TD_LIST_HEAD_INIT(dir_list.list),
       .name = NULL
     };
@@ -200,8 +217,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
       while(1)
       {
         char current_file[4096];
-	struct dirent *dir_entrie;
-        dir_entrie=readdir(dir);
+	const struct dirent *dir_entrie=readdir(dir);
         if(dir_entrie==NULL)
 	  break;
 	/* hide filename beginning by '.' except '.' and '..' */
@@ -217,6 +233,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 #endif
           )
         {
+	  struct stat file_stat;
           strcpy(current_file,dst_directory);
 #if defined(DJGPP) || defined(__OS2__)
           if(current_file[0]!='\0'&&current_file[1]!='\0'&&current_file[2]!='\0'&&current_file[3]!='\0')
@@ -226,26 +243,34 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
               strcat(current_file,SPATH_SEP);
           strcat(current_file,dir_entrie->d_name);
 #ifdef HAVE_LSTAT
-          if(lstat(current_file,&file_info->stat)==0)
+          if(lstat(current_file,&file_stat)==0)
 #else
-            if(stat(current_file,&file_info->stat)==0)
+            if(stat(current_file,&file_stat)==0)
 #endif
 	    {
+	      file_info->st_ino=file_stat.st_ino;
+	      file_info->st_mode=file_stat.st_mode;
+	      file_info->st_uid=file_stat.st_uid;
+	      file_info->st_gid=file_stat.st_gid;
+	      file_info->st_size=file_stat.st_size;
+	      file_info->td_atime=file_stat.st_atime;
+	      file_info->td_mtime=file_stat.st_mtime;
+	      file_info->td_ctime=file_stat.st_ctime;
 #if defined(DJGPP) || defined(__OS2__)
 	      /* If the C library doesn't use posix definition, st_mode need to be fixed */
-	      if(S_ISDIR(file_info->stat.st_mode))
-		file_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+	      if(S_ISDIR(file_info->st_mode))
+		file_info->st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
 	      else
-		file_info->stat.st_mode=LINUX_S_IFREG|LINUX_S_IRWXUGO;
+		file_info->st_mode=LINUX_S_IFREG|LINUX_S_IRWXUGO;
 #endif
 #ifdef __CYGWIN__
 	      /* Fix Drive list */
 	      if(memcmp(dst_directory, "/cygdrive", 9)==0 && (dst_directory[10]=='\0' || dst_directory[11]=='\0'))
 	      {
-		file_info->stat.st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
-		file_info->stat.st_mtime=0;
-		file_info->stat.st_uid=0;
-		file_info->stat.st_gid=0;
+		file_info->st_mode=LINUX_S_IFDIR|LINUX_S_IRWXUGO;
+		file_info->td_mtime=0;
+		file_info->st_uid=0;
+		file_info->st_gid=0;
 	      }
 #endif
 	      file_info->name=strdup(dir_entrie->d_name);
@@ -263,41 +288,48 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
       struct td_list_head *current_file=dir_list.list.next;
       int offset=0;
       int pos_num=0;
-      int old_LINES=LINES;
+      int old_LINES=0;
+      int old_COLS=0;
       do
       {
 	int dst_directory_ok=0;
+	int line_directory;
+	int line_base;
 	if(old_LINES!=LINES)
 	{ /* Screen size has changed, reset to initial values */
 	  current_file=dir_list.list.next;
 	  offset=0;
 	  pos_num=0;
 	  old_LINES=LINES;
+	  old_COLS=COLS;
 	}
         aff_copy_short(window);
-        wmove(window,4,0);
+	line_directory=aff_txt(2, window, msg, src_dir);
+        wmove(window, line_directory, 0);
         wprintw(window,"Keys: ");
-
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
 	waddstr(window, "Arrow");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	wprintw(window," keys to select another directory");
-	wmove(window, 5, 6);
+        wmove(window, ++line_directory, 6);
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
 	waddstr(window, "C");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	wprintw(window, " when the destination is correct");
-	wmove(window, 6, 6);
+        wmove(window, ++line_directory, 6);
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
 	waddstr(window, "Q");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	waddstr(window," to quit");
+	line_directory++;
+	line_base=line_directory+1;
+	line_base+=(strlen("Directory ")+strlen(dst_directory))/old_COLS;
 	{
           struct td_list_head *file_walker = NULL;
           int i=0;
@@ -308,7 +340,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
             {
               file_info_t *file_info;
               file_info=td_list_entry(file_walker, file_info_t, list);
-              wmove(window, Y_DIR - 1 + i - offset, 0);
+              wmove(window, line_base - 1 + i - offset, 0);
               wclrtoeol(window);	/* before addstr for BSD compatibility */
               if(file_walker==current_file)
 	      {
@@ -323,16 +355,15 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 		dir_aff_entry(window,file_info);
 	      }
             }
-            if(offset+INTER_DIR<=i)
+            if(old_LINES-2 <= line_base - 1+i-offset)
               break;
           }
-	  wmove(window, Y_DIR + INTER_DIR, 4);
+	  wmove(window,  old_LINES-1, 4);
 	  wclrtoeol(window);
 	  if(file_walker!=&dir_list.list && file_walker->next!=&dir_list.list)
 	    wprintw(window, "Next");
         }
-	aff_txt(2, window, msg, src_dir);
-	wmove(window,7,0);
+	wmove(window, line_directory, 0);
 	wclrtoeol(window);	/* before addstr for BSD compatibility */
 	if(strcmp(dst_directory,".")==0)
 	{
@@ -366,7 +397,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
         wrefresh(window);
         do
         {
-	  int command=wgetch(window);
+	  const int command=wgetch(window);
           quit=ASK_LOCATION_WAITKEY;
 #if defined(KEY_MOUSE) && defined(ENABLE_MOUSE)
 	  if(command==KEY_MOUSE)
@@ -376,16 +407,16 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 	    {	/* When the user clicks left mouse button */
 	      if((event.bstate & BUTTON1_CLICKED) || (event.bstate & BUTTON1_DOUBLE_CLICKED))
 	      {
-		if(event.y >= Y_DIR && event.y < Y_DIR + INTER_DIR)
+		if(event.y => line_base - 1 && event.y < old_LINES)
 		{
 		  const int pos_num_old=pos_num;
 		  /* Disk selection */
-		  while(pos_num > event.y - (Y_DIR - offset) && current_file->prev!=&dir_list.list)
+		  while(pos_num > event.y - (line_base - offset) && current_file->prev!=&dir_list.list)
 		  {
 		    current_file=current_file->prev;
 		    pos_num--;
 		  }
-		  while(pos_num < event.y - (Y_DIR - offset) && current_file->next!=&dir_list.list)
+		  while(pos_num < event.y - (line_base - offset) && current_file->next!=&dir_list.list)
 		  {
 		    current_file=current_file->next;
 		    pos_num++;
@@ -444,7 +475,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
             case KEY_PPAGE:
               {
                 int i;
-                for(i=0; i<INTER_DIR-1 && current_file->prev!=&dir_list.list; i++)
+                for(i=0; i<old_LINES-line_base - 1-3 && current_file->prev!=&dir_list.list; i++)
                 {
                   current_file=current_file->prev;
                   pos_num--;
@@ -455,7 +486,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
             case KEY_NPAGE:
               {
                 int i;
-                for(i=0; i<INTER_DIR-1 && current_file->next!=&dir_list.list; i++)
+                for(i=0; i<old_LINES-line_base - 1-3 && current_file->next!=&dir_list.list; i++)
                 {
                   current_file=current_file->next;
                   pos_num++;
@@ -480,7 +511,7 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 		file_info_t *file_info;
 		file_info=td_list_entry(current_file, file_info_t, list);
 		if(current_file!=&dir_list.list &&
-		  (LINUX_S_ISDIR(file_info->stat.st_mode) || LINUX_S_ISLNK(file_info->stat.st_mode)))
+		  (LINUX_S_ISDIR(file_info->st_mode) || LINUX_S_ISLNK(file_info->st_mode)))
 		if(current_file!=&dir_list.list)
 		{
 		  if(strcmp(file_info->name, ".")==0)
@@ -509,11 +540,11 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
           }
 	  if(pos_num<offset)
 	    offset=pos_num;
-	  if(pos_num>=offset+INTER_DIR)
-	    offset=pos_num-INTER_DIR+1;
+	  if(offset+old_LINES <= pos_num+line_base+1)
+	    offset=pos_num + line_base + 2 - old_LINES;
         } while(quit==ASK_LOCATION_WAITKEY && old_LINES==LINES);
       } while(quit==ASK_LOCATION_UPDATE || old_LINES!=LINES);
-      delete_list_file_info(&dir_list.list);
+      delete_list_file(&dir_list);
     }
     else
     {
@@ -531,12 +562,11 @@ char *ask_location(const char*msg, const char *src_dir, const char *dst_org)
 
 static void dir_aff_entry(WINDOW *window, file_info_t *file_info)
 {
-  struct tm		*tm_p;
   char str[11];
   char		datestr[80];
-  if(file_info->stat.st_mtime!=0)
+  if(file_info->td_mtime!=0)
   {
-    tm_p = localtime(&file_info->stat.st_mtime);
+    const struct tm *tm_p= localtime(&file_info->td_mtime);
     snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
         tm_p->tm_mday, monstr[tm_p->tm_mon],
         1900 + tm_p->tm_year, tm_p->tm_hour,
@@ -545,10 +575,10 @@ static void dir_aff_entry(WINDOW *window, file_info_t *file_info)
   } else {
     strncpy(datestr, "                 ",sizeof(datestr));
   }
-  mode_string(file_info->stat.st_mode,str);
+  mode_string(file_info->st_mode, str);
   wprintw(window, "%s %5u %5u ", 
-      str, (unsigned int)file_info->stat.st_uid, (unsigned int)file_info->stat.st_gid);
-  wprintw(window, "%9llu", (long long unsigned int)file_info->stat.st_size);
+      str, (unsigned int)file_info->st_uid, (unsigned int)file_info->st_gid);
+  wprintw(window, "%9llu", (long long unsigned int)file_info->st_size);
   /* screen may overlap due to long filename */
   wprintw(window, " %s %s", datestr, file_info->name);
 }
@@ -561,5 +591,5 @@ static int aff_txt(int line, WINDOW *window, const char *_format, ...)
   va_end(ap);
   return line;
 }
-
 #endif
+
