@@ -41,10 +41,15 @@
 #include <io.h>
 #endif
 #endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "log.h"
 
+static int32_t secwest=0;
+
 static unsigned int up2power_aux(const unsigned int number);
 
 void *MALLOC(size_t size)
@@ -139,6 +144,31 @@ int strncasecmp(const char * s1, const char * s2, size_t len)
 }
 #endif
 
+#ifndef HAVE_STRCASESTR
+char * strcasestr (const char *haystack, const char *needle)
+{
+  const char *p, *startn = NULL, *np = NULL;
+  for (p = haystack; *p; p++)
+  {
+    if (np)
+    {
+      if (toupper(*p) == toupper(*np))
+      {
+	if (!*++np)
+	  return startn;
+      }
+      else
+	np = NULL;
+    }
+    else if (toupper(*p) == toupper(*needle))
+    {
+      np = needle + 1;
+      startn = p;
+    }
+  }
+  return NULL;
+}
+#endif
 unsigned int up2power(const unsigned int number)
 {
   /* log_trace("up2power(%u)=>%u\n",number, (1<<up2power_aux(number-1))); */
@@ -158,18 +188,18 @@ static unsigned int up2power_aux(const unsigned int number)
 void set_part_name(partition_t *partition,const char *src,const int max_size)
 {
   int i;
-  for(i=0;(i<max_size) && (src[i]!=(char)0);i++)
+  for(i=0; i<sizeof(partition->fsname)-1 && i<max_size && src[i]!='\0'; i++)
     partition->fsname[i]=src[i];
-  partition->fsname[i--]='\0';
+  partition->fsname[i]='\0';
 }
 
 void set_part_name_chomp(partition_t *partition, const unsigned char *src, const int max_size)
 {
   int i;
-  for(i=0;(i<max_size) && (src[i]!=(char)0);i++)
+  for(i=0; i<sizeof(partition->fsname) && i<max_size && src[i]!='\0'; i++)
     partition->fsname[i]=src[i];
-  partition->fsname[i--]='\0';
-  for(;(i>=0) && (src[i]==' ');i--);
+  partition->fsname[i]='\0';
+  for(i--; i>=0 && src[i]==' '; i--);
   partition->fsname[i+1]='\0';
 }
 
@@ -189,4 +219,57 @@ char* strip_dup(char* str)
   return strdup (str);
 }
 
+/* Convert a MS-DOS time/date pair to a UNIX date (seconds since 1 1 70). */
 
+int date_dos2unix(const unsigned short f_time, const unsigned short f_date)
+{
+  static const int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
+  /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */
+
+  int month,year,secs;
+
+  /* first subtract and mask after that... Otherwise, if
+     f_date == 0, bad things happen */
+  month = ((f_date >> 5) - 1) & 15;
+  year = f_date >> 9;
+  secs = (f_time & 31)*2+60*((f_time >> 5) & 63)+(f_time >> 11)*3600+86400*
+    ((f_date & 31)-1+day_n[month]+(year/4)+year*365-((year & 3) == 0 &&
+      month < 2 ? 1 : 0)+3653);
+  /* days since 1.1.70 plus 80's leap day */
+  return secs+secwest;
+}
+
+void set_secwest(void)
+{
+  const time_t t = time(NULL);
+  const struct  tm *tmptr = localtime(&t);
+#ifdef HAVE_STRUCT_TM_TM_GMTOFF
+  secwest = -1 * tmptr->tm_gmtoff;
+#elif defined (DJGPP) || defined(__ANDROID__)
+  secwest = 0;
+#else
+#if defined (__CYGWIN__)
+  secwest = _timezone;
+#else
+  secwest = timezone;
+#endif
+  /* account for daylight savings */
+  if (tmptr->tm_isdst)
+    secwest -= 3600;
+#endif
+}
+
+/**
+ * td_ntfs2utc - Convert an NTFS time to Unix time
+ * @time:  An NTFS time in 100ns units since 1601
+ *
+ * NTFS stores times as the number of 100ns intervals since January 1st 1601 at
+ * 00:00 UTC.  This system will not suffer from Y2K problems until ~57000AD.
+ *
+ * Return:  n  A Unix time (number of seconds since 1970)
+ */
+#define NTFS_TIME_OFFSET ((int64_t)(369 * 365 + 89) * 24 * 3600 * 10000000)
+time_t td_ntfs2utc (int64_t ntfstime)
+{
+  return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
+}
