@@ -36,11 +36,7 @@ struct efi_guid_s
   uint8_t  node[6];
 } __attribute__ ((__packed__));
 
-#define DEFAULT_SECTOR_SIZE     0x200
-
-#define MAX_CYLINDERS 65535
-#define MAX_HEADS 255
-#define MAX_SECTORS 63
+#define DEFAULT_SECTOR_SIZE     0x200u
 
 #define DISKNAME_MAX	64
 #define DISKDESCRIPTION_MAX	128
@@ -246,6 +242,9 @@ enum upart_type {
   UP_JFS,
   UP_LINSWAP,
   UP_LINSWAP2,
+  UP_LINSWAP_8K,
+  UP_LINSWAP2_8K,
+  UP_LINSWAP2_8KBE,
   UP_LUKS,
   UP_LVM,
   UP_LVM2,
@@ -263,7 +262,10 @@ enum upart_type {
   UP_SYSV4,
   UP_UFS,
   UP_UFS2,
+  UP_UFS_LE,
+  UP_UFS2_LE,
   UP_VMFS,
+  UP_WBFS,
   UP_XFS,
   UP_XFS2,
   UP_XFS3,
@@ -328,20 +330,20 @@ struct arch_fnct_struct
   const char *part_name;
   const char *part_name_option;
   const char *msg_part_type;
-  list_part_t *(*read_part)(disk_t *disk_car, const int verbose,const int saveheader);
-  int (*write_part)(disk_t *disk_car, const list_part_t *list_part, const int ro, const int verbose, const int align);
-  list_part_t *(*init_part_order)(const disk_t *disk_car, list_part_t *list_part);
+  list_part_t *(*read_part)(disk_t *disk, const int verbose,const int saveheader);
+  int (*write_part)(disk_t *disk, const list_part_t *list_part, const int ro, const int verbose, const int align);
+  list_part_t *(*init_part_order)(const disk_t *disk, list_part_t *list_part);
   /* geometry must be initialized to 0,0,0 in get_geometry_from_mbr()*/
   int (*get_geometry_from_mbr)(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
-  int (*check_part)(disk_t *disk_car,const int verbose,partition_t *partition, const int saveheader);
-  int (*write_MBR_code)(disk_t *disk_car);
-  void (*set_prev_status)(const disk_t *disk_car, partition_t *partition);
-  void (*set_next_status)(const disk_t *disk_car, partition_t *partition);
+  int (*check_part)(disk_t *disk,const int verbose,partition_t *partition, const int saveheader);
+  int (*write_MBR_code)(disk_t *disk);
+  void (*set_prev_status)(const disk_t *disk, partition_t *partition);
+  void (*set_next_status)(const disk_t *disk, partition_t *partition);
   int (*test_structure)(list_part_t *list_part);
   unsigned int (*get_part_type)(const partition_t *partition);
   int (*set_part_type)(partition_t *partition, unsigned int part_type);
-  void (*init_structure)(const disk_t *disk_car,list_part_t *list_part, const int verbose);
-  int (*erase_list_part)(disk_t *disk_car);
+  void (*init_structure)(const disk_t *disk,list_part_t *list_part, const int verbose);
+  int (*erase_list_part)(disk_t *disk);
   const char *(*get_partition_typename)(const partition_t *partition);
   int (*is_part_known)(const partition_t *partition);
 };
@@ -358,14 +360,14 @@ struct param_disk_struct
   char *model;
   char *serial_no;
   char *fw_rev;
-  const char *(*description)(disk_t *disk_car);
-  const char *(*description_short)(disk_t *disk_car);
-  void *(*pread_fast)(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
-  int (*pread)(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset);
-  int (*pwrite)(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
-  int (*sync)(disk_t *disk_car);
-  int (*clean)(disk_t *disk_car);
+  const char *(*description)(disk_t *disk);
+  const char *(*description_short)(disk_t *disk);
+  int (*pread)(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
+  int (*pwrite)(disk_t *disk, const void *buf, const unsigned int count, const uint64_t offset);
+  int (*sync)(disk_t *disk);
+  void (*clean)(disk_t *disk);
   const arch_fnct_t *arch;
+  const arch_fnct_t *arch_autodetected;
   void *data;
   uint64_t disk_real_size;
   uint64_t user_max;
@@ -427,6 +429,9 @@ unsigned int up2power(const unsigned int number);
 void set_part_name(partition_t *partition,const char *src,const int max_size);
 void set_part_name_chomp(partition_t *partition, const unsigned char *src, const int max_size);
 char* strip_dup(char* str);
+int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
+void set_secwest(void);
+time_t td_ntfs2utc (int64_t ntfstime);
 #ifndef BSD_MAXPARTITIONS
 #define	BSD_MAXPARTITIONS	8
 #endif
@@ -486,6 +491,9 @@ int vsnprintf(char *str, size_t size, const char *format, va_list ap);
 #ifndef HAVE_STRNCASECMP
 int strncasecmp(const char * s1, const char * s2, size_t len);
 #endif
+#ifndef HAVE_STRCASESTR
+char * strcasestr (const char *haystack, const char *needle);
+#endif
 /*
  * td_min()/td_max() macros that also do
  * strict type-checking.. See the
