@@ -99,6 +99,12 @@ void xml_set_command_line(const int argc, char **argv)
   }
 }
 
+void xml_clear_command_line(void)
+{
+  free(command_line);
+  command_line=NULL;
+}
+
 void xml_close()
 {
   if(xml_handle==NULL)
@@ -231,10 +237,8 @@ void xml_add_DFXML_creator(const char *package, const char *version)
 #endif
   {
     char outstr[200];
-    time_t t;
-    struct tm *tmp;
-    t = time(NULL);
-    tmp = localtime(&t);
+    const time_t t = time(NULL);
+    const struct tm *tmp = localtime(&t);
     if (tmp != NULL &&
 	strftime(outstr, sizeof(outstr), "%Y-%m-%dT%H:%M:%S%z", tmp) != 0)
     {
@@ -332,7 +336,7 @@ static void xml_log_file_recovered2_aux(const alloc_data_t *space, const alloc_d
   uint64_t size=0;
   for(tmp=&file->list; tmp!=&space->list; tmp=tmp->next)
   {
-    const alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
+    const alloc_data_t *element=td_list_entry_const(tmp, const alloc_data_t, list);
     if(size >= file_size)
       return ;
     if(element->data>0)
