@@ -33,11 +33,27 @@ extern "C" {
 #define FLAG_LIST_MASK12	2
 #define FLAG_LIST_MASK16	4
 #define FLAG_LIST_PATHNAME	8
+#define FLAG_LIST_ADS		16
 /* capabilities */
 #define CAPA_LIST_DELETED	1
+#define CAPA_LIST_ADS		2
 
+typedef enum { DIR_PART_ENOSYS=-2, DIR_PART_EIO=-1, DIR_PART_OK=0} dir_partition_t;
 typedef struct dir_data dir_data_t;
-typedef struct file_data file_data_t;
+typedef struct 
+{
+  struct td_list_head list;
+  char *name;
+  uint32_t st_ino;
+  uint32_t st_mode;
+  uint32_t st_uid;
+  uint32_t st_gid;
+  uint64_t st_size;
+  time_t    td_atime;   /* time of last access */
+  time_t    td_mtime;   /* time of last modification */
+  time_t    td_ctime;   /* time of last status change */
+  unsigned int status;
+} file_info_t;
 
 struct dir_data
 {
@@ -47,8 +63,8 @@ struct dir_data
   int verbose;
   unsigned int param;
   unsigned int capabilities;
-  file_data_t *(*get_dir)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_inode);
-  int (*copy_file)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
+  int(*get_dir)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_inode, file_info_t*list);
+  int (*copy_file)(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
   void (*close)(dir_data_t *dir_data);
   char *local_dir;
   void *private_dir_data;
@@ -56,35 +72,11 @@ struct dir_data
 
 #define	FILE_STATUS_DELETED	1
 #define	FILE_STATUS_MARKED	2
+#define	FILE_STATUS_ADS		4
 
-/* TODO: add status to file_info and migrate file_data to file_info */
-struct file_data
-{
-  file_data_t *prev;
-  file_data_t *next;
-  char	name[DIR_NAME_LEN];
-  struct stat stat;
-#ifdef DJGPP
-  uint64_t file_size;
-#endif
-  unsigned int status;
-};
-
-typedef struct 
-{
-  struct td_list_head list;
-  char *name;
-  struct stat stat;
-#ifdef DJGPP
-  uint64_t file_size;
-#endif
-  unsigned int status;
-} file_info_t;
-
-int dir_aff_log(const dir_data_t *dir_data, const file_data_t*dir_list);
+int dir_aff_log(const dir_data_t *dir_data, const file_info_t*dir_list);
 int log_list_file(const disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list);
-unsigned int delete_list_file(file_data_t *file_list);
-void delete_list_file_info(struct td_list_head *list);
+unsigned int delete_list_file(file_info_t *list);
 int dir_whole_partition_log(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode);
 void mode_string (const unsigned int mode, char *str);
 int set_mode(const char *pathname, unsigned int mode);
