@@ -47,13 +47,51 @@
 
 extern const char *monstr[];
 
-static int dir_partition_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, const unsigned int depth, char **current_cmd);
-static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, file_data_t*dir_list, const unsigned long int inode, const unsigned int depth);
-static int copy_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_data_t *dir);
+static void copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad);
 
 #define INTER_DIR (LINES-25+15)
 
-static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, file_data_t*dir_list, const unsigned long int inode, const unsigned int depth)
+static void copy_progress(WINDOW *window, const unsigned int copy_ok, const unsigned int copy_bad)
+{
+  static time_t prev_time=0;
+  const time_t tmp=time(NULL);
+  if(tmp!=prev_time)
+  {
+    prev_time=tmp;
+    wmove(window,5,0);
+    wclrtoeol(window);
+    if(has_colors())
+    {
+      if(copy_bad > 0)
+	wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
+      else
+	wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
+    }
+    wprintw(window,"Copying, please wait... %u ok, %u failed", copy_ok, copy_bad);
+    if(has_colors())
+      wbkgdset(window,' ' | COLOR_PAIR(0));
+    wrefresh(window);
+  }
+}
+
+static void copy_done(WINDOW *window, const unsigned int copy_ok, const unsigned int copy_bad)
+{
+  wmove(window,5,0);
+  wclrtoeol(window);
+  if(has_colors())
+  {
+    if(copy_bad > 0)
+      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
+    else
+      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
+  }
+  wprintw(window,"Copy done! %u ok, %u failed", copy_ok, copy_bad);
+  if(has_colors())
+    wbkgdset(window,' ' | COLOR_PAIR(0));
+  wrefresh(window);
+}
+
+static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, file_info_t*dir_list, const unsigned long int inode, const unsigned int depth)
 {
   /* Return value
    * -1: quit
@@ -61,14 +99,15 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
    *  other: new inode
    * */
   int quit=0;
+  int ask_destination=1;
   WINDOW *window=(WINDOW*)dir_data->display;
   do
   {
     int offset=0;
     int pos_num=0;
-    file_data_t *pos=dir_list;
-    int old_LINES=LINES;
-    int status=FILE_STATUS_MARKED;
+    struct td_list_head *pos=dir_list->list.next;
+    const int old_LINES=LINES;
+    unsigned int status=FILE_STATUS_MARKED;
     aff_copy(window);
     wmove(window,3,0);
     aff_part(window, AFF_PART_ORDER|AFF_PART_STATUS, disk, partition);
@@ -78,15 +117,20 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
     {
       int i;
       int car;
-      const file_data_t *current_file;
-      for(i=0,current_file=dir_list;(current_file!=NULL) && (i<offset);current_file=current_file->next,i++);
-      for(i=offset;(current_file!=NULL) &&((i-offset)<INTER_DIR);i++,current_file=current_file->next)
+      struct td_list_head *file_walker = NULL;
+      for(i=0, file_walker=dir_list->list.next;
+	  file_walker!=&dir_list->list && i<offset;
+	  file_walker=file_walker->next,i++);
+      for(i=offset;
+	  file_walker!=&dir_list->list && (i-offset)<INTER_DIR;
+	  file_walker=file_walker->next,i++)
       {
+	const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
 	char str[11];
 	char		datestr[80];
 	wmove(window, 6+i-offset, 0);
 	wclrtoeol(window);	/* before addstr for BSD compatibility */
-	if(current_file==pos)
+	if(&current_file->list==pos)
 	{
 	  wattrset(window, A_REVERSE);
 	  waddstr(window, ">");
@@ -97,10 +141,9 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	  wbkgdset(window,' ' | COLOR_PAIR(1));
 	else if((current_file->status&FILE_STATUS_MARKED)!=0 && has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(2));
-	if(current_file->stat.st_mtime!=0)
+	if(current_file->td_mtime!=0)
 	{
-	  struct tm		*tm_p;
-	  tm_p = localtime(&current_file->stat.st_mtime);
+	  const struct tm *tm_p = localtime(&current_file->td_mtime);
 	  snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
 	      tm_p->tm_mday, monstr[tm_p->tm_mon],
 	      1900 + tm_p->tm_year, tm_p->tm_hour,
@@ -109,20 +152,16 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	} else {
 	  strncpy(datestr, "                 ",sizeof(datestr));
 	}
-	mode_string(current_file->stat.st_mode,str);
+	mode_string(current_file->st_mode, str);
 	wprintw(window, "%s %5u %5u ", 
-	    str, (unsigned int)current_file->stat.st_uid, (unsigned int)current_file->stat.st_gid);
-#ifdef DJGPP
-	wprintw(window, "%9llu", (long long unsigned int)current_file->file_size);
-#else
-	wprintw(window, "%9llu", (long long unsigned int)current_file->stat.st_size);
-#endif
+	    str, (unsigned int)current_file->st_uid, (unsigned int)current_file->st_gid);
+	wprintw(window, "%9llu", (long long unsigned int)current_file->st_size);
 	/* screen may overlap due to long filename */
 	wprintw(window, " %s %s", datestr, current_file->name);
 	if(((current_file->status&FILE_STATUS_DELETED)!=0 ||
 	      (current_file->status&FILE_STATUS_MARKED)!=0) && has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
-	if(current_file==pos)
+	if(&current_file->list==pos)
 	  wattroff(window, A_REVERSE);
       }
       wmove(window, 6-1, 51);
@@ -134,9 +173,9 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
       wclrtoeol(window);
       wmove(window, 6+INTER_DIR, 51);
       wclrtoeol(window);
-      if(current_file!=NULL)
+      if(file_walker->next!=&dir_list->list)
 	wprintw(window, "Next");
-      if(dir_list==NULL)
+      if(td_list_empty(&dir_list->list))
       {
 	wmove(window,6,0);
 	wprintw(window,"No file found, filesystem may be damaged.");
@@ -171,6 +210,19 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	else
 	  waddstr(window," to hide deleted files");
       }
+      else if((dir_data->capabilities&CAPA_LIST_ADS)!=0)
+      {
+	waddstr(window,", ");
+	if(has_colors())
+	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
+	waddstr(window,"h");
+	if(has_colors())
+	  wbkgdset(window,' ' | COLOR_PAIR(0));
+	if((dir_data->param&FLAG_LIST_ADS)==0)
+	  waddstr(window," to unhide Alternate Data Stream");
+	else
+	  waddstr(window," to hide Alternate Data Stream");
+      }
       wmove(window,LINES-2,4);
       if(has_colors())
 	wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
@@ -231,15 +283,22 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	case 'h':
 	  if((dir_data->capabilities&CAPA_LIST_DELETED)!=0)
 	    dir_data->param^=FLAG_LIST_DELETED;
+	  else if((dir_data->capabilities&CAPA_LIST_ADS)!=0)
+	    dir_data->param^=FLAG_LIST_ADS;
 	  return inode;
+	case 0x0c:	/* ctrl+L */
+	  touchwin(stdscr);
+	  touchwin(window);
+	  wrefresh(window);
+	  break;
       }
-      if(dir_list!=NULL)
+      if(!td_list_empty(&dir_list->list))
       {
 	switch(car)
 	{
 	  case KEY_UP:
 	  case '8':
-	    if(pos->prev!=NULL)
+	    if(pos->prev!=&dir_list->list)
 	    {
 	      pos=pos->prev;
 	      pos_num--;
@@ -247,17 +306,21 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	    break;
 	  case KEY_DOWN:
 	  case '2':
-	    if(pos->next!=NULL)
+	    if(pos->next!=&dir_list->list)
 	    {
 	      pos=pos->next;
 	      pos_num++;
 	    }
 	    break;
 	  case ':':
-	    if(!(pos->name[0]=='.' && pos->name[1]=='\0') &&
-		!(pos->name[0]=='.' && pos->name[1]=='.' && pos->name[2]=='\0'))
-	    pos->status^=FILE_STATUS_MARKED;
-	    if(pos->next!=NULL)
+	    {
+	      file_info_t *selected_file;
+	      selected_file=td_list_entry(pos, file_info_t, list);
+	      if(!(selected_file->name[0]=='.' && selected_file->name[1]=='\0') &&
+		  !(selected_file->name[0]=='.' && selected_file->name[1]=='.' && selected_file->name[2]=='\0'))
+		selected_file->status^=FILE_STATUS_MARKED;
+	    }
+	    if(pos->next!=&dir_list->list)
 	    {
 	      pos=pos->next;
 	      pos_num++;
@@ -265,9 +328,10 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	    break;
 	  case 'a':
 	    {
-	      file_data_t *tmp;
-	      for(tmp=dir_list; tmp!=NULL; tmp=tmp->next)
+	      struct td_list_head *tmpw= NULL;
+	      td_list_for_each(tmpw, &dir_list->list)
 	      {
+		file_info_t *tmp=td_list_entry(tmpw, file_info_t, list);
 		if((tmp->name[0]=='.' && tmp->name[1]=='\0') ||
 		    (tmp->name[0]=='.' && tmp->name[1]=='.' && tmp->name[2]=='\0'))
 		{
@@ -294,32 +358,35 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 #ifdef PADENTER
 	  case PADENTER:
 #endif
-	    if((pos!=NULL) && (LINUX_S_ISDIR(pos->stat.st_mode)!=0))
 	    {
-	      unsigned long int new_inode=pos->stat.st_ino;
-	      if((new_inode!=inode) &&(strcmp(pos->name,".")!=0))
+	      file_info_t *tmp=td_list_entry(pos, file_info_t, list);
+	      if(pos!=&dir_list->list && (LINUX_S_ISDIR(tmp->st_mode)!=0))
 	      {
-		if(strcmp(pos->name,"..")==0)
-		  return 1;
-		if(strlen(dir_data->current_directory)+1+strlen(pos->name)+1<=sizeof(dir_data->current_directory))
+		const unsigned long int new_inode=tmp->st_ino;
+		if((new_inode!=inode) &&(strcmp(tmp->name,".")!=0))
 		{
-		  if(strcmp(dir_data->current_directory,"/"))
-		    strcat(dir_data->current_directory,"/");
-		  strcat(dir_data->current_directory,pos->name);
-		  return (long int)new_inode;
+		  if(strcmp(tmp->name,"..")==0)
+		    return 1;
+		  if(strlen(dir_data->current_directory)+1+strlen(tmp->name)+1<=sizeof(dir_data->current_directory))
+		  {
+		    if(strcmp(dir_data->current_directory,"/"))
+		      strcat(dir_data->current_directory,"/");
+		    strcat(dir_data->current_directory,tmp->name);
+		    return (long int)new_inode;
+		  }
 		}
 	      }
 	    }
 	    break;
 	  case KEY_PPAGE:
-	    for(i=0;(i<INTER_DIR-1)&&(pos->prev!=NULL);i++)
+	    for(i=0; i<INTER_DIR-1 && pos->prev!=&dir_list->list; i++)
 	    {
 	      pos=pos->prev;
 	      pos_num--;
 	    }
 	    break;
 	  case KEY_NPAGE:
-	    for(i=0;(i<INTER_DIR-1)&&(pos->next!=NULL);i++)
+	    for(i=0; i<INTER_DIR-1 && pos->next!=&dir_list->list; i++)
 	    {
 	      pos=pos->next;
 	      pos_num++;
@@ -329,60 +396,44 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	    if(dir_data->copy_file!=NULL)
 	    {
 	      const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-	      if(strcmp(pos->name,"..")!=0 &&
-		  current_directory_namelength+1+strlen(pos->name)<sizeof(dir_data->current_directory)-1)
+	      file_info_t *tmp=td_list_entry(pos, file_info_t, list);
+	      if(pos!=&dir_list->list &&
+		  strcmp(tmp->name,"..")!=0 &&
+		  current_directory_namelength+1+strlen(tmp->name)<sizeof(dir_data->current_directory)-1)
 	      {
 		if(strcmp(dir_data->current_directory,"/"))
 		  strcat(dir_data->current_directory,"/");
-		if(strcmp(pos->name,".")!=0)
-		  strcat(dir_data->current_directory,pos->name);
-		if(dir_data->local_dir==NULL)
+		if(strcmp(tmp->name,".")!=0)
+		  strcat(dir_data->current_directory,tmp->name);
+		if(dir_data->local_dir==NULL || ask_destination>0)
 		{
-		  if(LINUX_S_ISDIR(pos->stat.st_mode)!=0)
+		  char *local_dir=dir_data->local_dir;
+		  if(LINUX_S_ISDIR(tmp->st_mode)!=0)
 		    dir_data->local_dir=ask_location("Please select a destination where %s and any files below will be copied.",
-			dir_data->current_directory, NULL);
+			dir_data->current_directory, local_dir);
 		  else
 		    dir_data->local_dir=ask_location("Please select a destination where %s will be copied.",
-			dir_data->current_directory, NULL);
+			dir_data->current_directory, local_dir);
+		  free(local_dir);
+		  ask_destination=0;
 		}
 		if(dir_data->local_dir!=NULL)
 		{
-		  int res=-1;
-		  wmove(window,5,0);
-		  wclrtoeol(window);
-		  if(has_colors())
-		    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
-		  wprintw(window,"Copying, please wait...");
-		  if(has_colors())
-		    wbkgdset(window,' ' | COLOR_PAIR(0));
-		  wrefresh(window);
-		  if(LINUX_S_ISDIR(pos->stat.st_mode)!=0)
+		  unsigned int copy_bad=0;
+		  unsigned int copy_ok=0;
+		  if(LINUX_S_ISDIR(tmp->st_mode)!=0)
 		  {
-		    res=copy_dir(disk, partition, dir_data, pos);
+		    copy_dir(window, disk, partition, dir_data, tmp, &copy_ok, &copy_bad);
 		  }
-		  else if(LINUX_S_ISREG(pos->stat.st_mode)!=0)
+		  else if(LINUX_S_ISREG(tmp->st_mode)!=0)
 		  {
-		    res=dir_data->copy_file(disk, partition, dir_data, pos);
-		  }
-		  wmove(window,5,0);
-		  wclrtoeol(window);
-		  if(res < -1)
-		  {
-		    if(has_colors())
-		      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
-		    wprintw(window,"Copy failed!");
-		  }
-		  else
-		  {
-		    if(has_colors())
-		      wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
-		    if(res < 0)
-		      wprintw(window,"Copy done! (Failed to copy some files)");
+		    copy_progress(window, copy_ok, copy_bad);
+		    if(dir_data->copy_file(disk, partition, dir_data, tmp)==0)
+		      copy_ok++;
 		    else
-		      wprintw(window,"Copy done!");
+		      copy_bad++;
 		  }
-		  if(has_colors())
-		    wbkgdset(window,' ' | COLOR_PAIR(0));
+		  copy_done(window, copy_ok, copy_bad);
 		}
 		dir_data->current_directory[current_directory_namelength]='\0';
 	      }
@@ -391,80 +442,74 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	  case 'C':
 	    if(dir_data->copy_file!=NULL)
 	    {
-	      if(dir_data->local_dir==NULL)
+	      if(dir_data->local_dir==NULL || ask_destination>0)
 	      {
-		dir_data->local_dir=ask_location("Please select a destination where the marked files will be copied.", NULL, NULL);
+		char *local_dir=dir_data->local_dir;
+		dir_data->local_dir=ask_location("Please select a destination where the marked files will be copied.", NULL, local_dir);
+		free(local_dir);
+		ask_destination=0;
 	      }
 	      if(dir_data->local_dir!=NULL)
 	      {
-		file_data_t *tmp;
-		int copy_bad=0;
-		int copy_ok=0;
+		unsigned int copy_bad=0;
+		unsigned int copy_ok=0;
 		const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-		wmove(window,5,0);
-		wclrtoeol(window);
-		if(has_colors())
-		  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
-		wprintw(window,"Copying, please wait...");
-		if(has_colors())
-		  wbkgdset(window,' ' | COLOR_PAIR(0));
-		wrefresh(window);
-		for(tmp=dir_list; tmp!=NULL; tmp=tmp->next)
+		struct td_list_head *tmpw= NULL;
+		td_list_for_each(tmpw, &dir_list->list)
 		{
+		  file_info_t *tmp=td_list_entry(tmpw, file_info_t, list);
 		  if((tmp->status&FILE_STATUS_MARKED)!=0 &&
 		      current_directory_namelength + 1 + strlen(tmp->name) <
 		      sizeof(dir_data->current_directory)-1)
 		  {
+		    tmp->status&=~FILE_STATUS_MARKED;
 		    if(strcmp(dir_data->current_directory,"/"))
 		      strcat(dir_data->current_directory,"/");
 		    if(strcmp(tmp->name,".")!=0)
 		      strcat(dir_data->current_directory,tmp->name);
-		    if(LINUX_S_ISDIR(tmp->stat.st_mode)!=0)
+		    if(LINUX_S_ISDIR(tmp->st_mode)!=0)
 		    {
-		      const int res=copy_dir(disk, partition, dir_data, tmp);
-		      if(res >=-1)
-		      {
-			tmp->status&=~FILE_STATUS_MARKED;
-			copy_ok=1;
-		      }
-		      else if(res < 0)
-			copy_bad=1;
+		      copy_dir(window, disk, partition, dir_data, tmp, &copy_ok, &copy_bad);
 		    }
-		    else if(LINUX_S_ISREG(tmp->stat.st_mode)!=0)
+		    else if(LINUX_S_ISREG(tmp->st_mode)!=0)
 		    {
+		      copy_progress(window, copy_ok, copy_bad);
 		      if(dir_data->copy_file(disk, partition, dir_data, tmp) == 0)
-		      {
-			tmp->status&=~FILE_STATUS_MARKED;
-			copy_ok=1;
-		      }
+			copy_ok++;
 		      else
-			copy_bad=1;
+			copy_bad++;
 		    }
 		  }
 		  dir_data->current_directory[current_directory_namelength]='\0';
 		}
-		wmove(window,5,0);
-		wclrtoeol(window);
-		if(copy_bad > 0 && copy_ok==0)
-		{
-		  if(has_colors())
-		    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(1));
-		  wprintw(window,"Copy failed!");
-		}
-		else
-		{
-		  if(has_colors())
-		    wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
-		  if(copy_bad > 0)
-		    wprintw(window,"Copy done! (Failed to copy some files)");
-		  else
-		    wprintw(window,"Copy done!");
-		}
-		if(has_colors())
-		  wbkgdset(window,' ' | COLOR_PAIR(0));
+		copy_done(window, copy_ok, copy_bad);
 	      }
 	    }
 	    break;	
+	  case 'f':
+	    {
+	      const char *needle=ask_string_ncurses("Filename to find ? ");
+	      if(needle!=NULL && needle[0]!='\0')
+	      {
+		file_info_t *tmp;
+		struct td_list_head *pos_org=pos;
+		const int pos_num_org=pos_num;
+		tmp=td_list_entry(pos, file_info_t, list);
+		while(pos->next!=&dir_list->list &&
+		  strcmp(tmp->name, needle)!=0)
+		{
+		  pos=pos->next;
+		  tmp=td_list_entry(pos, file_info_t, list);
+		  pos_num++;
+		}
+		if(pos==&dir_list->list)
+		{
+		  pos=pos_org;
+		  pos_num=pos_num_org;
+		}
+	      }
+	    }
+	    break;
 	}
 	if(pos_num<offset)
 	  offset=pos_num;
@@ -491,22 +536,25 @@ static int dir_partition_aux(disk_t *disk, const partition_t *partition, dir_dat
   {
     const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
     long int new_inode=-1;	/* Quit */
-    file_data_t *dir_list;
+    file_info_t dir_list = {
+      .list = TD_LIST_HEAD_INIT(dir_list.list),
+      .name = NULL
+    };
     /* Not perfect for FAT32 root cluster */
     inode_known[depth]=inode;
-    dir_list=dir_data->get_dir(disk, partition, dir_data, inode);
-    dir_aff_log(dir_data, dir_list);
+    dir_data->get_dir(disk, partition, dir_data, inode, &dir_list);
+    dir_aff_log(dir_data, &dir_list);
     if(*current_cmd!=NULL)
     {
       /* TODO: handle copy_files */
       dir_data->current_directory[current_directory_namelength]='\0';
-      delete_list_file(dir_list);
+      delete_list_file(&dir_list);
       return -1;	/* Quit */
     }
-    new_inode=dir_aff_ncurses(disk, partition, dir_data,dir_list,inode,depth);
+    new_inode=dir_aff_ncurses(disk, partition, dir_data, &dir_list, inode, depth);
     if(new_inode==-1 || new_inode==1) /* -1:Quit or 1:Back */
     {
-      delete_list_file(dir_list);
+      delete_list_file(&dir_list);
       return new_inode;
     }
     if(new_inode>=2)
@@ -523,7 +571,7 @@ static int dir_partition_aux(disk_t *disk, const partition_t *partition, dir_dat
     }
     /* restore current_directory name */
     dir_data->current_directory[current_directory_namelength]='\0';
-    delete_list_file(dir_list);
+    delete_list_file(&dir_list);
   }
 }
 
@@ -541,32 +589,35 @@ Returns
 0: all files has been copied
 */
 #define MAX_DIR_NBR 256
-static int copy_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_data_t *dir)
+static void copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad)
 {
   static unsigned int dir_nbr=0;
   static unsigned long int inode_known[MAX_DIR_NBR];
-  file_data_t *dir_list;
+  file_info_t dir_list = {
+    .list = TD_LIST_HEAD_INIT(dir_list.list),
+    .name = NULL
+  };
   const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-  file_data_t *current_file;
   char *dir_name;
-  int copy_bad=0;
-  int copy_ok=0;
+  struct td_list_head *file_walker = NULL;
   if(dir_data->get_dir==NULL || dir_data->copy_file==NULL)
-    return -2;
-  inode_known[dir_nbr++]=dir->stat.st_ino;
+    return;
+  inode_known[dir_nbr++]=dir->st_ino;
   dir_name=mkdir_local(dir_data->local_dir, dir_data->current_directory);
-  dir_list=dir_data->get_dir(disk, partition, dir_data, (const unsigned long int)dir->stat.st_ino);
-  for(current_file=dir_list;current_file!=NULL;current_file=current_file->next)
+  dir_data->get_dir(disk, partition, dir_data, (const unsigned long int)dir->st_ino, &dir_list);
+  td_list_for_each(file_walker, &dir_list.list)
   {
+    const file_info_t *current_file;
+    current_file=td_list_entry(file_walker, file_info_t, list);
     dir_data->current_directory[current_directory_namelength]='\0';
     if(current_directory_namelength+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
     {
       if(strcmp(dir_data->current_directory,"/"))
 	strcat(dir_data->current_directory,"/");
       strcat(dir_data->current_directory,current_file->name);
-      if(LINUX_S_ISDIR(current_file->stat.st_mode)!=0)
+      if(LINUX_S_ISDIR(current_file->st_mode)!=0)
       {
-	const unsigned long int new_inode=current_file->stat.st_ino;
+	const unsigned long int new_inode=current_file->st_ino;
 	unsigned int new_inode_ok=1;
 	unsigned int i;
 	if(new_inode<2)
@@ -578,32 +629,24 @@ static int copy_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_
 	    new_inode_ok=0;
 	if(new_inode_ok>0)
 	{
-	  int tmp;
-	  tmp=copy_dir(disk, partition, dir_data, current_file);
-	  if(tmp>=-1)
-	    copy_ok=1;
-	  if(tmp<0)
-	    copy_bad=1;
+	  copy_dir(window, disk, partition, dir_data, current_file, copy_ok, copy_bad);
 	}
       }
-      else if(LINUX_S_ISREG(current_file->stat.st_mode)!=0)
+      else if(LINUX_S_ISREG(current_file->st_mode)!=0)
       {
-//	log_trace("copy_file %s\n",dir_data->current_directory);
-	int tmp;
-	tmp=dir_data->copy_file(disk, partition, dir_data, current_file);
-	if(tmp==0)
-	  copy_ok=1;
+	copy_progress(window, *copy_ok, *copy_bad);
+	if(dir_data->copy_file(disk, partition, dir_data, current_file)==0)
+	  (*copy_ok)++;
 	else
-	  copy_bad=1;
+	  (*copy_bad)++;
       }
     }
   }
   dir_data->current_directory[current_directory_namelength]='\0';
-  delete_list_file(dir_list);
-  set_date(dir_name, dir->stat.st_atime, dir->stat.st_mtime);
+  delete_list_file(&dir_list);
+  set_date(dir_name, dir->td_atime, dir->td_mtime);
   free(dir_name);
   dir_nbr--;
-  return (copy_bad>0?(copy_ok>0?-1:-2):0);
 }
 
 #endif
