@@ -72,12 +72,14 @@
 #include "log.h"
 #include "hdaccess.h"
 
+extern const arch_fnct_t arch_none;
+
 static const char *fewf_description(disk_t *disk);
 static const char *fewf_description_short(disk_t *disk);
-static int fewf_clean(disk_t *disk);
-static void *fewf_pread_fast(disk_t *disk, void *buffer, const unsigned int count, const uint64_t offset);
+static void fewf_clean(disk_t *disk);
 static int fewf_pread(disk_t *disk, void *buffer, const unsigned int count, const uint64_t offset);
 static int fewf_nopwrite(disk_t *disk, const void *buffer, const unsigned int count, const uint64_t offset);
+static int fewf_pwrite(disk_t *disk, const void *buffer, const unsigned int count, const uint64_t offset);
 static int fewf_sync(disk_t *disk);
 
 struct info_fewf_struct
@@ -94,7 +96,7 @@ struct info_fewf_struct
   unsigned int buffer_size;
 };
 
-disk_t *fewf_init(const char *device, const arch_fnct_t *arch, const int mode)
+disk_t *fewf_init(const char *device, const int mode)
 {
   unsigned int num_files=0;
   char **filenames= NULL;
@@ -106,13 +108,19 @@ disk_t *fewf_init(const char *device, const arch_fnct_t *arch, const int mode)
   data=(struct info_fewf_struct *)MALLOC(sizeof(struct info_fewf_struct));
   memset(data, 0, sizeof(struct info_fewf_struct)); 
   data->file_name = strdup(device);
+  data->handle=NULL;
   data->mode = mode;
 
-#if defined( HAVE_LIBEWF_V2_API )
 #ifdef DEBUG_EWF
+#if defined( HAVE_LIBEWF_V2_API )
   libewf_notify_set_stream( stderr, NULL );
   libewf_notify_set_verbose( 1 );
+#else
+  libewf_set_notify_values( stderr, 1 );
 #endif
+#endif
+
+#if defined( HAVE_LIBEWF_V2_API )
   if( libewf_glob(
        data->file_name,
        strlen(data->file_name),
@@ -125,45 +133,7 @@ disk_t *fewf_init(const char *device, const arch_fnct_t *arch, const int mode)
       free(data);
       return NULL;
   }
-  if( libewf_handle_initialize(
-	&( data->handle ),
-	NULL ) != 1 )
-  {
-    log_error("libewf_handle_initialize failed\n");
-
-      libewf_glob_free(
-       filenames,
-       num_files,
-       NULL );
-    free(data);
-    return NULL;
-  }
-  if( libewf_handle_open(
-	data->handle,
-	filenames,
-	num_files,
-	LIBEWF_OPEN_READ,
-	NULL ) != 1 )
-  {
-    log_error("libewf_handle_open(%s) failed\n", device);
-
-    libewf_handle_free(
-	&( data->handle ),
-	NULL );
-
-      libewf_glob_free(
-       filenames,
-       num_files,
-       NULL );
-    free(data);
-    return NULL;
-  }
-
-#else
-#ifdef DEBUG_EWF
-  libewf_set_notify_values( stderr, 1 );
-#endif
-#if defined( HAVE_GLOB_H )
+#elif defined( HAVE_GLOB_H )
   {
     globbuf.gl_offs = 0;
     glob(data->file_name, GLOB_DOOFFS, NULL, &globbuf);
@@ -188,18 +158,91 @@ disk_t *fewf_init(const char *device, const arch_fnct_t *arch, const int mode)
     num_files++;
   }
 #endif
-  data->handle=libewf_open(filenames, num_files, LIBEWF_OPEN_READ);
+
+  if((mode&TESTDISK_O_RDWR)==TESTDISK_O_RDWR)
+  {
+#if defined( HAVE_LIBEWF_V2_API )
+    if( libewf_handle_initialize(
+	  &( data->handle ),
+	  NULL ) != 1 )
+    {
+      log_error("libewf_handle_initialize failed\n");
+
+      libewf_glob_free(
+	  filenames,
+	  num_files,
+	  NULL );
+      free(data);
+      return NULL;
+    }
+    if( libewf_handle_open(
+	  data->handle,
+	  filenames,
+	  num_files,
+	  LIBEWF_OPEN_READ_WRITE,
+	  NULL ) != 1 )
+    {
+      log_error("libewf_handle_open(%s) failed\n", device);
+    }
+#else
+    data->handle=libewf_open(filenames, num_files, LIBEWF_OPEN_READ_WRITE);
+    if(data->handle==NULL)
+    {
+      log_error("libewf_open(%s) failed\n", device);
+    }
+#endif /* defined( HAVE_LIBEWF_V2_API ) */
+  }
   if(data->handle==NULL)
   {
-    log_error("libewf_open(%s) failed\n", device);
+    data->mode&=~TESTDISK_O_RDWR;
+#if defined( HAVE_LIBEWF_V2_API )
+    if( libewf_handle_initialize(
+	  &( data->handle ),
+	  NULL ) != 1 )
+    {
+      log_error("libewf_handle_initialize failed\n");
+
+      libewf_glob_free(
+	  filenames,
+	  num_files,
+	  NULL );
+      free(data);
+      return NULL;
+    }
+    if( libewf_handle_open(
+	  data->handle,
+	  filenames,
+	  num_files,
+	  LIBEWF_OPEN_READ,
+	  NULL ) != 1 )
+    {
+      log_error("libewf_handle_open(%s) failed\n", device);
+
+      libewf_handle_free(
+	  &( data->handle ),
+	  NULL );
+
+      libewf_glob_free(
+	  filenames,
+	  num_files,
+	  NULL );
+      free(data);
+      return NULL;
+    }
+#else
+    data->handle=libewf_open(filenames, num_files, LIBEWF_OPEN_READ);
+    if(data->handle==NULL)
+    {
+      log_error("libewf_open(%s) failed\n", device);
 #if defined( HAVE_GLOB_H )
-    globfree(&globbuf);
+      globfree(&globbuf);
 #endif
-    free(filenames);
-    free(data);
-    return NULL;
-  }
+      free(filenames);
+      free(data);
+      return NULL;
+    }
 #endif /* defined( HAVE_LIBEWF_V2_API ) */
+  }
 
 #if defined( HAVE_LIBEWF_V2_API )
   if( libewf_handle_set_header_values_date_format(
@@ -217,16 +260,15 @@ disk_t *fewf_init(const char *device, const arch_fnct_t *arch, const int mode)
 #endif
   disk=(disk_t *)MALLOC(sizeof(*disk));
   init_disk(disk);
-  disk->arch=arch;
+  disk->arch=&arch_none;
   disk->device=strdup(device);
   disk->data=data;
   disk->description=fewf_description;
   disk->description_short=fewf_description_short;
-  disk->pread_fast=fewf_pread_fast;
   disk->pread=fewf_pread;
-  disk->pwrite=fewf_nopwrite;
+  disk->pwrite=(data->mode&TESTDISK_O_RDWR?fewf_pwrite:fewf_nopwrite);
   disk->sync=fewf_sync;
-  disk->access_mode=TESTDISK_O_RDONLY;
+  disk->access_mode=(data->mode&TESTDISK_O_RDWR);
   disk->clean=fewf_clean;
 #if defined( HAVE_LIBEWF_V2_API ) || defined( LIBEWF_GET_BYTES_PER_SECTOR_HAVE_TWO_ARGUMENTS )
   {
@@ -322,7 +364,7 @@ static const char *fewf_description_short(disk_t *disk)
   return disk->description_short_txt;
 }
 
-static int fewf_clean(disk_t *disk)
+static void fewf_clean(disk_t *disk)
 {
   if(disk->data!=NULL)
   {
@@ -346,7 +388,7 @@ static int fewf_clean(disk_t *disk)
     free(disk->data);
     disk->data=NULL;
   }
-  return 0;
+  generic_clean(disk);
 }
 
 static int fewf_sync(disk_t *disk)
@@ -386,11 +428,29 @@ static int fewf_pread(disk_t *disk, void *buffer, const unsigned int count, cons
   return taille;
 }
 
-static void *fewf_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
+static int fewf_pwrite(disk_t *disk, const void *buffer, const unsigned int count, const uint64_t offset)
 {
-  if(fewf_pread(disk, buf, count, offset)==offset)
-    return buf;
-  return NULL;
+  struct info_fewf_struct *data=(struct info_fewf_struct *)disk->data;
+  int64_t taille;
+#if defined( HAVE_LIBEWF_V2_API )
+  taille = libewf_handle_write_random(
+            data->handle,
+            buffer,
+            count,
+            offset,
+            NULL );
+#else
+  taille=libewf_write_random(data->handle, buffer, count, offset);
+#endif
+  if(taille!=count)
+  {
+    log_error("fewf_pwrite(xxx,%u,buffer,%lu(%u/%u/%u)) write err: ",
+	(unsigned)(count/disk->sector_size), (long unsigned)(offset/disk->sector_size),
+	offset2cylinder(disk,offset), offset2head(disk,offset), offset2sector(disk,offset));
+    log_error("%s\n", strerror(errno));
+    return -1;
+  }
+  return taille;
 }
 
 static int fewf_nopwrite(disk_t *disk, const void *buffer, const unsigned int count, const uint64_t offset)
