@@ -61,14 +61,11 @@ struct exfat_dir_struct
 };
 
 
-static int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
-static file_data_t *exfat_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster);
-static inline void exfat16_towchar(wchar_t *dst, const uint8_t *src, size_t len);
-static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
+static int exfat_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list);
+static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
 static void dir_partition_exfat_close(dir_data_t *dir_data);
 
-static int32_t secwest;
-
+#if 0
 static inline void exfat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 {
 	while (len--) {
@@ -76,6 +73,7 @@ static inline void exfat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 		src += 2;
 	}
 }
+#endif
 
 #define ATTR_RO      1  /* read-only */
 #define ATTR_HIDDEN  2  /* hidden */
@@ -84,7 +82,7 @@ static inline void exfat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 #define ATTR_ARCH    32 /* archived */
 #define EXFAT_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
 
-static file_data_t *dir_exfat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int param)
+static int dir_exfat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int param, file_info_t *dir_list)
 {
   /*
    * 0x83 Volume label
@@ -98,8 +96,7 @@ static file_data_t *dir_exfat_aux(const unsigned char*buffer, const unsigned int
    * 0xE2 Windows CE ACL
    *
    */
-  file_data_t *dir_list=NULL;
-  file_data_t *current_file=NULL;
+  file_info_t *current_file=NULL;
   unsigned int offset=0;
   unsigned int sec_count=0;
   for(offset=0; offset<size; offset+=0x20)
@@ -110,41 +107,21 @@ static file_data_t *dir_exfat_aux(const unsigned char*buffer, const unsigned int
     if((buffer[offset]&0x7f)==0x05)
     { /* File directory entry */
       const struct exfat_file_entry *entry=(const struct exfat_file_entry *)&buffer[offset];
-      file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
+      file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
       sec_count=entry->sec_count;
+      new_file->name=(char *)MALLOC(512);
       new_file->name[0]=0;
-      new_file->stat.st_dev=0;
-      new_file->stat.st_ino=0;
-      new_file->stat.st_mode = EXFAT_MKMODE(entry->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
-      new_file->stat.st_nlink=0;
-      new_file->stat.st_uid=0;
-      new_file->stat.st_gid=0;
-      new_file->stat.st_rdev=0;
-      new_file->stat.st_size=0;
-#ifdef DJGPP
-      new_file->file_size=0;
-#endif
-#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-      new_file->stat.st_blksize=0;
-#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-      if(new_file->stat.st_blksize!=0)
-      {
-	new_file->stat.st_blocks=0;
-      }
-#endif
-#endif
-      new_file->stat.st_atime=date_dos2unix(le16(entry->atime),le16(entry->adate));
-      new_file->stat.st_ctime=date_dos2unix(le16(entry->ctime),le16(entry->cdate));
-      new_file->stat.st_mtime=date_dos2unix(le16(entry->mtime),le16(entry->mdate));
+      new_file->st_ino=0;
+      new_file->st_mode = EXFAT_MKMODE(entry->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
+      new_file->st_uid=0;
+      new_file->st_gid=0;
+      new_file->st_size=0;
+      new_file->td_atime=date_dos2unix(le16(entry->atime),le16(entry->adate));
+      new_file->td_ctime=date_dos2unix(le16(entry->ctime),le16(entry->cdate));
+      new_file->td_mtime=date_dos2unix(le16(entry->mtime),le16(entry->mdate));
       new_file->status=((entry->type&0x80)==0x80?0:FILE_STATUS_DELETED);
-      new_file->prev=current_file;
-      new_file->next=NULL;
-      /* log_debug("exfat: new file %s de=%p size=%u\n",new_file->name,de,le32(de->size)); */
-      if(current_file!=NULL)
-        current_file->next=new_file;
-      else
-        dir_list=new_file;
       current_file=new_file;
+      td_list_add_tail(&new_file->list, &dir_list->list);
     }
     else if(sec_count>0 && current_file!=NULL)
     {
@@ -152,14 +129,11 @@ static file_data_t *dir_exfat_aux(const unsigned char*buffer, const unsigned int
       {
 	/* Stream extension */
 	const struct exfat_stream_ext_entry *entry=(const struct exfat_stream_ext_entry*)&buffer[offset];
-	current_file->stat.st_size=le64(entry->data_length);
-#ifdef DJGPP
-	current_file->file_size=le64(entry->data_length);
-#endif
-	current_file->stat.st_ino=le32(entry->first_cluster);
+	current_file->st_size=le64(entry->data_length);
+	current_file->st_ino=le32(entry->first_cluster);
 #if 0
 	if((entry->first_cluster&2)!=0)
-	  current_file->stat.st_size=0;
+	  current_file->st_size=0;
 #endif
       }
       else if((buffer[offset]&0x7f)==0x41)
@@ -174,30 +148,10 @@ static file_data_t *dir_exfat_aux(const unsigned char*buffer, const unsigned int
       sec_count--;
     }
   }
-  return dir_list;
+  return 0;
 }
 
-static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
-		  /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */
-
-/* Convert a MS-DOS time/date pair to a UNIX date (seconds since 1 1 70). */
-
-static int date_dos2unix(const unsigned short f_time, const unsigned short f_date)
-{
-	int month,year,secs;
-
-	/* first subtract and mask after that... Otherwise, if
-	   f_date == 0, bad things happen */
-	month = ((f_date >> 5) - 1) & 15;
-	year = f_date >> 9;
-	secs = (f_time & 31)*2+60*((f_time >> 5) & 63)+(f_time >> 11)*3600+86400*
-	    ((f_date & 31)-1+day_n[month]+(year/4)+year*365-((year & 3) == 0 &&
-	    month < 2 ? 1 : 0)+3653);
-			/* days since 1.1.70 plus 80's leap day */
-	return secs+secwest;
-}
-
-enum {exFAT_FOLLOW_CLUSTER, exFAT_NEXT_FREE_CLUSTER, exFAT_NEXT_CLUSTER};
+typedef enum {exFAT_FOLLOW_CLUSTER, exFAT_NEXT_FREE_CLUSTER, exFAT_NEXT_CLUSTER} exfat_method_t;
 
 static int is_EOC(const unsigned int cluster)
 {
@@ -205,25 +159,24 @@ static int is_EOC(const unsigned int cluster)
 }
 
 #define NBR_CLUSTER_MAX 30
-static file_data_t *exfat_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster)
+static int exfat_dir(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list)
 {
   const struct exfat_dir_struct *ls=(const struct exfat_dir_struct*)dir_data->private_dir_data;
   const struct exfat_super_block*exfat_header=ls->boot_sector;
   const unsigned int cluster_shift=exfat_header->block_per_clus_bits + exfat_header->blocksize_bits;
-  file_data_t *dir_list=NULL;
   unsigned int cluster;
   unsigned char *buffer_dir=(unsigned char *)MALLOC(NBR_CLUSTER_MAX << cluster_shift);
   unsigned int nbr_cluster;
-  unsigned int clus_blocknr;
+//  unsigned int clus_blocknr;
   unsigned int total_clusters;
-  unsigned int exfat_meth=exFAT_FOLLOW_CLUSTER;
+  exfat_method_t exfat_meth=exFAT_FOLLOW_CLUSTER;
   int stop=0;
   if(first_cluster<2)
     cluster=le32(exfat_header->rootdir_clusnr);
   else
     cluster=first_cluster;
   memset(buffer_dir, 0, NBR_CLUSTER_MAX<<cluster_shift);
-  clus_blocknr=le32(exfat_header->clus_blocknr);
+//  clus_blocknr=le32(exfat_header->clus_blocknr);
   total_clusters=le32(exfat_header->total_clusters);
   nbr_cluster=0;
   while(!is_EOC(cluster) && cluster>=2 && nbr_cluster<NBR_CLUSTER_MAX && stop==0)
@@ -270,35 +223,12 @@ static file_data_t *exfat_dir(disk_t *disk, const partition_t *partition, dir_da
     }
   }
   if(nbr_cluster>0)
-    dir_list=dir_exfat_aux(buffer_dir, nbr_cluster<<cluster_shift, dir_data->param);
+    dir_exfat_aux(buffer_dir, nbr_cluster<<cluster_shift, dir_data->param, dir_list);
   free(buffer_dir);
-  return dir_list;
+  return 0;
 }
 
-static void set_secwest(void)
-{
-  struct  tm *tmptr;
-  time_t t;
-
-  t = time(NULL);
-  tmptr = localtime(&t);
-#ifdef HAVE_STRUCT_TM_TM_GMTOFF
-  secwest = -1 * tmptr->tm_gmtoff;
-#elif defined (DJGPP)
-  secwest = 0;
-#else
-#if defined (__CYGWIN__)
-  secwest = _timezone;
-#else
-  secwest = timezone;
-#endif
-  /* account for daylight savings */
-  if (tmptr->tm_isdst)
-    secwest -= 3600;
-#endif
-}
-
-int dir_partition_exfat_init(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const int verbose)
+dir_partition_t dir_partition_exfat_init(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
   static struct exfat_dir_struct *ls;
   struct exfat_super_block *exfat_header;
@@ -309,14 +239,14 @@ int dir_partition_exfat_init(disk_t *disk, const partition_t *partition, dir_dat
   {
     log_error("Can't read exFAT boot sector.\n");
     free(exfat_header);
-    return -1;
+    return DIR_PART_EIO;
   }
   if(le16(exfat_header->signature)!=0xAA55 ||
       memcmp(exfat_header->oem_id, "EXFAT   ", sizeof(exfat_header->oem_id))!=0)
   {
     log_error("Not an exFAT boot sector.\n");
     free(exfat_header);
-    return -1;
+    return DIR_PART_EIO;
   }
   ls=(struct exfat_dir_struct *)MALLOC(sizeof(*ls));
   ls->boot_sector=exfat_header;
@@ -336,7 +266,7 @@ int dir_partition_exfat_init(disk_t *disk, const partition_t *partition, dir_dat
   dir_data->local_dir=NULL;
   dir_data->private_dir_data=ls;
   dir_data->get_dir=exfat_dir;
-  return 0;
+  return DIR_PART_OK;
 }
 
 static void dir_partition_exfat_close(dir_data_t *dir_data)
@@ -350,7 +280,7 @@ static void dir_partition_exfat_close(dir_data_t *dir_data)
   free(ls);
 }
 
-static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
+static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file)
 {
   char *new_file;	
   FILE *f_out;
@@ -359,12 +289,8 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
   const unsigned int cluster_shift=exfat_header->block_per_clus_bits + exfat_header->blocksize_bits;
   unsigned char *buffer_file=(unsigned char *)MALLOC(1<<cluster_shift);
   unsigned int cluster;
-#ifdef DJGPP
-  unsigned int file_size=file->file_size;
-#else
-  unsigned int file_size=file->stat.st_size;
-#endif
-  unsigned int exfat_meth=exFAT_FOLLOW_CLUSTER;
+  uint64_t file_size=file->st_size;
+  exfat_method_t exfat_meth=exFAT_FOLLOW_CLUSTER;
   uint64_t start_exfat1,clus_blocknr;
   unsigned long int total_clusters;
   f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
@@ -375,7 +301,7 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
     free(buffer_file);
     return -1;
   }
-  cluster = file->stat.st_ino;
+  cluster = file->st_ino;
   start_exfat1=le32(exfat_header->fat_blocknr) << exfat_header->blocksize_bits;
   clus_blocknr=le32(exfat_header->clus_blocknr);
   total_clusters=le32(exfat_header->total_clusters);
@@ -397,7 +323,7 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
     {
       log_error("exfat_copy: no space left on destination.\n");
       fclose(f_out);
-      set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
+      set_date(new_file, file->td_atime, file->td_mtime);
       free(new_file);
       free(buffer_file);
       return -1;
@@ -410,7 +336,7 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
 	const unsigned int next_cluster=get_next_cluster(disk, partition, UP_FAT32, start_exfat1, cluster);
 	if(next_cluster>=2 && next_cluster<=total_clusters)
 	  cluster=next_cluster;
-	else if(cluster==file->stat.st_ino && next_cluster==0)
+	else if(cluster==file->st_ino && next_cluster==0)
 	  exfat_meth=exFAT_NEXT_FREE_CLUSTER;	/* Recovery of a deleted file */
 	else
 	  exfat_meth=exFAT_NEXT_CLUSTER;		/* exFAT is corrupted, don't trust it */
@@ -425,7 +351,7 @@ static int exfat_copy(disk_t *disk, const partition_t *partition, dir_data_t *di
     }
   }
   fclose(f_out);
-  set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
+  set_date(new_file, file->td_atime, file->td_mtime);
   free(new_file);
   free(buffer_file);
   return 0;
