@@ -89,7 +89,7 @@ static unsigned long int get_subdirectory(disk_t *disk_car,const uint64_t hd_off
 
 #ifdef HAVE_NCURSES
 static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned short *msdate);
-static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, const unsigned int reserved, const unsigned int fat_length, const unsigned int root_cluster, const unsigned int sectors_per_cluster, const int verbose, file_data_t *rootdir_list, const unsigned int fats);
+static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, const unsigned int reserved, const unsigned int fat_length, const unsigned int root_cluster, const unsigned int sectors_per_cluster, const int verbose, file_info_t *rootdir_list, const unsigned int fats);
 static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsigned int nbr_offset,unsigned int*reserved, unsigned int*fat_length, const unsigned long int max_sector_offset, unsigned int *fats);
 #endif
 
@@ -176,7 +176,7 @@ static unsigned long int get_subdirectory(disk_t *disk_car,const uint64_t hd_off
 #ifdef HAVE_NCURSES
 #define INTER_DIR 16
 
-static int ask_root_directory(disk_t *disk_car, const partition_t *partition, const file_data_t*dir_list, const unsigned long int cluster)
+static int ask_root_directory(disk_t *disk_car, const partition_t *partition, const file_info_t*dir_list, const unsigned long int cluster)
 {
   /* Return value
    * -1: quit
@@ -187,8 +187,7 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
   int quit=0;
   int offset=0;
   int pos_num=0;
-  const file_data_t *current_file;
-  const file_data_t *pos=dir_list;
+  struct td_list_head *pos=dir_list->list.next;
   WINDOW *window;
   window=newwin(LINES, COLS, 0, 0);	/* full screen */
   aff_copy(window);
@@ -200,24 +199,29 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
   do
   {
     int i;
-    for(i=0,current_file=dir_list;(current_file!=NULL) && (i<offset);current_file=current_file->next,i++);
-    for(i=offset;(current_file!=NULL) &&((i-offset)<INTER_DIR);i++,current_file=current_file->next)
+    struct td_list_head *file_walker = NULL;
+    for(i=0, file_walker=dir_list->list.next;
+	file_walker!=&dir_list->list && i<offset;
+	file_walker=file_walker->next,i++);
+    for(i=offset;
+	file_walker!=&dir_list->list && (i-offset)<INTER_DIR;
+	file_walker=file_walker->next,i++)
     {
-      struct tm		*tm_p;
+      const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
       char str[11];
       char		datestr[80];
       wmove(window,8+i-offset,0);
       wclrtoeol(window);	/* before addstr for BSD compatibility */
-      if(current_file==pos)
+      if(&current_file->list==pos)
       {
 	wattrset(window, A_REVERSE);
 	waddstr(window, ">");
       }
       else
 	waddstr(window, " ");
-      if(current_file->stat.st_mtime!=0)
+      if(current_file->td_mtime!=0)
       {
-	tm_p = localtime(&current_file->stat.st_mtime);
+	const struct tm	*tm_p=localtime(&current_file->td_mtime);
 	snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
 	    tm_p->tm_mday, monstr[tm_p->tm_mon],
 	    1900 + tm_p->tm_year, tm_p->tm_hour,
@@ -225,17 +229,13 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
       } else {
 	strncpy(datestr, "                 ",sizeof(datestr));
       }
-      mode_string(current_file->stat.st_mode,str);
+      mode_string(current_file->st_mode, str);
       wprintw(window, "%s %3u %3u ", 
-	  str, (unsigned int)current_file->stat.st_uid, (unsigned int)current_file->stat.st_gid);
-#ifdef DJGPP
-      wprintw(window, "%9llu", (long long unsigned int)current_file->file_size);
-#else
-      wprintw(window, "%9llu", (long long unsigned int)current_file->stat.st_size);
-#endif
+	  str, (unsigned int)current_file->st_uid, (unsigned int)current_file->st_gid);
+      wprintw(window, "%9llu", (long long unsigned int)current_file->st_size);
       /* FIXME: screen overlaps due to long filename */
       wprintw(window, " %s %s\n", datestr, current_file->name);
-      if(current_file==pos)
+      if(&current_file->list==pos)
 	wattroff(window, A_REVERSE);
     }
     /* Clear the last line, useful if overlapping */
@@ -260,33 +260,33 @@ static int ask_root_directory(disk_t *disk_car, const partition_t *partition, co
 	quit=1;
 	break;
     }
-    if(dir_list!=NULL)
+    if(!td_list_empty(&dir_list->list))
     {
       switch(car)
       {
 	case KEY_UP:
-	  if(pos->prev!=NULL)
+	  if(pos->prev!=&dir_list->list)
 	  {
 	    pos=pos->prev;
 	    pos_num--;
 	  }
 	  break;
 	case KEY_DOWN:
-	  if(pos->next!=NULL)
+	  if(pos->next!=&dir_list->list)
 	  {
 	    pos=pos->next;
 	    pos_num++;
 	  }
 	  break;
 	case KEY_PPAGE:
-	  for(i=0; i<INTER_DIR-1 && pos->prev!=NULL; i++)
+	  for(i=0; i<INTER_DIR-1 && pos->prev!=&dir_list->list; i++)
 	  {
 	    pos=pos->prev;
 	    pos_num--;
 	  }
 	  break;
 	case KEY_NPAGE:
-	  for(i=0; i<INTER_DIR-1 && pos->next!=NULL; i++)
+	  for(i=0; i<INTER_DIR-1 && pos->next!=&dir_list->list; i++)
 	  {
 	    pos=pos->next;
 	    pos_num++;
@@ -320,14 +320,16 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
   if(sectors_per_cluster==0)
     return 0;
   {
-    file_data_t *rootdir_list=NULL;
-    file_data_t *current_file=NULL;
     unsigned int dir_nbr=0;
 #ifdef HAVE_NCURSES
     int interactive=1;
 #endif
     unsigned char *buffer;
     int ind_stop=0;
+    file_info_t rootdir_list= {
+      .list = TD_LIST_HEAD_INIT(rootdir_list.list),
+      .name = NULL
+    };
     buffer=(unsigned char *)MALLOC(cluster_size);
 #ifdef HAVE_NCURSES
     if(interface)
@@ -371,27 +373,26 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
               (buffer[1*0x20+0xB]!=ATTR_EXT && (buffer[1*0x20+0xB]&ATTR_DIR)!=0) && (cluster==0)
               && (buffer[0x40]!=0)) /* First-level directory */
           {
-            file_data_t *dir_list;
+	    file_info_t dir_list = {
+	      .list = TD_LIST_HEAD_INIT(dir_list.list),
+	      .name = NULL
+	    };
             log_info("First-level directory found at cluster %lu\n",root_cluster);
             /*    dump_ncurses(buffer,cluster_size); */
-            dir_list=dir_fat_aux(buffer, cluster_size, cluster_size, 0);
+            dir_fat_aux(buffer, cluster_size, cluster_size, 0, &dir_list);
             if(verbose>0)
             {
-              dir_aff_log(NULL, dir_list);
+              dir_aff_log(NULL, &dir_list);
             }
             {
-              file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
-              memcpy(new_file,dir_list,sizeof(*new_file));
-              new_file->prev=current_file;
-              new_file->next=NULL;
-              if(current_file!=NULL)
-                current_file->next=new_file;
-              else
-                rootdir_list=new_file;
-              current_file=new_file;
-              snprintf(new_file->name,sizeof(new_file->name),"DIR%05u",++dir_nbr);
+	      const file_info_t *first_entry=td_list_entry(&dir_list.list, file_info_t, list);
+              file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
+              memcpy(new_file, first_entry, sizeof(*new_file));
+	      new_file->name=(char*)MALLOC(32);
+              snprintf(new_file->name, 32,"DIR%05u",++dir_nbr);
+	      td_list_add_tail(&new_file->list, &rootdir_list.list);
             }
-            delete_list_file(dir_list);
+            delete_list_file(&dir_list);
           }
         }
         else if( memcmp(&buffer[0*0x20],&buffer[1*0x20],0x20)!=0)
@@ -503,30 +504,35 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
               }
             }
             {
-              file_data_t *dir_list;
-              dir_list=dir_fat_aux(buffer, cluster_size, cluster_size, 0);
-              if(dir_list!=NULL && (dir_list->next==NULL || dir_list->stat.st_ino!=dir_list->next->stat.st_ino))
+	      file_info_t dir_list = {
+		.list = TD_LIST_HEAD_INIT(dir_list.list),
+		.name = NULL
+	      };
+	      const file_info_t *file1=td_list_entry(dir_list.list.next, file_info_t, list);
+	      const file_info_t *file2=td_list_entry(file1->list.next, file_info_t, list);
+              dir_fat_aux(buffer, cluster_size, cluster_size, 0, &dir_list);
+              if(!td_list_empty(&dir_list.list) && (&file2->list==&dir_list.list || file1->st_ino!=file2->st_ino))
               {
                 int test_date=1;
                 if(verbose>0)
                 {
                   log_verbose("Potential root_cluster %lu\n",root_cluster);
-                  test_date=dir_aff_log(NULL, dir_list);
+                  test_date=dir_aff_log(NULL, &dir_list);
                 }
 #ifdef HAVE_NCURSES
                 if(interface>0 && interactive>0 && test_date>0)
                 {
-                  switch(ask_root_directory(disk_car,partition,dir_list,root_cluster))
+                  switch(ask_root_directory(disk_car, partition, &dir_list, root_cluster))
                   {
                     case c_YES:
-                      delete_list_file(dir_list);
+                      delete_list_file(&dir_list);
                       free(buffer);
                       return root_cluster;
                     case 'A':
                       interactive=0;
                       break;
 		    case 'Q':
-                      delete_list_file(dir_list);
+                      delete_list_file(&dir_list);
                       free(buffer);
                       return 0;
                     default:
@@ -535,7 +541,7 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
                 }
 #endif
               }
-              delete_list_file(dir_list);
+              delete_list_file(&dir_list);
             }
           }
         }
@@ -547,24 +553,24 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
     else
       log_error("Search root cluster failed\n");
     root_cluster=0;
-    if(rootdir_list==NULL)
+    if(td_list_empty(&rootdir_list.list))
     {
       log_warning("No first-level directory found.\n");
     }
     else
     {
-      dir_aff_log(NULL, rootdir_list);
+      dir_aff_log(NULL, &rootdir_list);
 #ifdef HAVE_NCURSES
       if(interface && expert>0)
       {
         if(ask_confirmation("Create a new root cluster with %u first-level directories (Expert only) (Y/N)",dir_nbr)!=0 && ask_confirmation("Write root cluster, confirm ? (Y/N)")!=0)
         {
           root_cluster=first_free_cluster;
-          fat32_create_rootdir(disk_car, partition, reserved, fat_length, root_cluster, sectors_per_cluster, verbose, rootdir_list, fats);
+          fat32_create_rootdir(disk_car, partition, reserved, fat_length, root_cluster, sectors_per_cluster, verbose, &rootdir_list, fats);
         }
       }
 #endif
-      delete_list_file(rootdir_list);
+      delete_list_file(&rootdir_list);
     }
     free(buffer);
   }
@@ -572,11 +578,11 @@ static unsigned int fat32_find_root_cluster(disk_t *disk_car,const partition_t *
 }
 
 #ifdef HAVE_NCURSES
-static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
-		  /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */
 
 static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned short *msdate)
 {
+  static const int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
+                          /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */
   int day,year,nl_day,month;
 
 /*  unix_date -= sys_tz.tz_minuteswest*60; */
@@ -603,7 +609,7 @@ static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned sho
   *msdate = le16(nl_day-day_n[month-1]+1+(month << 5)+(year << 9));
 }
 
-static int file2entry(struct msdos_dir_entry *de, const file_data_t *current_file)
+static int file2entry(struct msdos_dir_entry *de, const file_info_t *current_file)
 {
   unsigned int i,j;
   /* Name */
@@ -625,26 +631,22 @@ static int file2entry(struct msdos_dir_entry *de, const file_data_t *current_fil
   {
     de->ext[j]=' ';
   }
-  de->attr=(LINUX_S_ISDIR(current_file->stat.st_mode)!=0?ATTR_DIR:ATTR_NONE);
-  fat_date_unix2dos(current_file->stat.st_mtime,&de->time,&de->date);
-  de->start=le16(current_file->stat.st_ino);
-  de->starthi=le16(current_file->stat.st_ino>>16);
-#ifdef DJGPP
-  de->size=le32(current_file->file_size);
-#else
-  de->size=le32(current_file->stat.st_size);
-#endif
+  de->attr=(LINUX_S_ISDIR(current_file->st_mode)!=0?ATTR_DIR:ATTR_NONE);
+  fat_date_unix2dos(current_file->td_mtime,&de->time,&de->date);
+  de->start=le16(current_file->st_ino);
+  de->starthi=le16(current_file->st_ino>>16);
+  de->size=le32(current_file->st_size);
   return 0;
 }
 
-static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, const unsigned int reserved, const unsigned int fat_length, const unsigned int root_cluster, const unsigned int sectors_per_cluster, const int verbose, file_data_t *rootdir_list, const unsigned int fats)
+static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, const unsigned int reserved, const unsigned int fat_length, const unsigned int root_cluster, const unsigned int sectors_per_cluster, const int verbose, file_info_t *rootdir_list, const unsigned int fats)
 {
   const uint64_t start_data=reserved+fats*fat_length;
   const unsigned int cluster_size=disk_car->sector_size*sectors_per_cluster;
   unsigned int current_entry=0;
   unsigned int cluster;
   unsigned char *buffer;
-  file_data_t *current_file;
+  struct td_list_head *file_walker = NULL;
   if(verbose>0)
   {
     log_trace("fat32_create_rootdir(reserved=%u,fat_length=%u,root_cluster=%u,sectors_per_cluster=%u)\n",reserved,fat_length,root_cluster,sectors_per_cluster);
@@ -652,8 +654,9 @@ static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, c
   cluster=root_cluster;
   buffer=(unsigned char *)MALLOC(cluster_size);
   memset(buffer,0,cluster_size);
-  for(current_file=rootdir_list;current_file!=NULL;current_file=current_file->next)
+  td_list_for_each(file_walker, &rootdir_list->list)
   {
+    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
     file2entry((struct msdos_dir_entry*)buffer+current_entry,current_file);
     if(++current_entry==(cluster_size/sizeof(struct msdos_dir_entry)))
     {
@@ -675,7 +678,7 @@ static int fat32_create_rootdir(disk_t *disk_car,const partition_t *partition, c
       {
 	display_message("Can't modify the FAT entries.\n");
       }
-cluster=next_cluster;
+      cluster=next_cluster;
     }
   }
   if((unsigned)disk_car->pwrite(disk_car, buffer, cluster_size, partition->part_offset + (start_data + (uint64_t)(cluster - 2) * sectors_per_cluster) * disk_car->sector_size) != cluster_size)
@@ -692,7 +695,7 @@ cluster=next_cluster;
   }
 #ifdef DEBUG
   {
-    file_data_t *dir_list;
+    file_info_t *dir_list;
     dir_list=dir_fat_aux(buffer, cluster_size, cluster_size, 0);
     dir_aff_log(NULL, dir_list);
     delete_list_file(dir_list);
@@ -816,10 +819,13 @@ static int analyse_dir_entries(disk_t *disk_car,const partition_t *partition, co
 
 static int analyse_dir_entries2(disk_t *disk_car,const partition_t *partition, const unsigned int reserved, const unsigned int fat_length,const int verbose, unsigned int root_size_max,const upart_type_t upart_type, const unsigned int fats)
 {
-  file_data_t *dir_list=NULL;
-  file_data_t *current_file;
   unsigned char *buffer_dir;
   unsigned int root_dir_size;
+  file_info_t dir_list = {
+    .list = TD_LIST_HEAD_INIT(dir_list.list),
+    .name = NULL
+  };
+  struct td_list_head *file_walker = NULL;
   if(root_size_max==0)
   {
     root_size_max=4096;
@@ -834,61 +840,64 @@ static int analyse_dir_entries2(disk_t *disk_car,const partition_t *partition, c
     return 0;
   }
   {
-    uint64_t start_data=reserved+fats*fat_length+(root_size_max+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32);
-    unsigned int sectors_per_cluster=calcul_sectors_per_cluster(upart_type,partition->part_size/disk_car->sector_size-start_data,fat_length,disk_car->sector_size);
-    dir_list=dir_fat_aux(buffer_dir, root_dir_size, sectors_per_cluster*disk_car->sector_size,
-	(partition->upart_type==UP_FAT12?FLAG_LIST_MASK12:FLAG_LIST_MASK16));
+    const uint64_t start_data=reserved+fats*fat_length+(root_size_max+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32);
+    const unsigned int sectors_per_cluster=calcul_sectors_per_cluster(upart_type,partition->part_size/disk_car->sector_size-start_data,fat_length,disk_car->sector_size);
+    dir_fat_aux(buffer_dir, root_dir_size, sectors_per_cluster*disk_car->sector_size,
+	(partition->upart_type==UP_FAT12?FLAG_LIST_MASK12:FLAG_LIST_MASK16), &dir_list);
   }
   if(verbose>1)
   {
-    dir_aff_log(NULL, dir_list);
+    dir_aff_log(NULL, &dir_list);
   }
-  for(current_file=dir_list;(current_file!=NULL)&&(LINUX_S_ISDIR(current_file->stat.st_mode)==0);current_file=current_file->next);
-  if(current_file!=NULL)
+  td_list_for_each(file_walker, &dir_list.list)
   {
-    unsigned long int new_inode=current_file->stat.st_ino;
-    unsigned int dir_entries;
-    if(verbose>1)
+    const file_info_t *current_file=td_list_entry(file_walker, file_info_t, list);
+    if(LINUX_S_ISDIR(current_file->st_mode) && (current_file->status&FILE_STATUS_DELETED)==0)
     {
-      log_verbose("Directory %s used inode=%lu\n",current_file->name,new_inode);
-    }
-    for(dir_entries=(disk_car->sector_size/32);dir_entries<=root_size_max;dir_entries+=(disk_car->sector_size/32))
-    {
-      uint64_t start_data=reserved+fats*fat_length+(dir_entries+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32);
-      unsigned int sectors_per_cluster=calcul_sectors_per_cluster(upart_type,partition->part_size/disk_car->sector_size-start_data,fat_length,disk_car->sector_size);
+      const unsigned long int new_inode=current_file->st_ino;
+      unsigned int dir_entries;
       if(verbose>1)
       {
-        log_verbose("dir_entries %u, sectors_per_cluster %u\n",dir_entries,sectors_per_cluster);
+	log_verbose("Directory %s used inode=%lu\n",current_file->name,new_inode);
       }
-      if((unsigned)disk_car->pread(disk_car, buffer_dir, disk_car->sector_size,
-	    partition->part_offset + (start_data + (uint64_t)(new_inode - 2) * sectors_per_cluster) * disk_car->sector_size) == disk_car->sector_size)
+      for(dir_entries=disk_car->sector_size/32;
+	  dir_entries <= root_size_max;
+	  dir_entries+=disk_car->sector_size/32)
       {
-        if((memcmp(&buffer_dir[0],".          ",8+3)==0)&&(memcmp(&buffer_dir[0x20],"..         ",8+3)==0))
-        {
-          unsigned long int cluster=(buffer_dir[0*0x20+0x15]<<24)+(buffer_dir[0*0x20+0x14]<<16)+
-            (buffer_dir[0*0x20+0x1B]<<8)+buffer_dir[0*0x20+0x1A];
-          unsigned long int cluster_prev=(buffer_dir[1*0x20+0x15]<<24)+(buffer_dir[1*0x20+0x14]<<16)+
-            (buffer_dir[1*0x20+0x1B]<<8)+buffer_dir[1*0x20+0x1A];
-          if(verbose>1)
-          {
-            log_verbose("Cluster %lu, directory .. found link to %lu\n",cluster,cluster_prev);
-          }
-          if(cluster_prev==0 && cluster==new_inode)
-          {
-            free(buffer_dir);
-            delete_list_file(dir_list);
-            return ((dir_entries+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32))*(disk_car->sector_size/32);
-          }
-        }
+	const uint64_t start_data=reserved+fats*fat_length+(dir_entries+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32);
+	const unsigned int sectors_per_cluster=calcul_sectors_per_cluster(upart_type,partition->part_size/disk_car->sector_size-start_data,fat_length,disk_car->sector_size);
+	if(verbose>1)
+	{
+	  log_verbose("dir_entries %u, sectors_per_cluster %u\n",dir_entries,sectors_per_cluster);
+	}
+	if((unsigned)disk_car->pread(disk_car, buffer_dir, disk_car->sector_size,
+	      partition->part_offset + (start_data + (uint64_t)(new_inode - 2) * sectors_per_cluster) * disk_car->sector_size) == disk_car->sector_size)
+	{
+	  if(memcmp(&buffer_dir[0],".          ",8+3)==0 &&
+	      memcmp(&buffer_dir[0x20],"..         ",8+3)==0)
+	  {
+	    const unsigned long int cluster=(buffer_dir[0*0x20+0x15]<<24)+(buffer_dir[0*0x20+0x14]<<16)+
+	      (buffer_dir[0*0x20+0x1B]<<8)+buffer_dir[0*0x20+0x1A];
+	    const unsigned long int cluster_prev=(buffer_dir[1*0x20+0x15]<<24)+(buffer_dir[1*0x20+0x14]<<16)+
+	      (buffer_dir[1*0x20+0x1B]<<8)+buffer_dir[1*0x20+0x1A];
+	    if(verbose>1)
+	    {
+	      log_verbose("Cluster %lu, directory .. found link to %lu\n",cluster,cluster_prev);
+	    }
+	    if(cluster_prev==0 && cluster==new_inode)
+	    {
+	      free(buffer_dir);
+	      delete_list_file(&dir_list);
+	      return ((dir_entries+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32))*(disk_car->sector_size/32);
+	    }
+	  }
+	}
       }
     }
   }
-  else
-  {
-    log_warning("No directory found\n");
-  }
+  log_warning("No directory found\n");
   free(buffer_dir);
-  delete_list_file(dir_list);
+  delete_list_file(&dir_list);
   return root_size_max;
 }
 
@@ -912,20 +921,23 @@ static void fat32_dump_ncurses(disk_t *disk_car, const partition_t *partition, c
 }
 #endif
 
-static void fat32_dump(disk_t *disk_car, const partition_t *partition, const upart_type_t upart_type, const unsigned char *orgboot, const unsigned char *newboot)
+static void fat32_dump(disk_t *disk_car, const partition_t *partition, const upart_type_t upart_type, const unsigned char *orgboot, const unsigned char *newboot, char **current_cmd)
 {
   log_info("     Rebuild Boot sector           Boot sector\n");
   dump2_log(newboot,orgboot, (unsigned int)(upart_type==UP_FAT32?3*disk_car->sector_size:DEFAULT_SECTOR_SIZE));
+  if(*current_cmd==NULL)
+  {
 #ifdef HAVE_NCURSES
-  fat32_dump_ncurses(disk_car, partition, upart_type, orgboot, newboot);
+    fat32_dump_ncurses(disk_car, partition, upart_type, orgboot, newboot);
 #endif
+  }
 }
 
 static void menu_write_fat_boot_sector(disk_t *disk_car, partition_t *partition, const int verbose, const upart_type_t upart_type, const unsigned char *orgboot, const unsigned char*newboot, const int error, char **current_cmd)
 {
   const struct fat_boot_sector *fat_header=(const struct fat_boot_sector *)newboot;
 #ifdef HAVE_NCURSES
-  struct MenuItem menuSaveBoot[]=
+  const struct MenuItem menuSaveBoot[]=
   {
     { 'D', "Dump", "Dump sector" },
     { 'L', "List", "List directories and files" },
@@ -1016,7 +1028,7 @@ static void menu_write_fat_boot_sector(disk_t *disk_car, partition_t *partition,
       case 'd':
       case 'D':
 	if(strchr(options,'D')!=NULL)
-	  fat32_dump(disk_car, partition, upart_type, orgboot, newboot);
+	  fat32_dump(disk_car, partition, upart_type, orgboot, newboot, current_cmd);
 	break;
       case 'l':
       case 'L':
@@ -1074,13 +1086,13 @@ static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, con
 {
   unsigned char *orgboot;
   unsigned char *newboot;
-  struct fat_boot_sector *org_fat_header;
+  const struct fat_boot_sector *org_fat_header;
   struct fat_boot_sector *fat_header;
   int error=0;
   unsigned long int part_size=0;
   orgboot=(unsigned char *)MALLOC(3*disk_car->sector_size);
   newboot=(unsigned char *)MALLOC(3*disk_car->sector_size);
-  org_fat_header=(struct fat_boot_sector *)orgboot;
+  org_fat_header=(const struct fat_boot_sector *)orgboot;
   fat_header=(struct fat_boot_sector *)newboot;
   if((unsigned)disk_car->pread(disk_car, orgboot, 3 * disk_car->sector_size, partition->part_offset) != 3 * disk_car->sector_size)
   {
@@ -1113,9 +1125,11 @@ static void create_fat_boot_sector(disk_t *disk_car, partition_t *partition, con
       memcmp(fat_header->system_id,"MSWIN4.1",8))
     memcpy(fat_header->system_id,"MSWIN4.1",8);
   /* FIXME, need to know where the extended or logical partition start */
+#if 0
   if(partition->status==STATUS_LOG)
     fat_header->hidden=le32(disk_car->geom.sectors_per_head);
   else
+#endif
     fat_header->hidden=le32((partition->part_offset/disk_car->sector_size));
   fat_header->sectors_per_cluster=sectors_per_cluster;
   fat_header->reserved=le16(reserved);
@@ -1592,7 +1606,7 @@ static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uin
       offset+=disk_car->sector_size)
   {
 #ifdef HAVE_NCURSES
-    unsigned long int percent=offset*100/max_offset;
+    const unsigned long int percent=offset*100/max_offset;
     if(interface && (percent!=old_percent))
     {
       wmove(stdscr,8,30);
@@ -1606,8 +1620,7 @@ static int fat_find_type(disk_t *disk_car,const partition_t *partition,const uin
     if((unsigned)disk_car->pread(disk_car, buffer, disk_car->sector_size, partition->part_offset + offset) == disk_car->sector_size)
     {
       unsigned long int fat_offset=0;
-      unsigned int fat_type;
-      fat_type=fat_find_fat_start(buffer,p_fat12,p_fat16,p_fat32,&fat_offset,disk_car->sector_size);
+      const unsigned int fat_type=fat_find_fat_start(buffer,p_fat12,p_fat16,p_fat32,&fat_offset,disk_car->sector_size);
       if(fat_type!=0 && fat_offset<=(offset/disk_car->sector_size))
       {
 	unsigned int j;
@@ -1803,7 +1816,7 @@ static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsi
 {
   unsigned int i;
   unsigned long int fat2_location=*reserved+*fat_length;
-  struct MenuItem menuSelectFAT[]=
+  const struct MenuItem menuSelectFAT[]=
   {
     { 'P', "Previous",""},
     { 'N', "Next","" },
@@ -1829,7 +1842,11 @@ static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsi
   {
     wmove(stdscr,INTER_FAT_ASK_Y, INTER_FAT_ASK_X);
     fat2_location=ask_number(fat2_location,0,max_sector_offset,"FAT2 location ");
-    if(fat2_location>*reserved)
+    if(fat2_location == *reserved)
+    {
+      *fats=1;
+    }
+    else if(fat2_location > *reserved)
     {
       *fat_length=fat2_location-*reserved;
       wmove(stdscr,INTER_FAT_ASK_Y, INTER_FAT_ASK_X);
@@ -1876,7 +1893,6 @@ static upart_type_t select_fat_info(const info_offset_t *info_offset, const unsi
 
 int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose, const int dump_ind,const int interface,const unsigned int expert, char**current_cmd)
 {
-  unsigned long int data_size;
   unsigned long int max_offset;
   unsigned int fat_length=0;
   unsigned int sectors_per_cluster=0;
@@ -2015,7 +2031,7 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
       (fat_length==0)||(reserved==0))
   {
     uint64_t start_data=0;
-    if(find_sectors_per_cluster(disk_car, partition, verbose, dump_ind, interface,&sectors_per_cluster,&start_data)==0)
+    if(find_sectors_per_cluster(disk_car, partition, verbose, dump_ind, interface, &sectors_per_cluster, &start_data, upart_type)==0)
     {
       display_message("Can't find cluster size\n");
       return 0;
@@ -2063,9 +2079,12 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
 	fat_length=(start_data-reserved-((dir_entries-1)/16+1))/fats;
 	break;
       case UP_FAT32:
-	reserved=32;
-	if((start_data&1)!=0)
-	  reserved+=1;
+	if(reserved==0)
+	{
+	  reserved=32;
+	  if((start_data&1)!=0)
+	    reserved+=1;
+	}
 	fat_length=(start_data-reserved)/fats;
 	break;
       default:	/* No compiler warning */
@@ -2084,6 +2103,7 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
 #endif
   if(upart_type && (fat_length>1))
   {
+    unsigned long int data_size;
     /* Initialized by fat32_free_info */
     unsigned int free_cluster_count=0;
     unsigned int first_free_cluster=0;
@@ -2207,7 +2227,7 @@ int rebuild_FAT_BS(disk_t *disk_car, partition_t *partition, const int verbose,
   return 0;
 }
 
-int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose)
+int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose, char **current_cmd)
 {
   unsigned long int fat_length,sector;
   uint64_t start_rootdir,start_data;
@@ -2253,7 +2273,10 @@ int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose
   }
   if(error==0)
   {
-    display_message("TestDisk doesn't seem needed to reset the root directory.\n");
+    if(*current_cmd!=NULL)
+      log_info("TestDisk doesn't seem needed to reset the root directory.\n");
+    else
+      display_message("TestDisk doesn't seem needed to reset the root directory.\n");
     free(buffer);
     return 0;
   }
@@ -2283,10 +2306,10 @@ int FAT_init_rootdir(disk_t *disk_car, partition_t *partition, const int verbose
   return 0;
 }
 
-enum fat_status_type { FAT_UNREADABLE=0, FAT_CORRUPTED=1, FAT_OK=2 };
-enum fat_ask_repair { FAT_REPAIR_ASK=0, FAT_REPAIR_YES=1, FAT_REPAIR_NO=2 };
+typedef enum { FAT_UNREADABLE=0, FAT_CORRUPTED=1, FAT_OK=2 } fat_status_t;
+typedef enum { FAT_REPAIR_ASK=0, FAT_REPAIR_YES=1, FAT_REPAIR_NO=2 } fat_repair_t;
 
-int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose)
+int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose, char **current_cmd)
 {
   if(check_FAT(disk_car,partition,verbose)!=0)
   {
@@ -2327,8 +2350,8 @@ int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose
       return 1;
     {
       const unsigned int buffer_size=(partition->upart_type==UP_FAT12?2*disk_car->sector_size:disk_car->sector_size);
-      unsigned int fat_status[2];
-      unsigned int allow_write[2];
+      fat_status_t fat_status[2];
+      fat_repair_t allow_write[2];
       unsigned int fat_history[2][3];
       unsigned int old_offset_s=1234;
       unsigned int fat_mismatch=0;
@@ -2800,7 +2823,10 @@ int repair_FAT_table(disk_t *disk_car, partition_t *partition, const int verbose
     }
     if(fat_damaged==0)
     {
-      display_message("FATs seems Ok, nothing to do.\n");
+      if(current_cmd!=NULL)
+	log_info("FATs seems Ok, nothing to do.\n");
+      else
+	display_message("FATs seems Ok, nothing to do.\n");
     }
     else
     {
