@@ -55,15 +55,12 @@ struct fat_dir_struct
 };
 
 
-static int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
-static file_data_t *fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const struct fat_boot_sector*fat_header);
-static file_data_t *fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster);
+static int fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const struct fat_boot_sector*fat_header, file_info_t *dir_list);
+static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list);
 static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len);
-static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
+static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
 static void dir_partition_fat_close(dir_data_t *dir_data);
 
-static int32_t secwest;
-
 static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 {
 	while (len--) {
@@ -72,18 +69,17 @@ static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 	}
 }
 
-file_data_t *dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int cluster_size, const unsigned int param)
+int dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int cluster_size, const unsigned int param, file_info_t *dir_list)
 {
   const struct msdos_dir_entry *de=(const struct msdos_dir_entry*)buffer;
   wchar_t unicode[1000];
   unsigned char long_slots;
-  file_data_t *dir_list=NULL;
-  file_data_t *current_file=NULL;
   unsigned int status;
   unsigned int inode;
   int utf8=1;
 #ifdef HAVE_WCTOMB
-  wctomb(NULL, 0);
+  if(wctomb(NULL, 0) < 0)
+    utf8=0;
 #endif
 GetNew:
   status=0;
@@ -108,7 +104,7 @@ ParseLongDeleted:
     while (1)
     {
       if((const void*)de>=(const void*)(buffer+size))
-	goto EODir;
+	return 0;
       ds = (const struct msdos_dir_slot *) de;
       if(de->name[0] != (int8_t) DELETED_FLAG)
 	goto GetNew;
@@ -188,7 +184,7 @@ ParseLong:
 	}
 	de++;
 	if((const void*)de>=(const void*)(buffer+size))
-	  goto EODir;
+	  return 0;
 	if (slot == 0)
 	  break;
 	ds = (const struct msdos_dir_slot *) de;
@@ -247,8 +243,8 @@ RecEnd:
      {
       status=FILE_STATUS_DELETED;
       if((de->attr&ATTR_DIR)==ATTR_DIR &&
-	((dir_list==NULL && unicode[1]=='\0') ||
-	 (dir_list!=NULL && dir_list->next==NULL && unicode[1]=='.' && unicode[2]=='\0')))
+	((td_list_empty(&dir_list->list) && unicode[1]=='\0') ||
+	 (!td_list_empty(&dir_list->list) && dir_list->list.next==dir_list->list.prev && unicode[1]=='.' && unicode[2]=='\0')))
 	unicode[0]='.';	/* "." and ".." are the first two entries */
       else
 	unicode[0]='_';
@@ -258,12 +254,13 @@ RecEnd:
       !(de->attr & ATTR_VOLUME))
   {
     if(unicode[0]==0)
-      return dir_list;
+      return 0;
     if((int8_t) unicode[0] != (int8_t) DELETED_FLAG)
     {
       unsigned int i,o;
-      file_data_t *new_file=(file_data_t *)MALLOC(sizeof(*new_file));
-      for(i=0,o=0; unicode[i]!=0 && o<sizeof(new_file->name)-1; i++)
+      file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
+      new_file->name=(char*)MALLOC(DIR_NAME_LEN);
+      for(i=0,o=0; unicode[i]!=0 && o<DIR_NAME_LEN-1; i++)
       {
 	if(utf8 && unicode[i]>0x7f)
 	{
@@ -284,67 +281,26 @@ RecEnd:
 	  new_file->name[o++]=(char) unicode[i];
       }
       new_file->name[o]=0;
-      new_file->stat.st_dev=0;
-      new_file->stat.st_ino=inode;
-      new_file->stat.st_mode = MSDOS_MKMODE(de->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
-      new_file->stat.st_nlink=0;
-      new_file->stat.st_uid=0;
-      new_file->stat.st_gid=0;
-      new_file->stat.st_rdev=0;
-      new_file->stat.st_size=le32(de->size);
-#ifdef DJGPP
-      new_file->file_size=le32(de->size);
-#endif
-#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-      new_file->stat.st_blksize=cluster_size;
-#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-      if(new_file->stat.st_blksize!=0)
-      {
-	new_file->stat.st_blocks=(le32(de->size) + new_file->stat.st_blksize - 1) / new_file->stat.st_blksize;
-      }
-#endif
-#endif
-      new_file->stat.st_atime=new_file->stat.st_ctime=new_file->stat.st_mtime=date_dos2unix(le16(de->time),le16(de->date));
+      new_file->st_ino=inode;
+      new_file->st_mode = MSDOS_MKMODE(de->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
+      new_file->st_uid=0;
+      new_file->st_gid=0;
+      new_file->st_size=le32(de->size);
+//      new_file->st_blksize=cluster_size;
+      new_file->td_atime=new_file->td_ctime=new_file->td_mtime=date_dos2unix(le16(de->time),le16(de->date));
       new_file->status=status;
-      new_file->prev=current_file;
-      new_file->next=NULL;
       /* log_debug("fat: new file %s de=%p size=%u\n",new_file->name,de,le32(de->size)); */
-      if(current_file!=NULL)
-        current_file->next=new_file;
-      else
-        dir_list=new_file;
-      current_file=new_file;
+      td_list_add_tail(&new_file->list, &dir_list->list);
     }
   }
   de++;
   if((const void *)de<(const void *)(buffer+size-1) &&
       de->name[0] != (int8_t) 0)
     goto GetNew;
-EODir:
-  return dir_list;
+  return 0;
 }
 
-static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
-		  /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */
-
-/* Convert a MS-DOS time/date pair to a UNIX date (seconds since 1 1 70). */
-
-static int date_dos2unix(const unsigned short f_time, const unsigned short f_date)
-{
-	int month,year,secs;
-
-	/* first subtract and mask after that... Otherwise, if
-	   f_date == 0, bad things happen */
-	month = ((f_date >> 5) - 1) & 15;
-	year = f_date >> 9;
-	secs = (f_time & 31)*2+60*((f_time >> 5) & 63)+(f_time >> 11)*3600+86400*
-	    ((f_date & 31)-1+day_n[month]+(year/4)+year*365-((year & 3) == 0 &&
-	    month < 2 ? 1 : 0)+3653);
-			/* days since 1.1.70 plus 80's leap day */
-	return secs+secwest;
-}
-
-enum {FAT_FOLLOW_CLUSTER, FAT_NEXT_FREE_CLUSTER, FAT_NEXT_CLUSTER};
+typedef enum {FAT_FOLLOW_CLUSTER, FAT_NEXT_FREE_CLUSTER, FAT_NEXT_CLUSTER} fat_method_t;
 
 static int is_EOC(const unsigned int cluster, const upart_type_t upart_type)
 {
@@ -357,7 +313,7 @@ static int is_EOC(const unsigned int cluster, const upart_type_t upart_type)
 }
 
 #define NBR_CLUSTER_MAX 30
-static file_data_t *fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster)
+static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list)
 {
   const struct fat_dir_struct *ls=(const struct fat_dir_struct*)dir_data->private_dir_data;
   const struct fat_boot_sector*fat_header=ls->boot_sector;
@@ -365,33 +321,32 @@ static file_data_t *fat_dir(disk_t *disk_car, const partition_t *partition, dir_
   if(fat_header->sectors_per_cluster<1)
   {
     log_error("FAT: Can't list files, bad cluster size.\n");
-    return NULL;
+    return -1;
   }
   if(fat_sector_size(fat_header)==0)
   {
     log_error("FAT: Can't list files, bad sector size.\n");
-    return NULL;
+    return -1;
   }
   if(cluster==0)
   {
     if(partition->upart_type!=UP_FAT32)
-      return fat1x_rootdir(disk_car, partition, dir_data, fat_header);
+      return fat1x_rootdir(disk_car, partition, dir_data, fat_header, dir_list);
     if(le32(fat_header->root_cluster)<2)
     {
       log_error("FAT32: Can't list files, bad root cluster.\n");
-      return NULL;
+      return -1;
     }
     cluster=le32(fat_header->root_cluster);
   }
   {
-    file_data_t *dir_list=NULL;
     const unsigned int cluster_size=fat_header->sectors_per_cluster * fat_sector_size(fat_header);
     unsigned char *buffer_dir=(unsigned char *)MALLOC(cluster_size*NBR_CLUSTER_MAX);
     unsigned int nbr_cluster;
     int stop=0;
     uint64_t start_fat1,start_data,part_size;
     unsigned long int no_of_cluster,fat_length;
-    unsigned int fat_meth=FAT_FOLLOW_CLUSTER;
+    fat_method_t fat_meth=FAT_FOLLOW_CLUSTER;
     memset(buffer_dir,0,cluster_size*NBR_CLUSTER_MAX);
     fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
     part_size=(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect));
@@ -451,23 +406,23 @@ static file_data_t *fat_dir(disk_t *disk_car, const partition_t *partition, dir_
       }
     }
     if(nbr_cluster>0)
-      dir_list=dir_fat_aux(buffer_dir, cluster_size*nbr_cluster, cluster_size, dir_data->param);
+      dir_fat_aux(buffer_dir, cluster_size*nbr_cluster, cluster_size, dir_data->param, dir_list);
     free(buffer_dir);
-    return dir_list;
+    return 0;
   }
 }
 
-static file_data_t *fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const struct fat_boot_sector*fat_header)
+static int fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const struct fat_boot_sector*fat_header, file_info_t *dir_list)
 {
   const unsigned int root_size=(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
   if(root_size==0)
-    return NULL;
+    return -1;
   if(dir_data->verbose>1)
   {
     log_trace("fat1x_rootdir root_size=%u sectors\n",root_size/disk_car->sector_size);
   }
   {
-    file_data_t *res=NULL;
+    int res;
     uint64_t start;
     unsigned char *buffer_dir;
     buffer_dir=(unsigned char*)MALLOC(root_size);
@@ -477,36 +432,14 @@ static file_data_t *fat1x_rootdir(disk_t *disk_car, const partition_t *partition
       log_error("FAT 1x: Can't read root directory.\n");
       /* Don't return yet, it may have been a partial read */
     }
-    res=dir_fat_aux(buffer_dir, root_size, fat_header->sectors_per_cluster * fat_sector_size(fat_header), dir_data->param);
+    res=dir_fat_aux(buffer_dir, root_size, fat_header->sectors_per_cluster * fat_sector_size(fat_header), dir_data->param, dir_list);
     free(buffer_dir);
     return res;
   }
 }
 
-static void set_secwest(void)
-{
-  struct  tm *tmptr;
-  time_t t;
 
-  t = time(NULL);
-  tmptr = localtime(&t);
-#ifdef HAVE_STRUCT_TM_TM_GMTOFF
-  secwest = -1 * tmptr->tm_gmtoff;
-#elif defined (DJGPP)
-  secwest = 0;
-#else
-#if defined (__CYGWIN__)
-  secwest = _timezone;
-#else
-  secwest = timezone;
-#endif
-  /* account for daylight savings */
-  if (tmptr->tm_isdst)
-    secwest -= 3600;
-#endif
-}
-
-int dir_partition_fat_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
+dir_partition_t dir_partition_fat_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
   static unsigned char *buffer;
   static struct fat_dir_struct *ls;
@@ -515,7 +448,7 @@ int dir_partition_fat_init(disk_t *disk_car, const partition_t *partition, dir_d
   {
     log_error("Can't read FAT boot sector.\n");
     free(buffer);
-    return -1;
+    return DIR_PART_EIO;
   }
   set_secwest();
   ls=(struct fat_dir_struct *)MALLOC(sizeof(*ls));
@@ -534,7 +467,7 @@ int dir_partition_fat_init(disk_t *disk_car, const partition_t *partition, dir_d
   dir_data->local_dir=NULL;
   dir_data->private_dir_data=ls;
   dir_data->get_dir=fat_dir;
-  return 0;
+  return DIR_PART_OK;
 }
 
 static void dir_partition_fat_close(dir_data_t *dir_data)
@@ -544,7 +477,7 @@ static void dir_partition_fat_close(dir_data_t *dir_data)
   free(ls);
 }
 
-static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
+static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file)
 {
   char *new_file;	
   FILE *f_out;
@@ -554,12 +487,8 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
   const unsigned int block_size=fat_sector_size(fat_header)*sectors_per_cluster;
   unsigned char *buffer_file=(unsigned char *)MALLOC(block_size);
   unsigned int cluster;
-#ifdef DJGPP
-  unsigned int file_size=file->file_size;
-#else
-  unsigned int file_size=file->stat.st_size;
-#endif
-  unsigned int fat_meth=FAT_FOLLOW_CLUSTER;
+  unsigned int file_size=file->st_size;
+  fat_method_t fat_meth=FAT_FOLLOW_CLUSTER;
   uint64_t start_fat1,start_data,part_size;
   unsigned long int no_of_cluster,fat_length;
   f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
@@ -570,7 +499,7 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
     free(buffer_file);
     return -1;
   }
-  cluster = file->stat.st_ino;
+  cluster = file->st_ino;
   fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
   part_size=(sectors(fat_header)>0?sectors(fat_header):le32(fat_header->total_sect));
   start_fat1=le16(fat_header->reserved);
@@ -595,7 +524,7 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
     {
       log_error("fat_copy: failed to write data %s\n", strerror(errno));
       fclose(f_out);
-      set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
+      set_date(new_file, file->td_atime, file->td_mtime);
       free(new_file);
       free(buffer_file);
       return -1;
@@ -608,7 +537,7 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
 	const unsigned int next_cluster=get_next_cluster(disk_car, partition, partition->upart_type, start_fat1, cluster);
 	if(next_cluster>=2 && next_cluster<=no_of_cluster+2)
 	  cluster=next_cluster;
-	else if(cluster==file->stat.st_ino && next_cluster==0)
+	else if(cluster==file->st_ino && next_cluster==0)
 	  fat_meth=FAT_NEXT_FREE_CLUSTER;	/* Recovery of a deleted file */
 	else
 	  fat_meth=FAT_NEXT_CLUSTER;		/* FAT is corrupted, don't trust it */
@@ -623,7 +552,7 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
     }
   }
   fclose(f_out);
-  set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
+  set_date(new_file, file->td_atime, file->td_mtime);
   free(new_file);
   free(buffer_file);
   return 0;
