@@ -96,6 +96,7 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
 #endif
       return 0;
     }
+    file_recovery_new->time=le32(pe_hdr->TimeDateStamp);
 #ifdef DEBUG_EXE
     {
       const struct pe_image_optional_hdr32 *pe_image_optional32=(const struct pe_image_optional_hdr32 *)
@@ -171,8 +172,7 @@ static int header_check_exe(const unsigned char *buffer, const unsigned int buff
     )
   {
     /* MSDOS EXE */
-    uint64_t coff_offset=0;
-    coff_offset=le16(dos_hdr->blocks_in_file)*512;
+    uint64_t coff_offset=le16(dos_hdr->blocks_in_file)*512;
     if(le16(dos_hdr->bytes_in_last_block))
       coff_offset-=512-le16(dos_hdr->bytes_in_last_block);
 
@@ -253,10 +253,9 @@ static unsigned int ReadUnicodeStr(const char *buffer, unsigned int pos, const u
   return pos;
 }
 
-static int PEVersion_aux(const char*buffer, const unsigned int length, const char *old_filename, const char *needle, const unsigned int needle_len, const int force_ext)
+static int PEVersion_aux(const char*buffer, const unsigned int end, const char *old_filename, const char *needle, const unsigned int needle_len, const int force_ext)
 {
   unsigned int pos=0;
-  unsigned int end=length;
   while(1)
   {
     const struct PE_index *PE_index;
