@@ -26,9 +26,6 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#ifdef HAVE_TIME_H
-#include <time.h>
-#endif
 #include <stdio.h>
 #include "types.h"
 #include "filegen.h"
