@@ -47,8 +47,11 @@
 #include "log.h"
 #include "file_jpg.h"
 #include "file_tiff.h"
+#include "setdate.h"
 
+extern const file_hint_t file_hint_doc;
 extern const file_hint_t file_hint_indd;
+extern const file_hint_t file_hint_mov;
 extern const file_hint_t file_hint_riff;
 extern int data_check_avi_stream(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 
@@ -72,9 +75,12 @@ static const unsigned char jpg_header_app1[4]= { 0xff,0xd8,0xff,0xe1};
 static const unsigned char jpg_header_app12[4]= { 0xff,0xd8,0xff,0xec};
 static const unsigned char jpg_header_com[4]= { 0xff,0xd8,0xff,0xfe};
 static const unsigned char jpg_footer[2]= { 0xff,0xd9};
-static const unsigned char jpg_header_app0_avi[0x14]= {
-  0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'A', 'V', 'I', '1', 0x00, 0x00,
-  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
+static const unsigned char jpg_header_app0_avi[0x0c]= {
+  0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'A', 'V', 'I', '1', 0x00, 0x00
+};
+static const unsigned char jpg_header_app0_jfif11_null[0x14]= {
+  0xff, 0xd8, 0xff, 0xe0, 0x00, 0x10, 'J', 'F', 'I', 'F', 0x00, 0x01,
+  0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 };
 
 static void register_header_check_jpg(file_stat_t *file_stat)
@@ -110,57 +116,196 @@ static void jpg_get_size(const unsigned char *buffer, const unsigned int buffer_
   }
 }
 
+struct MP_IFD_Field
+{
+  uint16_t tag;
+  uint16_t type;
+  uint32_t count;
+  char     value[0];
+} __attribute__ ((__packed__));
+
+struct MP_Entry
+{
+  uint32_t attr;
+  uint32_t size;
+  uint32_t offset;
+  uint16_t dep1;
+  uint16_t dep2;
+} __attribute__ ((__packed__));
+
+static uint64_t check_mpo_le(const unsigned char *mpo, const uint64_t mpo_offset, const unsigned int size)
+{
+  const uint16_t *tmp16;
+  const uint32_t *tmp32=(const uint32_t *)(&mpo[4]);
+  unsigned int offset=le32(*tmp32);
+  unsigned int i;
+  unsigned int nbr;
+  unsigned int NumberOfImages=0;
+  unsigned int MPEntry_offset=0;
+  const struct MP_Entry* MPEntry;
+  uint64_t max_offset=0;
+#ifdef DEBUG_JPEG
+  log_info("check_mpo_le\n");
+#endif
+  if(offset+2 >= size)
+    return 0;
+  tmp16=(const uint16_t*)(&mpo[offset]);
+  nbr=le16(*tmp16);
+  offset+=2;
+  for(i=0; i< nbr; i++)
+  {
+    const struct MP_IFD_Field *field=(const struct MP_IFD_Field *)(&mpo[offset]);
+    if(offset+12 > size)
+      return 0;
+    switch(le16(field->tag))
+    {
+      case 0xb000:
+	/* MPFVersion, type must be undefined */
+	if(le16(field->type)!=7 || le32(field->count)!=4)
+	  return 0;
+	break;
+      case 0xb001:
+	/* NumberOfImages, type must be long */
+	if(le16(field->type)!=4 || le32(field->count)!=1)
+	  return 0;
+	{
+	  const uint32_t *tmp=(const uint32_t *)&field->value[0];
+	  NumberOfImages=le32(*tmp);
+	}
+	break;
+      case 0xb002:
+	/* MPEntry, type must be undefined */
+	if(le16(field->type)!=7 || le32(field->count)!=16*NumberOfImages)
+	  return 0;
+	{
+	  const uint32_t *tmp=(const uint32_t *)&field->value[0];
+	  MPEntry_offset=le32(*tmp);
+	}
+	break;
+    }
+    offset+=12;
+  }
+#ifdef DEBUG_JPEG
+  log_info("MPEntry_offset=%u, NumberOfImages=%u\n", MPEntry_offset, NumberOfImages);
+#endif
+  if(MPEntry_offset + 16*NumberOfImages > size)
+    return 0;
+  for(i=0, MPEntry=(const struct MP_Entry*)(&mpo[MPEntry_offset]);
+      i<NumberOfImages;
+      i++, MPEntry++)
+  {
+    uint64_t tmp=le32(MPEntry->offset)+le32(MPEntry->size);
+#ifdef DEBUG_JPEG
+    log_info("offset=%lu, size=%lu\n",
+	(long unsigned)le32(MPEntry->offset),
+	(long unsigned)le32(MPEntry->size));
+#endif
+    if(le32(MPEntry->offset)>0)
+      tmp+=mpo_offset;
+    if(max_offset < tmp)
+      max_offset = tmp;
+  }
+  return max_offset;
+}
+
+static void file_check_mpo(file_recovery_t *fr)
+{
+  unsigned char buffer[512];
+  uint64_t offset=0;
+  unsigned int size=0;
+  size_t nbytes;
+  const unsigned char *mpo;
+  uint64_t max_offset=0;
+  do
+  {
+    offset+=2+size;
+    fseek(fr->handle, offset, SEEK_SET);
+    nbytes=fread(&buffer, 1, sizeof(buffer), fr->handle);
+//    log_info("file_check_mpo offset=%llu => nbytes=%d, buffer=%02x %02x\n",
+//    (long long unsigned)offset, nbytes, buffer[0], buffer[1]);
+    /* 0xda SOS Start Of Scan */
+    if(nbytes<8 || buffer[0]!=0xff || buffer[1]==0xda)
+    {
+      fr->file_size=0;
+      return ;
+    }
+    size=(buffer[2]<<8)+buffer[3];
+  } while(!(buffer[1]==0xe2 &&
+	  buffer[4]=='M' && buffer[5]=='P' && buffer[6]=='F' && buffer[7]==0));
+#ifdef DEBUG_JPEG
+  log_info("Found at %lu\n", (long unsigned)offset);
+#endif
+  if(2+size < nbytes)
+    size=nbytes-2;
+  if(size<12)
+  {
+    fr->file_size=0;
+    return ;
+  }
+  offset+=8;
+  size-=8;
+  mpo=buffer+8;
+  if(mpo[0]=='I' && mpo[1]=='I' && mpo[2]=='*' && mpo[3]==0)
+    max_offset=check_mpo_le(mpo, offset, size);
+  else if(mpo[0]=='M' && mpo[1]=='M' && mpo[2]==0 && mpo[3]=='*')
+    return ;
+  fr->file_size=( max_offset > fr->file_size ? 0: max_offset);
+}
+
 static int header_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  unsigned int width=0;
-  unsigned int height=0;
-  jpg_get_size(buffer, buffer_size, &height, &width);
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_indd)
-    return 0;
-  /* Don't recover the thumb instead of the jpg itself */
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_jpg &&
-      file_recovery->file_size <= 1024 &&
-      memcmp(buffer, jpg_header_app12, sizeof(jpg_header_app12))==0)
+  if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
   {
-    log_info("jpg %llu %llu\n",
-	(long long unsigned)file_recovery->calculated_file_size,
-	(long long unsigned)file_recovery->file_size);
-    return 0;
+    unsigned int width=0;
+    unsigned int height=0;
+    jpg_get_size(buffer, buffer_size, &height, &width);
+    if(file_recovery->file_stat->file_hint==&file_hint_indd)
+      return 0;
+    if(file_recovery->file_stat->file_hint==&file_hint_doc &&
+      strstr(file_recovery->filename, ".albm")!=NULL)
+      return 0;
+    if( file_recovery->file_stat->file_hint==&file_hint_jpg)
+    {
+      /* Don't recover the thumb instead of the jpg itself */
+      if( file_recovery->file_size <= 1024 &&
+	  memcmp(buffer, jpg_header_app12, sizeof(jpg_header_app12))==0)
+      {
+	log_info("jpg %llu %llu\n",
+	    (long long unsigned)file_recovery->calculated_file_size,
+	    (long long unsigned)file_recovery->file_size);
+	return 0;
+      }
+      /* Don't recover the thumb instead of the jpg itself */
+      if(file_recovery->file_size <= 4096 &&
+	  memcmp(buffer, jpg_header_app0, sizeof(jpg_header_app0))==0 &&
+	  width>0 && width<200 && height>0 && height<200)
+      {
+	return 0;
+      }
+      /* Some JPG have two APP1 markers, avoid to dicard the first one */
+      if( memcmp(buffer, jpg_header_app1, sizeof(jpg_header_app1))==0 &&
+	  memcmp(&buffer[6], "http://ns.adobe.com/xap/", 24)==0)
+      {
+	return 0;
+      }
+    }
+    /* Don't extract jpg inside AVI */
+    if( file_recovery->file_stat->file_hint==&file_hint_riff &&
+	(memcmp(buffer,  jpg_header_app0_avi, sizeof(jpg_header_app0_avi))==0 ||
+	 file_recovery->data_check == &data_check_avi_stream))
+      return 0;
+    /* Don't extract jpg inside MOV */
+    if( file_recovery->file_stat->file_hint==&file_hint_mov &&
+	memcmp(buffer,  jpg_header_app0_jfif11_null, sizeof(jpg_header_app0_jfif11_null))==0)
+      return 0;
   }
-  /* Don't recover the thumb instead of the jpg itself */
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_jpg &&
-      file_recovery->file_size <= 4096 &&
-      memcmp(buffer, jpg_header_app0, sizeof(jpg_header_app0))==0 &&
-      width>0 && width<200 && height>0 && height<200)
-  {
-    return 0;
-  }
-  /* Some JPG have two APP1 markers, avoid to dicard the first one */
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_jpg &&
-      memcmp(buffer, jpg_header_app1, sizeof(jpg_header_app1))==0 &&
-      memcmp(&buffer[6], "http://ns.adobe.com/xap/", 24)==0)
-  {
-    return 0;
-  }
-
-  /* Don't extract jpg inside AVI */
-  if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      file_recovery->file_stat->file_hint==&file_hint_riff &&
-      (memcmp(buffer,  jpg_header_app0_avi, sizeof(jpg_header_app0_avi))==0 ||
-       file_recovery->data_check == &data_check_avi_stream))
-    return 0;
   if(buffer[0]==0xff && buffer[1]==0xd8)
   {
+    int mpo=0;
     unsigned int i=2;
     time_t jpg_time=0;
-    while(i<6*512 && i+4<buffer_size)
+    while(i+4<buffer_size && buffer[i]==0xff)
     {
-      if(buffer[i]!=0xff)
-	return 0;
       /* 0x00 */
       /* 0x01 */
       /* 0xc4 Define Huffman table */
@@ -188,6 +333,16 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
 	  jpg_time=get_date_from_tiff_header((const TIFFHeader*)&buffer[i+0x0A], tiff_size);
 	}
       }
+      else if(buffer[i+1]==0xe2)
+      {
+	/* APP2 Exif information */
+	if(i+8 < buffer_size &&
+	    buffer[i+4]=='M' && buffer[i+5]=='P' && buffer[i+6]=='F' && buffer[i+7]==0)
+	{
+	  /* Multi-picture format */
+	  mpo=1;
+	}
+      }
       else if((buffer[i+1]>=0xe0 && buffer[i+1]<=0xef) ||
 	 buffer[i+1]==0xfe ||
 	 buffer[i+1]==0xdb)
@@ -196,24 +351,42 @@ static int header_check_jpg(const unsigned char *buffer, const unsigned int buff
       else
       {
 	reset_file_recovery(file_recovery_new);
-	file_recovery_new->extension=file_hint_jpg.extension;
-	file_recovery_new->file_check=&file_check_jpg;
 	file_recovery_new->min_filesize=(i>288?i:288);
-	file_recovery_new->data_check=&data_check_jpg;
 	file_recovery_new->calculated_file_size=2;
 	file_recovery_new->time=jpg_time;
+	if(mpo > 0)
+	{
+	  file_recovery_new->extension="jpg";
+	  file_recovery_new->file_check=&file_check_mpo;
+	}
+	else
+	{
+	  file_recovery_new->extension=file_hint_jpg.extension;
+	  file_recovery_new->file_check=&file_check_jpg;
+	  file_recovery_new->data_check=&data_check_jpg;
+	}
 	return 1;
       }
       i+=2+(buffer[i+2]<<8)+buffer[i+3];
     }
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_jpg.extension;
-    file_recovery_new->file_check=&file_check_jpg;
-    file_recovery_new->min_filesize=i;
-    file_recovery_new->data_check=&data_check_jpg;
-    file_recovery_new->calculated_file_size=2;
-    file_recovery_new->time=jpg_time;
-    return 1;
+    if(i >= 6*512)
+    {
+      reset_file_recovery(file_recovery_new);
+      file_recovery_new->min_filesize=i;
+      file_recovery_new->calculated_file_size=2;
+      file_recovery_new->time=jpg_time;
+      if(mpo > 0)
+      {
+	file_recovery_new->extension="mpo";
+      }
+      else
+      {
+	file_recovery_new->extension=file_hint_jpg.extension;
+	file_recovery_new->file_check=&file_check_jpg;
+	file_recovery_new->data_check=&data_check_jpg;
+      }
+      return 1;
+    }
   }
   return 0;
 }
@@ -810,13 +983,12 @@ static uint64_t jpg_find_error(FILE *handle, const unsigned int output_scanline,
 	if(stop==1
 	    && is_line_cut(output_scanline, output_width, output_components, frame, y))
 	{
-	  uint64_t offset_rel1,offset_rel2;
+	  const uint64_t offset_rel1=offsets[result_y / 8];
+	  const uint64_t offset_rel2=offsets[result_y / 8 + 1];
 #ifdef DEBUG_JPEG
 	  log_info("x %4u, y %4u: %6u, result=%u, output_scanline_max=%u\n",
 	      result_x, result_y, result_max, result, output_scanline_max);
 #endif
-	  offset_rel1=offsets[result_y / 8];
-	  offset_rel2=offsets[result_y / 8 + 1];
 	  if(offset_rel1 < offset_rel2)
 	    return jpg_xy_to_offset(handle, result_x, result_y,
 //		offset_rel1, offset_rel2, offset, blocksize);
@@ -938,8 +1110,7 @@ static void jpg_check_picture(file_recovery_t *file_recovery)
 #if 1
     if(jpeg_session.frame!=NULL && jpeg_session.flags!=0)
     {
-      uint64_t offset_error;
-      offset_error=jpg_find_error(jpeg_session.handle, jpeg_session.cinfo.output_scanline, jpeg_session.output_width, jpeg_session.output_components, jpeg_session.frame, &offsets[0], jpeg_session.offset, jpeg_session.blocksize, file_recovery->checkpoint_offset);
+      const uint64_t offset_error=jpg_find_error(jpeg_session.handle, jpeg_session.cinfo.output_scanline, jpeg_session.output_width, jpeg_session.output_components, jpeg_session.frame, &offsets[0], jpeg_session.offset, jpeg_session.blocksize, file_recovery->checkpoint_offset);
       if(offset_error !=0 && file_recovery->offset_error > offset_error)
 	file_recovery->offset_error=offset_error;
 #ifdef DEBUG_JPEG
@@ -1107,7 +1278,8 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
     for(offset=file_recovery->blocksize; offset < nbytes && file_recovery->offset_error==0; offset+=file_recovery->blocksize)
     {
       if(buffer[offset]==0xff && buffer[offset+1]==0xd8 && buffer[offset+2]==0xff &&
-	(buffer[offset+3]==0xe1 || buffer[offset+3]==0xec))
+	((buffer[offset+3]==0xe1 && memcmp(&buffer[offset+6], "http://ns.adobe.com/xap/", 24)!=0)
+	 || buffer[offset+3]==0xec))
       {
 	file_recovery->offset_error=offset;
       }
@@ -1173,8 +1345,9 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 	    {
 	      unsigned int j=thumb_offset+2;
 	      unsigned int thumb_sos_found=0;
-	      unsigned int j_old;
-	      j_old=j;
+#ifdef DEBUG_JPEG
+	      unsigned int j_old=j;
+#endif
 	      if(buffer[thumb_offset]!=0xff)
 	      {
 		file_recovery->offset_error=thumb_offset;
@@ -1238,7 +1411,9 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 		}
 		if(file_recovery->offset_ok<j)
 		  file_recovery->offset_ok=j;
+#ifdef DEBUG_JPEG
 		j_old=j;
+#endif
 		j+=2+(buffer[j+2]<<8)+buffer[j+3];
 	      }
 	      if(thumb_sos_found>0 && extract_thumb>0
@@ -1259,6 +1434,8 @@ static uint64_t jpg_check_structure(file_recovery_t *file_recovery, const unsign
 		      log_error("Can't write to %s: %s\n", thumbname, strerror(errno));
 		    }
 		    fclose(out);
+		    if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
+		      set_date(thumbname, file_recovery->time, file_recovery->time);
 		  }
 		  else
 		  {
@@ -1493,7 +1670,13 @@ int data_check_jpg(const unsigned char *buffer, const unsigned int buffer_size,
 const char*td_jpeg_version(void)
 {
 #if defined(HAVE_LIBJPEG)
-#if defined(JPEG_LIB_VERSION)
+#ifdef LIBJPEG_TURBO_VERSION
+#define td_xstr(s) td_str(s)
+#define td_str(s) #s
+  static char buffer[32];
+  sprintf(buffer,"libjpeg-turbo-%s", "" td_xstr(LIBJPEG_TURBO_VERSION));
+  return buffer;
+#elif defined(JPEG_LIB_VERSION)
   static char buffer[32];
   sprintf(buffer,"%d", JPEG_LIB_VERSION);
   return buffer;
