@@ -19,6 +19,12 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-int64_t test_jpeg(FILE *infile);
+const char*td_jpeg_version(void);
 
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
