@@ -32,9 +32,12 @@ extern const file_hint_t file_hint_7z;
 extern const file_hint_t file_hint_DB;
 extern const file_hint_t file_hint_a;
 extern const file_hint_t file_hint_abcdp;
+extern const file_hint_t file_hint_abr;
+extern const file_hint_t file_hint_acb;
 extern const file_hint_t file_hint_accdb;
 extern const file_hint_t file_hint_ace;
 extern const file_hint_t file_hint_addressbook;
+extern const file_hint_t file_hint_ado;
 extern const file_hint_t file_hint_ahn;
 extern const file_hint_t file_hint_aif;
 extern const file_hint_t file_hint_all;
@@ -46,11 +49,14 @@ extern const file_hint_t file_hint_ape;
 extern const file_hint_t file_hint_apple;
 extern const file_hint_t file_hint_arj;
 extern const file_hint_t file_hint_asf;
+extern const file_hint_t file_hint_asl;
 extern const file_hint_t file_hint_asm;
 extern const file_hint_t file_hint_atd;
 extern const file_hint_t file_hint_au;
+extern const file_hint_t file_hint_axx;
 extern const file_hint_t file_hint_bac;
 extern const file_hint_t file_hint_bim;
+extern const file_hint_t file_hint_binvox;
 extern const file_hint_t file_hint_bkf;
 extern const file_hint_t file_hint_blend;
 extern const file_hint_t file_hint_bmp;
@@ -66,6 +72,7 @@ extern const file_hint_t file_hint_cm;
 extern const file_hint_t file_hint_compress;
 extern const file_hint_t file_hint_cow;
 extern const file_hint_t file_hint_crw;
+extern const file_hint_t file_hint_csh;
 extern const file_hint_t file_hint_ctg;
 extern const file_hint_t file_hint_cwk;
 extern const file_hint_t file_hint_d2s;
@@ -101,10 +108,12 @@ extern const file_hint_t file_hint_exs;
 extern const file_hint_t file_hint_ext2_sb;
 extern const file_hint_t file_hint_ext2_fs;
 extern const file_hint_t file_hint_fat;
+extern const file_hint_t file_hint_fbf;
 extern const file_hint_t file_hint_fbk;
 extern const file_hint_t file_hint_fcp;
 extern const file_hint_t file_hint_fcs;
 extern const file_hint_t file_hint_fdb;
+extern const file_hint_t file_hint_fds;
 extern const file_hint_t file_hint_fh10;
 extern const file_hint_t file_hint_fh5;
 extern const file_hint_t file_hint_filevault;
@@ -119,6 +128,7 @@ extern const file_hint_t file_hint_fp7;
 extern const file_hint_t file_hint_freeway;
 extern const file_hint_t file_hint_frm;
 extern const file_hint_t file_hint_fs;
+extern const file_hint_t file_hint_fwd;
 extern const file_hint_t file_hint_gam;
 extern const file_hint_t file_hint_gct;
 extern const file_hint_t file_hint_gho;
@@ -129,22 +139,28 @@ extern const file_hint_t file_hint_gpg;
 extern const file_hint_t file_hint_gz;
 extern const file_hint_t file_hint_hdf;
 extern const file_hint_t file_hint_hds;
+extern const file_hint_t file_hint_hfsp;
 extern const file_hint_t file_hint_hr9;
+extern const file_hint_t file_hint_http;
 extern const file_hint_t file_hint_icc;
 extern const file_hint_t file_hint_ico;
 extern const file_hint_t file_hint_ifo;
 extern const file_hint_t file_hint_imb;
 extern const file_hint_t file_hint_indd;
+extern const file_hint_t file_hint_info;
 extern const file_hint_t file_hint_iso;
 extern const file_hint_t file_hint_it;
 extern const file_hint_t file_hint_itunes;
 extern const file_hint_t file_hint_jpg;
 extern const file_hint_t file_hint_kdb;
+extern const file_hint_t file_hint_kdbx;
 extern const file_hint_t file_hint_ldf;
+extern const file_hint_t file_hint_lit;
 extern const file_hint_t file_hint_logic;
 extern const file_hint_t file_hint_lnk;
 extern const file_hint_t file_hint_lso;
 extern const file_hint_t file_hint_lxo;
+extern const file_hint_t file_hint_lzh;
 extern const file_hint_t file_hint_lzo;
 extern const file_hint_t file_hint_m2ts;
 extern const file_hint_t file_hint_mat;
@@ -155,9 +171,12 @@ extern const file_hint_t file_hint_mdb;
 extern const file_hint_t file_hint_mdf;
 extern const file_hint_t file_hint_mfa;
 extern const file_hint_t file_hint_mfg;
+extern const file_hint_t file_hint_mft;
 extern const file_hint_t file_hint_mid;
+extern const file_hint_t file_hint_mig;
 extern const file_hint_t file_hint_mk5;
 extern const file_hint_t file_hint_mkv;
+extern const file_hint_t file_hint_mobi;
 extern const file_hint_t file_hint_mov;
 extern const file_hint_t file_hint_mp3;
 extern const file_hint_t file_hint_mpg;
@@ -167,6 +186,7 @@ extern const file_hint_t file_hint_mxf;
 extern const file_hint_t file_hint_myo;
 extern const file_hint_t file_hint_mysql;
 extern const file_hint_t file_hint_nds;
+extern const file_hint_t file_hint_nes;
 extern const file_hint_t file_hint_njx;
 extern const file_hint_t file_hint_nk2;
 extern const file_hint_t file_hint_nsf;
@@ -183,11 +203,13 @@ extern const file_hint_t file_hint_pcx;
 extern const file_hint_t file_hint_pdf;
 extern const file_hint_t file_hint_pds;
 extern const file_hint_t file_hint_pfx;
+extern const file_hint_t file_hint_plt;
 extern const file_hint_t file_hint_png;
 extern const file_hint_t file_hint_pnm;
 extern const file_hint_t file_hint_prc;
 extern const file_hint_t file_hint_prt;
 extern const file_hint_t file_hint_ps;
+extern const file_hint_t file_hint_psb;
 extern const file_hint_t file_hint_psd;
 extern const file_hint_t file_hint_psf;
 extern const file_hint_t file_hint_psp;
@@ -215,8 +237,10 @@ extern const file_hint_t file_hint_rm;
 extern const file_hint_t file_hint_rns;
 extern const file_hint_t file_hint_rpm;
 extern const file_hint_t file_hint_rw2;
+extern const file_hint_t file_hint_rx2;
 extern const file_hint_t file_hint_save;
 extern const file_hint_t file_hint_ses;
+extern const file_hint_t file_hint_shn;
 extern const file_hint_t file_hint_sib;
 extern const file_hint_t file_hint_sig;
 extern const file_hint_t file_hint_sit;
@@ -238,17 +262,21 @@ extern const file_hint_t file_hint_tiff;
 extern const file_hint_t file_hint_tivo;
 extern const file_hint_t file_hint_torrent;
 extern const file_hint_t file_hint_tph;
+extern const file_hint_t file_hint_tpl;
 extern const file_hint_t file_hint_ts;
 extern const file_hint_t file_hint_ttf;
 extern const file_hint_t file_hint_txt;
 extern const file_hint_t file_hint_tz;
+extern const file_hint_t file_hint_v2i;
 extern const file_hint_t file_hint_vault;
 extern const file_hint_t file_hint_vdi;
 extern const file_hint_t file_hint_veg;
 extern const file_hint_t file_hint_vfb;
 extern const file_hint_t file_hint_vmdk;
 extern const file_hint_t file_hint_vmg;
+extern const file_hint_t file_hint_wallet;
 extern const file_hint_t file_hint_wdp;
+extern const file_hint_t file_hint_wim;
 extern const file_hint_t file_hint_win;
 extern const file_hint_t file_hint_wks;
 extern const file_hint_t file_hint_wmf;
@@ -264,6 +292,7 @@ extern const file_hint_t file_hint_xm;
 extern const file_hint_t file_hint_xsv;
 extern const file_hint_t file_hint_xpt;
 extern const file_hint_t file_hint_xv;
+extern const file_hint_t file_hint_xz;
 extern const file_hint_t file_hint_zip;
 
 file_enable_t list_file_enable[]=
@@ -274,9 +303,12 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_DB    },
   { .enable=0, .file_hint=&file_hint_a    },
   { .enable=0, .file_hint=&file_hint_abcdp},
+  { .enable=0, .file_hint=&file_hint_abr  },
+  { .enable=0, .file_hint=&file_hint_acb  },
   { .enable=0, .file_hint=&file_hint_accdb},
   { .enable=0, .file_hint=&file_hint_ace  },
   { .enable=0, .file_hint=&file_hint_addressbook},
+  { .enable=0, .file_hint=&file_hint_ado  },
   { .enable=0, .file_hint=&file_hint_ahn  },
   { .enable=0, .file_hint=&file_hint_aif  },
   { .enable=0, .file_hint=&file_hint_all  },
@@ -288,11 +320,14 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_apple },
   { .enable=0, .file_hint=&file_hint_arj  },
   { .enable=0, .file_hint=&file_hint_asf  },
+  { .enable=0, .file_hint=&file_hint_asl  },
   { .enable=0, .file_hint=&file_hint_asm  },
   { .enable=0, .file_hint=&file_hint_atd  },
   { .enable=0, .file_hint=&file_hint_au   },
+  { .enable=0, .file_hint=&file_hint_axx  },
   { .enable=0, .file_hint=&file_hint_bac  },
   { .enable=0, .file_hint=&file_hint_bim  },
+  { .enable=0, .file_hint=&file_hint_binvox  },
   { .enable=0, .file_hint=&file_hint_bkf  },
   { .enable=0, .file_hint=&file_hint_blend },
   { .enable=0, .file_hint=&file_hint_bmp  },
@@ -308,6 +343,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_compress },
   { .enable=0, .file_hint=&file_hint_cow  },
   { .enable=0, .file_hint=&file_hint_crw  },
+  { .enable=0, .file_hint=&file_hint_csh  },
   { .enable=0, .file_hint=&file_hint_ctg  },
   { .enable=0, .file_hint=&file_hint_cwk  },
   { .enable=0, .file_hint=&file_hint_d2s  },
@@ -343,10 +379,12 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_ext2_sb },
   { .enable=0, .file_hint=&file_hint_ext2_fs },
   { .enable=0, .file_hint=&file_hint_fat  },
+  { .enable=0, .file_hint=&file_hint_fbf  },
   { .enable=0, .file_hint=&file_hint_fbk  },
   { .enable=0, .file_hint=&file_hint_fcp  },
   { .enable=0, .file_hint=&file_hint_fcs  },
   { .enable=0, .file_hint=&file_hint_fdb  },
+  { .enable=0, .file_hint=&file_hint_fds  },
   { .enable=0, .file_hint=&file_hint_fh10  },
   { .enable=0, .file_hint=&file_hint_fh5  },
   { .enable=0, .file_hint=&file_hint_filevault },
@@ -360,6 +398,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_freeway  },
   { .enable=0, .file_hint=&file_hint_frm  },
   { .enable=0, .file_hint=&file_hint_fs   },
+  { .enable=0, .file_hint=&file_hint_fwd  },
   { .enable=0, .file_hint=&file_hint_gam  },
   { .enable=0, .file_hint=&file_hint_gct  },
   { .enable=0, .file_hint=&file_hint_gho  },
@@ -370,22 +409,28 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_gz   },
   { .enable=0, .file_hint=&file_hint_hdf  },
   { .enable=0, .file_hint=&file_hint_hds  },
+  { .enable=0, .file_hint=&file_hint_hfsp },
   { .enable=0, .file_hint=&file_hint_hr9  },
+  { .enable=0, .file_hint=&file_hint_http },
   { .enable=0, .file_hint=&file_hint_icc  },
   { .enable=0, .file_hint=&file_hint_ico  },
   { .enable=0, .file_hint=&file_hint_ifo  },
   { .enable=0, .file_hint=&file_hint_imb  },
   { .enable=0, .file_hint=&file_hint_indd  },
+  { .enable=0, .file_hint=&file_hint_info  },
   { .enable=0, .file_hint=&file_hint_iso  },
   { .enable=0, .file_hint=&file_hint_it  },
   { .enable=0, .file_hint=&file_hint_itunes  },
   { .enable=0, .file_hint=&file_hint_jpg  },
   { .enable=0, .file_hint=&file_hint_kdb  },
+  { .enable=0, .file_hint=&file_hint_kdbx },
   { .enable=0, .file_hint=&file_hint_ldf  },
+  { .enable=0, .file_hint=&file_hint_lit  },
   { .enable=0, .file_hint=&file_hint_logic},
   { .enable=0, .file_hint=&file_hint_lnk  },
   { .enable=0, .file_hint=&file_hint_lso  },
   { .enable=0, .file_hint=&file_hint_lxo  },
+  { .enable=0, .file_hint=&file_hint_lzh  },
   { .enable=0, .file_hint=&file_hint_lzo  },
   { .enable=0, .file_hint=&file_hint_m2ts },
   { .enable=0, .file_hint=&file_hint_mat  },
@@ -396,9 +441,12 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_mdf  },
   { .enable=0, .file_hint=&file_hint_mfa  },
   { .enable=0, .file_hint=&file_hint_mfg  },
+  { .enable=0, .file_hint=&file_hint_mft  },
   { .enable=0, .file_hint=&file_hint_mid  },
+  { .enable=0, .file_hint=&file_hint_mig  },
   { .enable=0, .file_hint=&file_hint_mk5  },
   { .enable=0, .file_hint=&file_hint_mkv  },
+  { .enable=0, .file_hint=&file_hint_mobi },
   { .enable=0, .file_hint=&file_hint_mov  },
   { .enable=0, .file_hint=&file_hint_mp3  },
   { .enable=0, .file_hint=&file_hint_mpg  },
@@ -408,6 +456,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_mysql },
   { .enable=0, .file_hint=&file_hint_mxf  },
   { .enable=0, .file_hint=&file_hint_nds  },
+  { .enable=0, .file_hint=&file_hint_nes  },
   { .enable=0, .file_hint=&file_hint_njx  },
   { .enable=0, .file_hint=&file_hint_nk2  },
   { .enable=0, .file_hint=&file_hint_nsf  },
@@ -424,11 +473,13 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_pdf  },
   { .enable=0, .file_hint=&file_hint_pds  },
   { .enable=0, .file_hint=&file_hint_pfx  },
+  { .enable=0, .file_hint=&file_hint_plt  },
   { .enable=0, .file_hint=&file_hint_png  },
   { .enable=0, .file_hint=&file_hint_pnm  },
   { .enable=0, .file_hint=&file_hint_prc  },
   { .enable=0, .file_hint=&file_hint_prt  },
   { .enable=0, .file_hint=&file_hint_ps   },
+  { .enable=0, .file_hint=&file_hint_psb  },
   { .enable=0, .file_hint=&file_hint_psd  },
   { .enable=0, .file_hint=&file_hint_psf  },
   { .enable=0, .file_hint=&file_hint_psp  },
@@ -456,8 +507,10 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_rns  },
   { .enable=0, .file_hint=&file_hint_rpm  },
   { .enable=0, .file_hint=&file_hint_rw2  },
+  { .enable=0, .file_hint=&file_hint_rx2  },
   { .enable=0, .file_hint=&file_hint_save  },
   { .enable=0, .file_hint=&file_hint_ses  },
+  { .enable=0, .file_hint=&file_hint_shn  },
   { .enable=0, .file_hint=&file_hint_sib  },
   { .enable=0, .file_hint=&file_hint_sit  },
   { .enable=0, .file_hint=&file_hint_skd  },
@@ -478,18 +531,22 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_tivo  },
   { .enable=0, .file_hint=&file_hint_torrent  },
   { .enable=0, .file_hint=&file_hint_tph  },
+  { .enable=0, .file_hint=&file_hint_tpl  },
   { .enable=0, .file_hint=&file_hint_ts   },
   { .enable=0, .file_hint=&file_hint_ttf  },
   { .enable=0, .file_hint=&file_hint_fasttxt  },
   { .enable=0, .file_hint=&file_hint_txt  },
   { .enable=0, .file_hint=&file_hint_tz   },
+  { .enable=0, .file_hint=&file_hint_v2i  },
   { .enable=0, .file_hint=&file_hint_vault  },
   { .enable=0, .file_hint=&file_hint_vfb  },
   { .enable=0, .file_hint=&file_hint_vdi  },
   { .enable=0, .file_hint=&file_hint_veg  },
   { .enable=0, .file_hint=&file_hint_vmdk },
   { .enable=0, .file_hint=&file_hint_vmg  },
+  { .enable=0, .file_hint=&file_hint_wallet  },
   { .enable=0, .file_hint=&file_hint_wdp  },
+  { .enable=0, .file_hint=&file_hint_wim  },
   { .enable=0, .file_hint=&file_hint_win  },
   { .enable=0, .file_hint=&file_hint_wks  },
   { .enable=0, .file_hint=&file_hint_wmf  },
@@ -505,6 +562,7 @@ file_enable_t list_file_enable[]=
   { .enable=0, .file_hint=&file_hint_xsv  },
   { .enable=0, .file_hint=&file_hint_xpt  },
   { .enable=0, .file_hint=&file_hint_xv   },
+  { .enable=0, .file_hint=&file_hint_xz   },
   { .enable=0, .file_hint=&file_hint_zip  },
   { .enable=0, .file_hint=NULL }
 };
