@@ -35,9 +35,7 @@
 extern const file_hint_t file_hint_mkv;
 
 static void register_header_check_mp3(file_stat_t *file_stat);
-static int header_check_id3(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static int data_check_id3(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
-static int header_check_mp3(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery);
 static unsigned int pos_in_mem(const unsigned char *haystack, const unsigned int haystack_size, const unsigned char *needle, const unsigned int needle_size);
 static unsigned int search_MMT(const unsigned char *buffer, const unsigned int i, const unsigned int buffer_size);
@@ -52,14 +50,6 @@ const file_hint_t file_hint_mp3= {
   .register_header_check=&register_header_check_mp3
 };
 
-static const unsigned char id3_header[3]= {'I','D','3'};
-static const unsigned char mp3_header[1]= {0xFF};
-
-static void register_header_check_mp3(file_stat_t *file_stat)
-{
-  register_header_check(0, id3_header,sizeof(id3_header), &header_check_id3, file_stat);
-  register_header_check(0, mp3_header,sizeof(mp3_header), &header_check_mp3, file_stat);
-}
 
 #define MPEG_V25	0
 #define MPEG_V2		0x2
@@ -169,13 +159,13 @@ static int header_check_mp3(const unsigned char *buffer, const unsigned int buff
 	http://www.dv.co.yu/mpgscript/mpeghdr.htm
   */
   if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
-      ((potential_frame_offset==0 && file_recovery->file_stat->file_hint==&file_hint_mp3)||
+      (file_recovery->file_stat->file_hint==&file_hint_mp3 ||
        file_recovery->file_stat->file_hint==&file_hint_mkv))
     return 0;
-  if(!(buffer[potential_frame_offset+0]==0xFF &&
-	((buffer[potential_frame_offset+1]&0xFE)==0xFA ||
-	 (buffer[potential_frame_offset+1]&0xFE)==0xF2 ||
-	 (buffer[potential_frame_offset+1]&0xFE)==0xE2)))
+  if(!(buffer[0]==0xFF &&
+	((buffer[1]&0xFE)==0xFA ||
+	 (buffer[1]&0xFE)==0xF2 ||
+	 (buffer[1]&0xFE)==0xE2)))
     return 0;
   while(potential_frame_offset+1 < buffer_size &&
       potential_frame_offset+1 < 2048)
@@ -183,33 +173,30 @@ static int header_check_mp3(const unsigned char *buffer, const unsigned int buff
     if(buffer[potential_frame_offset+0]!=0xFF)
       return 0;
     {
-      unsigned int mpeg_version	=(buffer[potential_frame_offset+1]>>3)&0x03;
-      unsigned int mpeg_layer	=(buffer[potential_frame_offset+1]>>1)&0x03;
-      unsigned int bit_rate_key	=(buffer[potential_frame_offset+2]>>4)&0x0F;
-      unsigned int sampling_rate_key=(buffer[potential_frame_offset+2]>>2)&0x03;
-      unsigned int padding		=(buffer[potential_frame_offset+2]>>1)&0x01;
-      unsigned int bit_rate= bit_rate_table[mpeg_version][mpeg_layer][bit_rate_key]*1000;
-      unsigned int sample_rate=sample_rate_table[mpeg_version][sampling_rate_key];
+      const unsigned int mpeg_version	=(buffer[potential_frame_offset+1]>>3)&0x03;
+      const unsigned int mpeg_layer	=(buffer[potential_frame_offset+1]>>1)&0x03;
+      const unsigned int bit_rate_key	=(buffer[potential_frame_offset+2]>>4)&0x0F;
+      const unsigned int sampling_rate_key=(buffer[potential_frame_offset+2]>>2)&0x03;
+      const unsigned int padding	=(buffer[potential_frame_offset+2]>>1)&0x01;
+      const unsigned int bit_rate	=bit_rate_table[mpeg_version][mpeg_layer][bit_rate_key];
+      const unsigned int sample_rate	=sample_rate_table[mpeg_version][sampling_rate_key];
       unsigned int frameLengthInBytes=0;
-      if(sample_rate>0 && bit_rate>0)
+      if(sample_rate==0 || bit_rate==0 || mpeg_layer==MPEG_L1)
+	return 0;
+      if(mpeg_layer==MPEG_L3)
       {
-	if(mpeg_layer==MPEG_L3)
-	{
-	  if(mpeg_version==MPEG_V1)
-	    frameLengthInBytes = 144 * bit_rate / sample_rate + padding;
-	  else
-	    frameLengthInBytes = 72 * bit_rate / sample_rate + padding;
-	}
-	else if(mpeg_layer==MPEG_L2)
-	  frameLengthInBytes = 144 * bit_rate / sample_rate + padding;
+	if(mpeg_version==MPEG_V1)
+	  frameLengthInBytes = 144000 * bit_rate / sample_rate + padding;
 	else
-	  frameLengthInBytes = (12 * bit_rate / sample_rate + padding)*4;
+	  frameLengthInBytes = 72000 * bit_rate / sample_rate + padding;
       }
+      else if(mpeg_layer==MPEG_L2)
+	frameLengthInBytes = 144000 * bit_rate / sample_rate + padding;
+      else
+	frameLengthInBytes = (12000 * bit_rate / sample_rate + padding)*4;
 #ifdef DEBUG_MP3
-      log_info("bit_rate=%u\n",bit_rate);
-      log_info("sample_rate=%u\n",sample_rate);
-      log_info("padding=%u\n", padding);
-      log_info("frameLengthInBytes=%u\n",frameLengthInBytes);
+      log_info("framesize: %u, layer: %u, bitrate: %u, padding: %u\n",
+	  frameLengthInBytes, 4-mpeg_layer, bit_rate, padding);
 #endif
       if(frameLengthInBytes==0)
 	return 0;
@@ -225,9 +212,12 @@ static int header_check_mp3(const unsigned char *buffer, const unsigned int buff
     reset_file_recovery(file_recovery_new);
     file_recovery_new->calculated_file_size=potential_frame_offset;
     file_recovery_new->min_filesize=287;
-    file_recovery_new->data_check=&data_check_mp3;
     file_recovery_new->extension=file_hint_mp3.extension;
-    file_recovery_new->file_check=&file_check_size;
+    if(file_recovery_new->blocksize >= 16/2)
+    {
+      file_recovery_new->data_check=&data_check_mp3;
+      file_recovery_new->file_check=&file_check_size;
+    }
     return 1;
   }
   return 0;
@@ -238,7 +228,7 @@ static int data_check_id3(const unsigned char *buffer, const unsigned int buffer
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 1 < file_recovery->file_size + buffer_size/2)
   {
-    unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
+    const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
     if(buffer[i]==0)
     { /* Padding is present */
       file_recovery->calculated_file_size++;
@@ -255,26 +245,30 @@ static int data_check_id3(const unsigned char *buffer, const unsigned int buffer
 
 static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
+#ifdef DEBUG_MP3
+  log_info("data_check_mp3  file_size=%llu, calculated_file_size=%llu\n",
+      (long long unsigned)file_recovery->file_size,
+      (long long unsigned)file_recovery->calculated_file_size);
+#endif
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 16 < file_recovery->file_size + buffer_size/2)
   {
-    unsigned int MMT_size = 0;
-    unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
-    /*
-    log_trace("data_check_mp3 start i=0x%x buffer_size=0x%x calculated_file_size=%lu file_size=%lu\n",
+    const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
+#ifdef DEBUG_MP3
+    log_info("data_check_mp3 start i=0x%x buffer_size=0x%x calculated_file_size=%lu file_size=%lu\n",
 	i, buffer_size,
 	(long unsigned)file_recovery->calculated_file_size,
 	(long unsigned)file_recovery->file_size);
-    */
+#endif
     if(buffer[i+0]==0xFF && ((buffer[i+1]&0xE0)==0xE0))
     {
-      unsigned int mpeg_version	=(buffer[i+1]>>3)&0x03;
-      unsigned int mpeg_layer	=(buffer[i+1]>>1)&0x03;
-      unsigned int bit_rate_key	=(buffer[i+2]>>4)&0x0F;
-      unsigned int sampling_rate_key=(buffer[i+2]>>2)&0x03;
-      unsigned int padding		=(buffer[i+2]>>1)&0x01;
-      unsigned int bit_rate= bit_rate_table[mpeg_version][mpeg_layer][bit_rate_key]*1000;
-      unsigned int sample_rate=sample_rate_table[mpeg_version][sampling_rate_key];
+      const unsigned int mpeg_version	=(buffer[i+1]>>3)&0x03;
+      const unsigned int mpeg_layer	=(buffer[i+1]>>1)&0x03;
+      const unsigned int bit_rate_key	=(buffer[i+2]>>4)&0x0F;
+      const unsigned int sampling_rate_key=(buffer[i+2]>>2)&0x03;
+      const unsigned int padding	=(buffer[i+2]>>1)&0x01;
+      const unsigned int bit_rate	=bit_rate_table[mpeg_version][mpeg_layer][bit_rate_key];
+      const unsigned int sample_rate	=sample_rate_table[mpeg_version][sampling_rate_key];
       unsigned int frameLengthInBytes=0;
       /*
       log_info("frame_offset=%u\n",i);
@@ -282,24 +276,21 @@ static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer
       log_info("sample_rate=%u\n",sample_rate);
       log_info("frameLengthInBytes=%u\n",frameLengthInBytes);
       */
-      if(sample_rate>0 && bit_rate>0)
-      {
-	if(mpeg_layer==MPEG_L3)
-	{
-	  if(mpeg_version==MPEG_V1)
-	    frameLengthInBytes = 144 * bit_rate / sample_rate + padding;
-	  else
-	    frameLengthInBytes = 72 * bit_rate / sample_rate + padding;
-	}
-	else if(mpeg_layer==MPEG_L2)
-	  frameLengthInBytes = 144 * bit_rate / sample_rate + padding;
-	else
-	  frameLengthInBytes = (12 * bit_rate / sample_rate + padding)*4;
-      }  
-      if(frameLengthInBytes<3)
-      {
+      if(sample_rate==0 || bit_rate==0 || mpeg_layer==MPEG_L1)
 	return 2;
+      if(mpeg_layer==MPEG_L3)
+      {
+	if(mpeg_version==MPEG_V1)
+	  frameLengthInBytes = 144000 * bit_rate / sample_rate + padding;
+	else
+	  frameLengthInBytes = 72000 * bit_rate / sample_rate + padding;
       }
+      else if(mpeg_layer==MPEG_L2)
+	frameLengthInBytes = 144000 * bit_rate / sample_rate + padding;
+      else
+	frameLengthInBytes = (12000 * bit_rate / sample_rate + padding)*4;
+      if(frameLengthInBytes<3)
+	return 2;
       file_recovery->calculated_file_size+=frameLengthInBytes;
     }
     else if(buffer[i]=='L' && buffer[i+1]=='Y' && buffer[i+2]=='R' && buffer[i+3]=='I' && buffer[i+4]=='C' && buffer[i+5]=='S' && buffer[i+6]=='B' && buffer[i+7]=='E' && buffer[i+8]=='G' && buffer[i+9]=='I' && buffer[i+10]=='N')
@@ -318,11 +309,11 @@ static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer
       /* FIXME */
       if(file_recovery->calculated_file_size + 5100 >= file_recovery->file_size + buffer_size/2)
 	return 1;
-      if((pos_lyrics=pos_in_mem(&buffer[i],4096,"LYRICS200",9)) != 0)
+      if((pos_lyrics=pos_in_mem(&buffer[i], 4096, (const unsigned char*)"LYRICS200", 9)) != 0)
       {
 	file_recovery->calculated_file_size+=pos_lyrics;
       }
-      else if((pos_lyrics=pos_in_mem(&buffer[i],5100,"LYRICSEND",9)) != 0)
+      else if((pos_lyrics=pos_in_mem(&buffer[i], 5100, (const unsigned char*)"LYRICSEND", 9)) != 0)
       {
 	file_recovery->calculated_file_size+=pos_lyrics;
       }
@@ -336,7 +327,7 @@ static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer
     }
     else if(buffer[i]=='A' && buffer[i+1]=='P' && buffer[i+2]=='E' && buffer[i+3]=='T' && buffer[i+4]=='A' && buffer[i+5]=='G' && buffer[i+6]=='E' && buffer[i+7]=='X')
     { /* APE Tagv2 (APE Tagv1 has no header) http://wiki.hydrogenaudio.org/index.php?title=APE_Tags_Header */
-      unsigned int ape_tag_size = (buffer[i+12] + (buffer[i+13]<<8) + (buffer[i+14]<<16) + (buffer[i+15]<<24))+32;
+      const unsigned int ape_tag_size = (buffer[i+12] + (buffer[i+13]<<8) + (buffer[i+14]<<16) + (buffer[i+15]<<24))+32;
       file_recovery->calculated_file_size+=ape_tag_size;
     }
     else if(buffer[i]=='T' && buffer[i+1]=='A' && buffer[i+2]=='G')	
@@ -347,17 +338,16 @@ static int data_check_mp3(const unsigned char *buffer, const unsigned int buffer
     {
       return 1;
     }
-    else if((MMT_size=search_MMT(buffer,i,buffer_size))!=0)
+    else
     {
+      const unsigned int MMT_size=search_MMT(buffer,i,buffer_size);
+      if(MMT_size==0)
+	return 2;
       /*
 	 log_info("MusicMatch Tag found at offset 0x%x with size 0x%x \n", file_recovery->calculated_file_size, MMT_size);
-       */
+	 */
       file_recovery->calculated_file_size+=MMT_size;
     }
-    else 
-    {
-      return 2;
-    }
   }
   return 1;
 }
@@ -462,3 +452,20 @@ static unsigned int search_MMT(const unsigned char *buffer, const unsigned int i
   /* log_trace("search_MMT: MMT found size=%u (0x%x)\n", size, size); */
   return(size);
 }
+
+static void register_header_check_mp3(file_stat_t *file_stat)
+{
+  static const unsigned char mpeg1_L3_header1[2]= {0xFF, 0xFA};
+  static const unsigned char mpeg1_L3_header2[2]= {0xFF, 0xFB};
+  static const unsigned char mpeg2_L3_header1[2]= {0xFF, 0xF2};
+  static const unsigned char mpeg2_L3_header2[2]= {0xFF, 0xF3};
+  static const unsigned char mpeg25_L3_header1[2]={0xFF, 0xE2};
+  static const unsigned char mpeg25_L3_header2[2]={0xFF, 0xE3};
+  register_header_check(0, "ID3", 3, &header_check_id3, file_stat);
+  register_header_check(0, mpeg1_L3_header1, sizeof(mpeg1_L3_header1), &header_check_mp3, file_stat);
+  register_header_check(0, mpeg1_L3_header2, sizeof(mpeg1_L3_header2), &header_check_mp3, file_stat);
+  register_header_check(0, mpeg2_L3_header1, sizeof(mpeg2_L3_header1), &header_check_mp3, file_stat);
+  register_header_check(0, mpeg2_L3_header2, sizeof(mpeg2_L3_header2), &header_check_mp3, file_stat);
+  register_header_check(0, mpeg25_L3_header1, sizeof(mpeg25_L3_header1), &header_check_mp3, file_stat);
+  register_header_check(0, mpeg25_L3_header2, sizeof(mpeg25_L3_header2), &header_check_mp3, file_stat);
+}
