@@ -45,7 +45,7 @@ static void register_header_check_sig(file_stat_t *file_stat);
 static int header_check_sig(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_sig= {
-  .extension="",
+  .extension="custom",
   .description="Own custom signatures",
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
@@ -124,13 +124,11 @@ static FILE *open_signature_file(void)
 #endif
 #ifndef DJGPP
   {
-    char *home;
-    home = getenv("HOME");
+    const char *home = getenv("HOME");
     if (home != NULL)
     {
       FILE*handle;
-      char *filename=NULL;
-      filename=(char*)MALLOC(strlen(home)+strlen(DOT_PHOTOREC_SIG)+1);
+      char *filename=(char*)MALLOC(strlen(home)+strlen(DOT_PHOTOREC_SIG)+1);
       strcpy(filename, home);
       strcat(filename, DOT_PHOTOREC_SIG);
       handle=fopen(filename,"rb");
@@ -145,8 +143,7 @@ static FILE *open_signature_file(void)
   }
 #endif
   {
-    FILE*handle=NULL;
-    handle=fopen(PHOTOREC_SIG,"rb");
+    FILE *handle=fopen(PHOTOREC_SIG,"rb");
     if(handle!=NULL)
     {
       log_info("Open signature file %s\n", PHOTOREC_SIG);
@@ -334,8 +331,7 @@ static char *parse_signature_file(file_stat_t *file_stat, char *pos)
 	  pos+=2;
 	  while(isxdigit(*pos) && isxdigit(*(pos+1)))
 	  {
-	    unsigned int val=0;
-	    val=(*pos);
+	    unsigned int val=(*pos);
 	    if(*pos>='0' && *pos<='9')
 	      val-='0';
 	    else if(*pos>='A' && *pos<='F')
