@@ -42,6 +42,7 @@ const file_hint_t file_hint_xm= {
   .min_header_distance=0,
   .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .recover=1,
+  .enable_by_default=1,
   .register_header_check=&register_header_check_xm
 };
 
@@ -147,6 +148,7 @@ static void file_check_xm(file_recovery_t *fr)
 
   fr->file_size = 0;
   fr->offset_error=0;
+  fr->offset_ok=0;
 
   if (fseek(fr->handle, 70, SEEK_SET) == -1)
     return;
