@@ -85,7 +85,8 @@ struct zip_file_entry {
   uint16_t unused1:2;               /** Unused */
 
   uint16_t compression;             /** Compression method */
-  uint32_t last_mod;                /** Last moditication file time */
+  uint16_t last_mod_time;           /** Last moditication file time */
+  uint16_t last_mod_date;           /** Last moditication file date */
   uint32_t crc32;                   /** CRC32 */
   uint32_t compressed_size;         /** Compressed size */
   uint32_t uncompressed_size;       /** Uncompressed size */
@@ -94,6 +95,17 @@ struct zip_file_entry {
 } __attribute__ ((__packed__));
 typedef struct zip_file_entry zip_file_entry_t;
 
+struct zip64_extra_entry
+{
+  uint16_t tag;
+  uint16_t size;
+  uint64_t uncompressed_size;
+  uint64_t compressed_size;
+  uint64_t offset;		/* Offset of local header record */
+  uint32_t disk_start_number;	/* Number of the disk on which this file starts  */
+} __attribute__ ((__packed__));
+typedef struct zip64_extra_entry zip64_extra_entry_t;
+
 static uint32_t expected_compressed_size=0;
 
 static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size)
@@ -137,8 +149,8 @@ static int64_t file_get_pos(FILE *f, const void* needle, const unsigned int size
 static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const unsigned int file_nbr)
 {
   zip_file_entry_t  file;
-  uint32_t          len;
-
+  zip64_extra_entry_t extra;
+  uint64_t          len;
   if (fread(&file, sizeof(file), 1, fr->handle) != 1)
   {
 #ifdef DEBUG_ZIP
@@ -154,6 +166,12 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
       le32(file.uncompressed_size),
       le32(file.crc32));
 #endif
+  {
+    /* Use the more recent file to set the time/date of the recovered archive */
+    const time_t tmp=date_dos2unix(le16(file.last_mod_time), le16(file.last_mod_date));
+    if(fr->time < tmp)
+      fr->time=tmp;
+  }
   len = le16(file.filename_length);
   if (len)
   {
@@ -161,7 +179,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
     if (fread(filename, len, 1, fr->handle) != 1)
     {
 #ifdef DEBUG_ZIP
-      log_trace("zip: Unexpected EOF in file_entry header: %u bytes expected\n", len);
+      log_trace("zip: Unexpected EOF in file_entry header: %lu bytes expected\n", len);
 #endif
       free(filename);
       return -1;
@@ -228,6 +246,9 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	}
 	else if(len==19 && memcmp(filename, "[Content_Types].xml", 19)==0)
 	  msoffice=1;
+	/* Zipped Keyhole Markup Language (KML) used by Google Earth */
+	else if(len==7 && memcmp(filename, "doc.kml", 7)==0)
+	  *ext="kmz";
 	else if(len==4 && memcmp(filename, "Home", 4)==0)
 	  sh3d=1;
       }
@@ -244,14 +265,21 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
 	  *ext="xlsx";
 	else if(strncmp(filename, "ppt/", 4)==0)
 	  *ext="pptx";
+	else
+	  *ext="docx";
       }
     }
     if(*ext==NULL)
     {
-      if(len==20 && strcasecmp(filename, "META-INF/MANIFEST.MF")==0)
+	/* iWork */
+      if(len==23 && memcmp(filename, "QuickLook/Thumbnail.jpg", 23)==0)
+	*ext="pages";
+      else if(len==20 && strcasecmp(filename, "META-INF/MANIFEST.MF")==0)
 	*ext="jar";
       else if(len==15 && strcasecmp(filename, "chrome.manifest")==0)
 	*ext="xpi";
+      else if(len==30 && memcmp(filename, "xsd/MindManagerApplication.xsd", 30)==0)
+	*ext="mmap";
     }
     free(filename);
   }
@@ -259,29 +287,41 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
   log_info("\n");
 #endif
   len = le16(file.extra_length);
+  memset(&extra, 0, sizeof(extra));
   if (len>0)
   {
-    if (fseek(fr->handle, len, SEEK_CUR) == -1)
+    if (fread(&extra, sizeof(extra), 1, fr->handle) != 1)
     {
 #ifdef DEBUG_ZIP
-      log_trace("zip: Unexpected EOF in file_entry header: %u bytes expected\n", len);
+      log_trace("zip: Unexpected EOF in file_entry header: %lu bytes expected\n", len);
+#endif
+    }
+    if (fseek(fr->handle, fr->file_size, SEEK_SET) == -1 ||
+	fseek(fr->handle, len, SEEK_CUR) == -1)
+    {
+#ifdef DEBUG_ZIP
+      log_trace("zip: Unexpected EOF in file_entry header: %lu bytes expected\n", len);
 #endif
       return -1;
     }
     fr->file_size += len;
   }
   len = le32(file.compressed_size);
+  if(len==0xffffffff && le16(extra.tag)==1)
+  {
+    len = le64(extra.compressed_size);
+  }
   if (len>0)
   {
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
-      log_trace("zip: Unexpected EOF in file_entry data: %u bytes expected\n", len);
+      log_trace("zip: Unexpected EOF in file_entry data: %lu bytes expected\n", len);
 #endif
       return -1;
     }
 #ifdef DEBUG_ZIP
-    log_trace("zip: Data of length %u\n", len);
+    log_trace("zip: Data of length %lu\n", len);
 #endif
     fr->file_size += len;
   }
@@ -296,7 +336,7 @@ static int zip_parse_file_entry(file_recovery_t *fr, const char **ext, const uns
        Typically used in OOO documents
        Search ZIP_DATA_DESCRIPTOR */
     static const unsigned char zip_data_desc_header[4]= {0x50, 0x4B, 0x07, 0x08};
-    int64_t pos = file_get_pos(fr->handle, zip_data_desc_header, 4);
+    const int64_t pos = file_get_pos(fr->handle, zip_data_desc_header, 4);
 #ifdef DEBUG_ZIP
     log_trace("Searched footer, got length %lli\n", (long long int)pos);
 #endif
@@ -395,7 +435,7 @@ static int zip64_parse_end_central_dir(file_recovery_t *fr)
 
   if (dir.end_size > 0)
   {
-    uint64_t len = le64(dir.end_size);
+    const uint64_t len = le64(dir.end_size);
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -435,7 +475,7 @@ static int zip_parse_end_central_dir(file_recovery_t *fr)
 
   if (dir.comment_length)
   {
-    uint16_t len = le16(dir.comment_length);
+    const uint16_t len = le16(dir.comment_length);
     if (fseek(fr->handle, len, SEEK_CUR) == -1)
     {
 #ifdef DEBUG_ZIP
@@ -731,6 +771,12 @@ static int header_check_zip(const unsigned char *buffer, const unsigned int buff
 	 strcmp(file_recovery->extension,"psmodel")==0)
 	&& memcmp(&buffer[30], "macrolog_1.mac", 14)==0)
       return 0;
+    /* A zip file begins by ZIP_FILE_ENTRY, this signature can also be
+     * found for each compressed file */
+    if(file_recovery!=NULL && file_recovery->file_stat!=NULL &&
+	file_recovery->file_stat->file_hint==&file_hint_zip &&
+	safe_header_only==0)
+      return 0;
     reset_file_recovery(file_recovery_new);
     file_recovery_new->min_filesize=21;
     file_recovery_new->file_check=&file_check_zip;
