@@ -48,8 +48,8 @@ static unsigned int index_header_check(void);
 
 static int file_check_cmp(const struct td_list_head *a, const struct td_list_head *b)
 {
-  const file_check_t *fc_a=td_list_entry(a, const file_check_t, list);
-  const file_check_t *fc_b=td_list_entry(b, const file_check_t, list);
+  const file_check_t *fc_a=td_list_entry_const(a, const file_check_t, list);
+  const file_check_t *fc_b=td_list_entry_const(b, const file_check_t, list);
   int res;
   if(fc_a->length==0 && fc_b->length!=0)
     return -1;
@@ -74,12 +74,11 @@ static void file_check_add_tail(file_check_t *file_check_new, file_check_list_t
     newe->file_checks[i].list.prev=&newe->file_checks[i].list;
     newe->file_checks[i].list.next=&newe->file_checks[i].list;
   }
-  td_list_add_tail(&file_check_new->list, &newe->file_checks[file_check_new->length==0?0:file_check_new->value[0]].list);
+  td_list_add_tail(&file_check_new->list, &newe->file_checks[file_check_new->length==0?0:((const unsigned char *)file_check_new->value)[0]].list);
   td_list_add_tail(&newe->list, &pos->list);
 }
 
-void register_header_check(const unsigned int offset, const unsigned char *value, const unsigned int length, 
-  int (*header_check)(const unsigned char *buffer, const unsigned int buffer_size,
+void register_header_check(const unsigned int offset, const void *value, const unsigned int length, int (*header_check)(const unsigned char *buffer, const unsigned int buffer_size,
       const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new),
   file_stat_t *file_stat)
 {
@@ -104,7 +103,7 @@ static void index_header_check_aux(file_check_t *file_check_new)
 	  pos->offset < file_check_new->offset+file_check_new->length)
       {
 	td_list_add_sorted(&file_check_new->list,
-	    &pos->file_checks[file_check_new->value[pos->offset-file_check_new->offset]].list,
+	    &pos->file_checks[((const unsigned char *)file_check_new->value)[pos->offset-file_check_new->offset]].list,
 	    file_check_cmp);
 	return ;
       }
@@ -123,6 +122,7 @@ static unsigned int index_header_check(void)
   struct td_list_head *tmp;
   struct td_list_head *next;
   unsigned int nbr=0;
+ /* Initialize file_check_list from file_check_plist */
   td_list_for_each_prev_safe(tmp, next, &file_check_plist.list)
   {
     file_check_t *current_check;
@@ -141,11 +141,11 @@ void free_header_check(void)
   td_list_for_each_safe(tmpl, nextl, &file_check_list.list)
   {
     unsigned int i;
-    struct td_list_head *tmp;
-    struct td_list_head *next;
     file_check_list_t *pos=td_list_entry(tmpl, file_check_list_t, list);
     for(i=0;i<256;i++)
     {
+      struct td_list_head *tmp;
+      struct td_list_head *next;
       td_list_for_each_safe(tmp, next, &pos->file_checks[i].list)
       {
 #ifdef DEBUG_HEADER_CHECK
@@ -228,6 +228,7 @@ void file_search_footer(file_recovery_t *file_recovery, const void*footer, const
     file_recovery->file_size+= footer_length + extra_length;
 }
 
+#if 0
 void file_search_lc_footer(file_recovery_t *file_recovery, const unsigned char*footer, const unsigned int footer_length)
 {
   const unsigned int read_size=4096;
@@ -268,6 +269,7 @@ void file_search_lc_footer(file_recovery_t *file_recovery, const unsigned char*f
   file_recovery->file_size=0;
   free(buffer);
 }
+#endif
 
 int data_check_size(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
@@ -355,7 +357,7 @@ file_stat_t * init_file_stats(file_enable_t *files_enable)
 }
 
 /* The original filename begins at offset in buffer and is null terminated */
-void file_rename(const char *old_filename, const unsigned char *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
+void file_rename(const char *old_filename, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
 {
   /* new_filename is large enough to avoid a buffer overflow */
   char *new_filename;
@@ -393,26 +395,32 @@ void file_rename(const char *old_filename, const unsigned char *buffer, const in
     int ok=0;
     int bad=0;
     *dst++ = '_';
-    for(off=offset; off<buffer_size && buffer[off]!='\0'; off++)
+    src=&((const char *)buffer)[offset];
+    for(off=offset; off<buffer_size && *src!='\0'; off++, src++)
     {
-      switch(buffer[off])
+      switch(*src)
       {
 	case '/':
 	case '\\':
 	case ':':
 	case '*':
-	  *dst++ = '_';
+	case '<':
+	case '>':
+	case '|':
+	  if(*(dst-1) != '_')
+	    *dst++ = '_';
 	  bad++;
 	  break;
 	default:
-	  if(isprint(buffer[off]) && !isspace(buffer[off]))
+	  if(isprint(*src) && !isspace(*src))
 	  {
-	    *dst++ = buffer[off];
+	    *dst++ = *src;
 	    ok++;
 	  }
 	  else
 	  {
-	    *dst++ = '_';
+	    if(*(dst-1) != '_')
+	      *dst++ = '_';
 	    bad++;
 	  }
 	  break;
@@ -450,7 +458,7 @@ void file_rename(const char *old_filename, const unsigned char *buffer, const in
 }
 
 /* The original filename begins at offset in buffer and is null terminated */
-void file_rename_unicode(const char *old_filename, const unsigned char *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
+void file_rename_unicode(const char *old_filename, const void *buffer, const int buffer_size, const int offset, const char *new_ext, const int force_ext)
 {
   /* new_filename is large enough to avoid a buffer overflow */
   char *new_filename;
@@ -488,9 +496,10 @@ void file_rename_unicode(const char *old_filename, const unsigned char *buffer,
     int ok=0;
     int bad=0;
     *dst++ = '_';
-    for(off=offset; off<buffer_size && buffer[off]!='\0'; off+=2)
+    src=&((const char *)buffer)[offset];
+    for(off=offset; off<buffer_size && *src!='\0'; off+=2, src+=2)
     {
-      switch(buffer[off])
+      switch(*src)
       {
 	case '/':
 	case '\\':
@@ -500,14 +509,15 @@ void file_rename_unicode(const char *old_filename, const unsigned char *buffer,
 	  bad++;
 	  break;
 	default:
-	  if(isprint(buffer[off]) && !isspace(buffer[off]))
+	  if(isprint(*src) && !isspace(*src))
 	  {
-	    *dst++ = buffer[off];
+	    *dst++ = *src;
 	    ok++;
 	  }
 	  else
 	  {
-	    *dst++ = '_';
+	    if(*(dst-1) != '_')
+	      *dst++ = '_';
 	    bad++;
 	  }
 	  break;
