@@ -117,10 +117,7 @@ list_disk_t *insert_new_disk(list_disk_t *list_disk, disk_t *disk_car)
       {
 	if(strcmp(cur->disk->device,disk_car->device)==0)
 	{
-	  if(disk_car->clean!=NULL)
-	    disk_car->clean(disk_car);
-	  free(disk_car->device);
-	  free(disk_car);
+	  disk_car->clean(disk_car);
 	  return list_disk;
 	}
       }
@@ -195,11 +192,7 @@ int delete_list_disk(list_disk_t *list_disk)
   {
     list_disk_t *element_disk_next=element_disk->next;
     write_used|=element_disk->disk->write_used;
-    if(element_disk->disk->clean!=NULL)
-      element_disk->disk->clean(element_disk->disk);
-    free(element_disk->disk->device);
-    free(element_disk->disk->model);
-    free(element_disk->disk);
+    element_disk->disk->clean(element_disk->disk);
     free(element_disk);
     element_disk=element_disk_next;
   }
@@ -375,7 +368,6 @@ unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part
 {
   const unsigned int head_list[]={8,16,32,64,128,240,255,0};
   unsigned int best_score;
-  unsigned int score;
   unsigned int i;
   unsigned int heads_per_cylinder=disk_car->geom.heads_per_cylinder;
   disk_t *new_disk_car=(disk_t *)MALLOC(sizeof(*new_disk_car));
@@ -383,6 +375,7 @@ unsigned int get_geometry_from_list_part(const disk_t *disk_car, const list_part
   best_score=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
   for(i=0; head_list[i]!=0; i++)
   {
+    unsigned int score;
     new_disk_car->geom.heads_per_cylinder=head_list[i];
     score=get_geometry_from_list_part_aux(new_disk_car, list_part, verbose);
     if(score >= best_score)
