@@ -112,6 +112,8 @@
 //#define HDCLONE 1
 #endif
 
+extern const arch_fnct_t arch_none;
+
 struct tdewf_file_header
 {
         /* The EWF file signature (magic header)
@@ -147,28 +149,32 @@ struct info_file_struct
 };
 
 static void autoset_geometry(disk_t * disk_car, const unsigned char *buffer, const int verbose);
-static const char *file_description(disk_t *disk_car);
-static const char *file_description_short(disk_t *disk_car);
-static int file_clean(disk_t *disk_car);
+static void file_clean(disk_t *disk);
 static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset);
-static void *file_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
 static int file_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_sync(disk_t *disk_car);
 #ifndef DJGPP
 static uint64_t compute_device_size(const int hd_h, const char *device, const int verbose, const unsigned int sector_size);
-static void disk_get_model(const int hd_h, disk_t *disk, const unsigned int verbose);
 #endif
 
-int generic_clean(disk_t *disk_car)
+void generic_clean(disk_t *disk)
 {
-  free(disk_car->data);
-  disk_car->data=NULL;
-  free(disk_car->rbuffer);
-  free(disk_car->wbuffer);
-  disk_car->rbuffer=NULL;
-  disk_car->wbuffer=NULL;
-  return 0;
+  free(disk->device);
+  free(disk->model);
+  free(disk->serial_no);
+  free(disk->fw_rev);
+  free(disk->data);
+  free(disk->rbuffer);
+  free(disk->wbuffer);
+  disk->device=NULL;
+  disk->model=NULL;
+  disk->serial_no=NULL;
+  disk->fw_rev=NULL;
+  disk->data=NULL;
+  disk->rbuffer=NULL;
+  disk->wbuffer=NULL;
+  free(disk);
 }
 
 #if defined(__CYGWIN__) || defined(__MINGW32__)
@@ -191,11 +197,7 @@ list_disk_t *insert_new_disk_nodup(list_disk_t *list_disk, disk_t *disk_car, con
     {
       if(verbose>1)
         log_verbose("%s is available but reject it to avoid duplicate disk.\n", device_name);
-      if(disk_car->clean!=NULL)
-        disk_car->clean(disk_car);
-      free(disk_car->device);
-      free(disk_car->model);
-      free(disk_car);
+      disk_car->clean(disk_car);
       return list_disk;
     }
     return insert_new_disk(list_disk,disk_car);
@@ -204,7 +206,7 @@ list_disk_t *insert_new_disk_nodup(list_disk_t *list_disk, disk_t *disk_car, con
 #endif
 
 #ifdef HAVE_GLOB_H
-static list_disk_t *hd_glob_parse(const char *device_pattern, list_disk_t *list_disk, const int verbose, const arch_fnct_t *arch, const int testdisk_mode)
+static list_disk_t *hd_glob_parse(const char *device_pattern, list_disk_t *list_disk, const int verbose, const int testdisk_mode)
 {
   glob_t globbuf;
   globbuf.gl_offs = 0;
@@ -214,7 +216,7 @@ static list_disk_t *hd_glob_parse(const char *device_pattern, list_disk_t *list_
     unsigned int i;
     for (i=0; i<globbuf.gl_pathc; i++)
     {
-      list_disk=insert_new_disk(list_disk,file_test_availability(globbuf.gl_pathv[i], verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(globbuf.gl_pathv[i], verbose, testdisk_mode));
     }
   }
   globfree(&globbuf);
@@ -223,14 +225,14 @@ static list_disk_t *hd_glob_parse(const char *device_pattern, list_disk_t *list_
 #endif
 
 
-list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct_t *arch, const int testdisk_mode)
+list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const int testdisk_mode)
 {
   unsigned int i;
 #ifdef DJGPP
   int ind_stop=0;
   for(i=0x80;(i<0x88)&&!ind_stop;i++)
   {
-    disk_t *disk_car=hd_identify(verbose,i,arch,testdisk_mode);
+    disk_t *disk_car=hd_identify(verbose, i, testdisk_mode);
     if(disk_car)
       list_disk=insert_new_disk(list_disk,disk_car);
     else
@@ -247,7 +249,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     for(i=0;i<16;i++)
     {
       device_scsi[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi, verbose, testdisk_mode));
     }
 #endif
     /* Disk */
@@ -258,7 +260,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
       {
 	disk_t *disk_car;
 	sprintf(device_hd,"\\\\.\\PhysicalDrive%u", i);
-	disk_car=file_test_availability_win32(device_hd,verbose,arch,testdisk_mode);
+	disk_car=file_test_availability_win32(device_hd, verbose, testdisk_mode);
 	if(do_insert>0 || (testdisk_mode&TESTDISK_O_ALL)==TESTDISK_O_ALL)
 	  list_disk=insert_new_disk(list_disk,disk_car);
 	else
@@ -270,7 +272,7 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     {
       disk_t *disk_car;
       device_cdrom[strlen(device_cdrom)-2]=i;
-      disk_car=file_test_availability_win32(device_cdrom,verbose,arch,testdisk_mode);
+      disk_car=file_test_availability_win32(device_cdrom, verbose, testdisk_mode);
       if((testdisk_mode&TESTDISK_O_ALL)==TESTDISK_O_ALL)
 	list_disk=insert_new_disk(list_disk,disk_car);
       else
@@ -284,12 +286,12 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     for(i=0;i<20;i++)
     {
       snprintf(device, sizeof(device), "/dev/disk%u", i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device, verbose, arch, testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device, verbose, testdisk_mode));
     }
     for(i=0;i<20;i++)
     {
       snprintf(device, sizeof(device), "/dev/rdisk%u", i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device, verbose, arch, testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device, verbose, testdisk_mode));
     }
   }
 #elif defined(TARGET_LINUX)
@@ -307,13 +309,13 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     for(i=0;i<8;i++)
     {
       device_ide[strlen(device_ide)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide, verbose, testdisk_mode));
     }
     /* Disk SCSI */
     for(i=0;i<26;i++)
     {
       device_scsi[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi, verbose, testdisk_mode));
     }
     /* Device RAID Compaq */
     for(j=0;j<8;j++)
@@ -322,51 +324,51 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
       for(i=0;i<8;i++)
       {
 	device_ida[strlen(device_ida)-1]='0'+i;
-	list_disk=insert_new_disk(list_disk,file_test_availability(device_ida,verbose,arch,testdisk_mode));
+	list_disk=insert_new_disk(list_disk, file_test_availability(device_ida, verbose, testdisk_mode));
       }
     }
     for(i=0;i<8;i++)
     {
       device_cciss[strlen(device_cciss)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_cciss,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_cciss, verbose, testdisk_mode));
     }
     /* Device RAID */
     for(i=0;i<10;i++)
     {
       snprintf(device,sizeof(device),"/dev/rd/c0d%u",i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device, verbose, testdisk_mode));
     }
     /* Device RAID IDE */
     for(i=0;i<15;i++)
     {
       snprintf(device,sizeof(device),"/dev/ataraid/d%u",i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device, verbose, testdisk_mode));
     }
     /* Parallel port IDE disk */
     for(i=0;i<4;i++)
     {
       device_p_ide[strlen(device_p_ide)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_p_ide,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_p_ide, verbose, testdisk_mode));
     }
     /* I2O hard disk */
     for(i=0;i<26;i++)
     {
       device_i2o_hd[strlen(device_i2o_hd)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_i2o_hd,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_i2o_hd, verbose, testdisk_mode));
     }
     /* Memory card */
     for(i=0;i<10;i++)
     {
       device_mmc[strlen(device_mmc)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_mmc,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_mmc, verbose, testdisk_mode));
     }
 #ifdef HAVE_GLOB_H
-    list_disk=hd_glob_parse("/dev/mapper/*", list_disk, verbose, arch, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/mapper/*", list_disk, verbose, testdisk_mode);
     /* Software Raid (partition level) */
-    list_disk=hd_glob_parse("/dev/md?", list_disk, verbose, arch, testdisk_mode);
-    list_disk=hd_glob_parse("/dev/sr?", list_disk, verbose, arch, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/md*", list_disk, verbose, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/sr?", list_disk, verbose, testdisk_mode);
     /* Software (ATA)Raid configured (disk level) via dmraid */
-    list_disk=hd_glob_parse("/dev/dm-*", list_disk, verbose, arch, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/dm-*", list_disk, verbose, testdisk_mode);
 #endif
   }
 #elif defined(TARGET_SOLARIS)
@@ -377,14 +379,14 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
       if(i!=7)
       {
 	rdsk[13]='0'+i;
-	list_disk=insert_new_disk(list_disk,file_test_availability(rdsk,verbose,arch,testdisk_mode));
+	list_disk=insert_new_disk(list_disk, file_test_availability(rdsk, verbose, testdisk_mode));
       }
     }
   }
 #elif defined(__HAIKU__)
   {
 #ifdef HAVE_GLOB_H
-    list_disk=hd_glob_parse("/dev/disk/*/*/master/raw", list_disk, verbose, arch, testdisk_mode);
+    list_disk=hd_glob_parse("/dev/disk/*/*/master/raw", list_disk, verbose, testdisk_mode);
 #endif
   }
 #else
@@ -406,61 +408,61 @@ list_disk_t *hd_parse(list_disk_t *list_disk, const int verbose, const arch_fnct
     for(i=0;i<8;i++)
     {
       device_ide[strlen(device_ide)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_ide2[strlen(device_ide2)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide2,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide2, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_ide3[strlen(device_ide3)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide3,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide3, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_ide4[strlen(device_ide4)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide4,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide4, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_ide_hd[strlen(device_ide_hd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide_hd,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_ide_hd, verbose, testdisk_mode));
     }
     /* Disk SCSI */
     for(i=0;i<8;i++)
     {
       device_scsi[strlen(device_scsi)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_scsi2[strlen(device_scsi2)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi2,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi2, verbose, testdisk_mode));
     }
     for(i=0;i<8;i++)
     {
       device_scsi_hd[strlen(device_scsi_hd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi_hd,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_scsi_hd, verbose, testdisk_mode));
     }
     /* optical disks */
     for(i=0;i<8;i++)
     {
       device_optdisk[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_optdisk,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_optdisk, verbose, testdisk_mode));
     } 
     /* CD */
     for(i=0;i<8;i++)
     {
       device_cd[strlen(device_cd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_cd,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_cd, verbose, testdisk_mode));
     }
     /* VND */
     for(i=0;i<4;i++)
     {
       device_vnd[strlen(device_vnd)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_vnd,verbose,arch,testdisk_mode));
+      list_disk=insert_new_disk(list_disk, file_test_availability(device_vnd, verbose, testdisk_mode));
     }
   }
 #endif
@@ -815,22 +817,6 @@ void update_disk_car_fields(disk_t *disk_car)
 }
 
 #ifdef TARGET_LINUX
-static char* strip_name(char* str)
-{
-  int     i;
-  int     end = 0;
-
-  for (i = 0; str[i] != 0; i++) {
-    if (!isspace (str[i])
-	|| (isspace (str[i]) && !isspace (str[i+1]) && str[i+1])) {
-      str [end] = str[i];
-      end++;
-    }
-  }
-  str[end] = 0;
-  return strdup (str);
-}
-
 /* This function reads the /sys entry named "file" for device "disk_car". */
 static char * read_device_sysfs_file (const disk_t *disk_car, const char *file)
 {
@@ -851,7 +837,7 @@ static char * read_device_sysfs_file (const disk_t *disk_car, const char *file)
   }
 
   fclose (f);
-  return strip_name (buf);
+  return strip_dup (buf);
 }
 #endif
 
@@ -866,86 +852,80 @@ static char * read_device_sysfs_file (const disk_t *disk_car, const char *file)
 #ifdef HAVE_SCSI_SCSI_IOCTL_H
 #include <scsi/scsi_ioctl.h>
 #endif
+#ifdef HAVE_SCSI_SG_H
+#include <scsi/sg.h>
+#endif
 #endif
 
-#if defined(TARGET_LINUX) && defined(SCSI_IOCTL_GET_IDLUN) && defined(SCSI_IOCTL_SEND_COMMAND)
-static int scsi_query_product_info (const int hd_h, char **vendor, char **product)
+#if defined(TARGET_LINUX) && defined(INQUIRY) && defined(SG_GET_VERSION_NUM)
+typedef struct _scsi_inquiry_data
 {
-  /* The following are defined by the SCSI-2 specification. */
-  typedef struct _scsi_inquiry_cmd
-  {
-    uint8_t op;
-    uint8_t lun;          /* bits 5-7 denote the LUN */
-    uint8_t page_code;
-    uint8_t reserved;
-    uint8_t alloc_length;
-    uint8_t control;
-  } __attribute__((packed)) scsi_inquiry_cmd_t;
-
-  typedef struct _scsi_inquiry_data
-  {
-    uint8_t peripheral_info;
-    uint8_t device_info;
-    uint8_t version_info;
-    uint8_t _field1;
-    uint8_t additional_length;
-    uint8_t _reserved1;
-    uint8_t _reserved2;
-    uint8_t _field2;
-    uint8_t vendor_id[8];
-    uint8_t product_id[16];
-    uint8_t product_revision[4];
-    uint8_t vendor_specific[20];
-    uint8_t _reserved3[40];
-  } __attribute__((packed)) scsi_inquiry_data_t;
-
-  struct scsi_arg
-  {
-    unsigned int inlen;
-    unsigned int outlen;
-
-    union arg_data
-    {
-      scsi_inquiry_data_t out;
-      scsi_inquiry_cmd_t  in;
-    } data;
-  } arg;
-
-  struct scsi_idlun
-  {
-    uint32_t dev_id;
-    uint32_t host_unique_id;
-  } idlun;
+  uint8_t peripheral_info;
+  uint8_t device_info;
+  uint8_t version_info;
+  uint8_t _field1;
+  uint8_t additional_length;
+  uint8_t _reserved1;
+  uint8_t _reserved2;
+  uint8_t _field2;
+  uint8_t vendor_id[8];
+  uint8_t product_id[16];
+  uint8_t product_revision[4];
+  uint8_t vendor_specific[20];
+  uint8_t _reserved3[40];
+} __attribute__((packed)) scsi_inquiry_data_t;
+#define INQ_CMD_LEN	6
+#define INQ_REPLY_LEN	sizeof(scsi_inquiry_data_t)
 
+static int scsi_query_product_info (const int sg_fd, char **vendor, char **product, char **fw_rev)
+{
+  unsigned char inqCmdBlk[INQ_CMD_LEN] = {INQUIRY, 0, 0, 0, INQ_REPLY_LEN, 0};
+  scsi_inquiry_data_t inqBuff;
+  unsigned char sense_buffer[32];
+  sg_io_hdr_t io_hdr;
+  int k;
   char    buf[32];
-
   *vendor = NULL;
   *product = NULL;
 
-  if (ioctl (hd_h, SCSI_IOCTL_GET_IDLUN, &idlun) < 0)
+  /* It is prudent to check we have a sg device by trying an ioctl */
+  if (ioctl(sg_fd, SG_GET_VERSION_NUM, &k) < 0 || k < 30000)
+    return -1;
+  /* Prepare INQUIRY command */
+  memset(&io_hdr, 0, sizeof(sg_io_hdr_t));
+  io_hdr.interface_id = 'S';
+  io_hdr.cmd_len = sizeof(inqCmdBlk);
+  /* io_hdr.iovec_count = 0; */  /* memset takes care of this */
+  io_hdr.mx_sb_len = sizeof(sense_buffer);
+  io_hdr.dxfer_direction = SG_DXFER_FROM_DEV;
+  io_hdr.dxfer_len = INQ_REPLY_LEN;
+  io_hdr.dxferp = (unsigned char*)&inqBuff;
+  io_hdr.cmdp = inqCmdBlk;
+  io_hdr.sbp = sense_buffer;
+  io_hdr.timeout = 20000;     /* 20000 millisecs == 20 seconds */
+  /* io_hdr.flags = 0; */     /* take defaults: indirect IO, etc */
+  /* io_hdr.pack_id = 0; */
+  /* io_hdr.usr_ptr = NULL; */
+
+  if (ioctl(sg_fd, SG_IO, &io_hdr) < 0)
     return -1;
 
-
-  memset (&arg, 0x00, sizeof(struct scsi_arg));
-  arg.inlen  = 0;
-  arg.outlen = sizeof(scsi_inquiry_data_t);
-  arg.data.in.op  = INQUIRY;
-  arg.data.in.lun = idlun.host_unique_id << 5;
-  arg.data.in.alloc_length = sizeof(scsi_inquiry_data_t);
-  arg.data.in.page_code = 0;
-  arg.data.in.reserved = 0;
-  arg.data.in.control = 0;
-
-  if (ioctl (hd_h, SCSI_IOCTL_SEND_COMMAND, &arg) < 0)
+  /* now for the error processing */
+  if ((io_hdr.info & SG_INFO_OK_MASK) != SG_INFO_OK)
     return -1;
 
-  memcpy (buf, arg.data.out.vendor_id, 8);
+  memcpy (buf, inqBuff.vendor_id, 8);
   buf[8] = '\0';
-  *vendor = strip_name (buf);
+  *vendor = strip_dup (buf);
 
-  memcpy (buf, arg.data.out.product_id, 16);
+  memcpy (buf, inqBuff.product_id, 16);
   buf[16] = '\0';
-  *product = strip_name (buf);
+  *product = strip_dup (buf);
+
+  /* Information is truncated */
+  memcpy (buf, inqBuff.product_revision, 4);
+  buf[4] = '\0';
+  *fw_rev= strip_dup (buf);
 
   return 0;
 }
@@ -954,6 +934,53 @@ static int scsi_query_product_info (const int hd_h, char **vendor, char **produc
 #ifndef DJGPP
 static void disk_get_model(const int hd_h, disk_t *dev, const unsigned int verbose)
 {
+#ifdef HDIO_GET_IDENTITY
+  if(dev->model!=NULL)
+    return;
+  {
+    struct hd_driveid       hdi;
+    memset(&hdi, 0, sizeof(hdi));
+    if (ioctl (hd_h, HDIO_GET_IDENTITY, &hdi)==0)
+    {
+      char tmp[41];
+      if(dev->model==NULL)
+      {
+	memcpy (tmp, hdi.model, 40);
+	tmp[40] = '\0';
+	dev->model=strip_dup(tmp);
+      }
+      if(dev->serial_no==NULL)
+      {
+	memcpy (tmp, hdi.serial_no, 20);
+	tmp[20] = '\0';
+	dev->serial_no=strip_dup(tmp);
+      }
+      if(dev->fw_rev==NULL)
+      {
+	memcpy (tmp, hdi.fw_rev, 8);
+	tmp[8] = '\0';
+	dev->fw_rev=strip_dup(tmp);
+      }
+    }
+  }
+#endif
+#if defined(TARGET_LINUX) && defined(SCSI_IOCTL_GET_IDLUN) && defined(SCSI_IOCTL_SEND_COMMAND)
+  if(dev->model!=NULL)
+    return;
+  {
+    /* Uses direct queries via the deprecated ioctl SCSI_IOCTL_SEND_COMMAND */
+    char *vendor=NULL;
+    char *product=NULL;
+    scsi_query_product_info (hd_h, &vendor, &product, &dev->fw_rev);
+    if (vendor && product)
+    {
+      dev->model = (char*) MALLOC (8 + 16 + 2);
+      sprintf (dev->model, "%.8s %.16s", vendor, product);
+    }
+    free(vendor);
+    free(product);
+  }
+#endif
 #ifdef TARGET_LINUX
   if(dev->model!=NULL)
     return;
@@ -972,39 +999,6 @@ static void disk_get_model(const int hd_h, disk_t *dev, const unsigned int verbo
     free(product);
   }
 #endif
-#ifdef HDIO_GET_IDENTITY
-  if(dev->model!=NULL)
-    return;
-  {
-    struct hd_driveid       hdi;
-    memset(&hdi, 0, sizeof(hdi));
-    if (ioctl (hd_h, HDIO_GET_IDENTITY, &hdi)==0)
-    {
-      char hdi_buf[41];
-      memcpy (hdi_buf, hdi.model, 40);
-      hdi_buf[40] = '\0';
-      if(dev!=NULL)
-	dev->model=strdup(hdi_buf);
-    }
-  }
-#endif
-#if defined(TARGET_LINUX) && defined(SCSI_IOCTL_GET_IDLUN) && defined(SCSI_IOCTL_SEND_COMMAND)
-  if(dev->model!=NULL)
-    return;
-  {
-    /* Uses direct queries via the deprecated ioctl SCSI_IOCTL_SEND_COMMAND */
-    char *vendor=NULL;
-    char *product=NULL;
-    scsi_query_product_info (hd_h, &vendor, &product);
-    if (vendor && product)
-    {
-      dev->model = (char*) MALLOC (8 + 16 + 2);
-      sprintf (dev->model, "%.8s %.16s", vendor, product);
-    }
-    free(vendor);
-    free(product);
-  }
-#endif
 #if defined(__CYGWIN__) || defined(__MINGW32__)
   if(dev->model!=NULL)
     return;
@@ -1064,16 +1058,24 @@ static uint64_t compute_device_size(const int hd_h, const char *device, const in
 }
 #endif
 
-static const char *file_description(disk_t *disk_car)
+static const char *file_description(disk_t *disk)
 {
-  const struct info_file_struct *data=(const struct info_file_struct *)disk_car->data;
+  const struct info_file_struct *data=(const struct info_file_struct *)disk->data;
   char buffer_disk_size[100];
-  size_to_unit(disk_car->disk_size, buffer_disk_size);
-  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - %s - CHS %lu %u %u%s",
-      disk_car->device, buffer_disk_size,
-      disk_car->geom.cylinders, disk_car->geom.heads_per_cylinder, disk_car->geom.sectors_per_head,
-      ((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
-  return disk_car->description_txt;
+  size_to_unit(disk->disk_size, buffer_disk_size);
+  if(disk->geom.heads_per_cylinder == 1 && disk->geom.sectors_per_head == 1)
+    snprintf(disk->description_txt, sizeof(disk->description_txt),
+	"Disk %s - %s - %lu sectors%s",
+	disk->device, buffer_disk_size,
+	disk->geom.cylinders,
+	((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
+  else
+    snprintf(disk->description_txt, sizeof(disk->description_txt),
+	"Disk %s - %s - CHS %lu %u %u%s",
+	disk->device, buffer_disk_size,
+	disk->geom.cylinders, disk->geom.heads_per_cylinder, disk->geom.sectors_per_head,
+	((data->mode&O_RDWR)==O_RDWR?"":" (RO)"));
+  return disk->description_txt;
 }
 
 static const char *file_description_short(disk_t *disk_car)
@@ -1093,17 +1095,17 @@ static const char *file_description_short(disk_t *disk_car)
   return disk_car->description_short_txt;
 }
 
-static int file_clean(disk_t *disk_car)
+static void file_clean(disk_t *disk)
 {
-  if(disk_car->data!=NULL)
+  if(disk->data!=NULL)
   {
-    struct info_file_struct *data=(struct info_file_struct *)disk_car->data;
+    struct info_file_struct *data=(struct info_file_struct *)disk->data;
     /*
 #ifdef BLKRRPART
     if (ioctl(data->handle, BLKRRPART, NULL)) {
-      log_error("%s BLKRRPART failed\n",disk_car->description(disk_car));
+      log_error("%s BLKRRPART failed\n",disk->description(disk));
     } else {
-      log_debug("%s BLKRRPART ok\n",disk_car->description(disk_car));
+      log_debug("%s BLKRRPART ok\n",disk->description(disk));
     }
 #endif
     */
@@ -1115,61 +1117,99 @@ static int file_clean(disk_t *disk_car)
     }
 #endif
     close(data->handle);
+    data->handle=0;
   }
-  return generic_clean(disk_car);
+  generic_clean(disk);
 }
 
-static int file_pread_aux(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset)
+static int file_pread_aux(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
 {
-  long int ret=-1;
-  int fd=((struct info_file_struct *)disk_car->data)->handle;
-#if defined(HAVE_PREAD) && !defined(__CYGWIN__)
+  long int ret;
+  int fd=((struct info_file_struct *)disk->data)->handle;
+#if defined(__CYGWIN__)
+  if(lseek(fd,offset,SEEK_SET) < 0)
+  {
+    log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) lseek err %s\n",
+	fd, (unsigned)(count/disk->sector_size),
+	(long unsigned int)(offset/disk->sector_size),
+	offset2cylinder(disk,offset),
+	offset2head(disk,offset),
+	offset2sector(disk,offset),
+	strerror(errno));
+    return -1;
+  }
+  {
+    /* November 28, 2004, CGR: cygwin read function is about 10 times slower
+       because it reads 60k each time, so lets call ReadFile directly */
+    DWORD dwByteRead;
+    HANDLE handle=(HANDLE)get_osfhandle(fd);
+    if(ReadFile(handle, buf,count,&dwByteRead,NULL)==0)
+    {
+      LPVOID lpMsgBuf;
+      DWORD dw = GetLastError(); 
+      FormatMessage(
+	  FORMAT_MESSAGE_ALLOCATE_BUFFER | 
+	  FORMAT_MESSAGE_FROM_SYSTEM,
+	  NULL,
+	  dw,
+	  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+	  (LPTSTR) &lpMsgBuf,
+	  0, NULL );
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) ReadFile %s\n",
+	fd, (unsigned)(count/disk->sector_size),
+	(long unsigned int)(offset/disk->sector_size),
+	offset2cylinder(disk, offset),
+	offset2head(disk, offset),
+	offset2sector(disk, offset),
+	(char*)lpMsgBuf);
+      LocalFree(lpMsgBuf);
+      return -1;
+    }
+    return dwByteRead;
+  }
+#elif defined(__MINGW32__)
+  if(_lseeki64(fd,offset,SEEK_SET) < 0)
+  {
+    log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n",
+	fd, (unsigned)(count/disk->sector_size),
+	(long unsigned int)(offset/disk->sector_size),
+	offset2cylinder(disk, offset),
+	offset2head(disk, offset),
+	offset2sector(disk, offset),
+	strerror(errno));
+    return -1;
+  }
+  ret=read(fd, buf, count);
+#else
+#if defined(HAVE_PREAD)
   ret=pread(fd,buf,count,offset);
   if(ret<0 && errno == ENOSYS)
 #endif
   {
-#ifdef __MINGW32__
-    if(_lseeki64(fd,offset,SEEK_SET)==-1)
-    {
-      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) seek err %s\n", fd,
-          (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
-          offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
-          strerror(errno));
-      return -1;
-    }
-#else
     if(lseek(fd,offset,SEEK_SET)==(off_t)-1)
     {
-      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) lseek err %s\n", fd,
-          (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
-          offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset),
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) lseek err %s\n",
+	  fd, (unsigned)(count/disk->sector_size),
+	  (long unsigned int)(offset/disk->sector_size),
+          offset2cylinder(disk, offset),
+	  offset2head(disk, offset),
+	  offset2sector(disk, offset),
           strerror(errno));
       return -1;
     }
-#endif
-#if defined(__CYGWIN__)
-    {
-      /* November 28, 2004, CGR: cygwin read function is about 10 times slower
-         because it reads 60k each time, so lets call ReadFile directly */
-      DWORD dwByteRead;
-      HANDLE handle=(HANDLE)get_osfhandle(fd);
-      ret=ReadFile(handle, buf,count,&dwByteRead,NULL);
-      if(ret==0)
-        ret=-1;
-      else
-        ret=dwByteRead;
-    }
-#else
     ret=read(fd, buf, count);
-#endif
   }
+#endif
   if(ret!=count)
   {
-    if(offset+count <= disk_car->disk_size && offset+count <= disk_car->disk_real_size)
+    if(offset+count <= disk->disk_size && offset+count <= disk->disk_real_size)
     {
-      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", fd,
-          (unsigned)(count/disk_car->sector_size), (long unsigned)(offset/disk_car->sector_size),
-          offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
+      log_error("file_pread(%d,%u,buffer,%lu(%u/%u/%u)) read err: ",
+	  fd, (unsigned)(count/disk->sector_size),
+	  (long unsigned)(offset/disk->sector_size),
+          offset2cylinder(disk, offset),
+	  offset2head(disk, offset),
+	  offset2sector(disk, offset));
       if(ret<0)
         log_error("%s\n", strerror(errno));
       else if(ret==0)
@@ -1185,13 +1225,13 @@ static int file_pread_aux(disk_t *disk_car, void *buf, const unsigned int count,
     memset((char*)buf+ret,0,count-ret);
   }
 #ifdef HDCLONE
-  else
+  if(ret>0)
   {
-    int handle_clone=((struct info_file_struct *)disk_car->data)->handle_clone;
+    int handle_clone=((struct info_file_struct *)disk->data)->handle_clone;
     if(handle_clone>0)
     {
-      pwrite(handle_clone, buf, count, offset);
-      fdatasync(handle_clone);
+      pwrite(handle_clone, buf, ret, offset);
+//      fdatasync(handle_clone);
     }
   }
 #endif
@@ -1203,17 +1243,10 @@ static int file_pread(disk_t *disk_car, void *buf, const unsigned int count, con
   return align_pread(&file_pread_aux, disk_car, buf, count, offset);
 }
 
-static void *file_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
-{
-  if(file_pread(disk, buf, count, offset)==offset)
-    return buf;
-  return NULL;
-}
-
 static int file_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   int fd=((struct info_file_struct *)disk_car->data)->handle;
-  long int ret=-1;
+  long int ret;
 #if defined(HAVE_PWRITE) && !defined(__CYGWIN__)
   ret=pwrite(fd,buf,count,offset);
   if(ret<0 && errno == ENOSYS)
@@ -1306,7 +1339,7 @@ static void autoset_geometry(disk_t *disk, const unsigned char *buffer, const in
     disk->geom.sectors_per_head / disk->geom.heads_per_cylinder;
 }
 
-disk_t *file_test_availability(const char *device, const int verbose, const arch_fnct_t *arch, int testdisk_mode)
+disk_t *file_test_availability(const char *device, const int verbose, int testdisk_mode)
 {
   disk_t *disk_car=NULL;
   struct stat stat_rec;
@@ -1411,7 +1444,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
       else if(strcmp(device, "/dev/sda135") == 0)
 	dos_nr = 0x87;
       if(dos_nr>0)
-	disk_car = hd_identify(verbose, dos_nr, arch, testdisk_mode);
+	disk_car = hd_identify(verbose, dos_nr, testdisk_mode);
       if(disk_car!=NULL)
 	return disk_car;
     }
@@ -1420,13 +1453,13 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
     if(strncmp(device,"/dev/",5)!=0)
     {
 #if defined(HAVE_LIBEWF_H) && defined(HAVE_LIBEWF) && defined(HAVE_GLOB_H)
-      return fewf_init(device, arch, testdisk_mode);
+      return fewf_init(device, testdisk_mode);
 #endif
     }
-    return disk_car;
+    return NULL;
   }
   disk_car=(disk_t *)MALLOC(sizeof(*disk_car));
-  disk_car->arch=arch;
+  disk_car->arch=&arch_none;
   init_disk(disk_car);
   disk_car->device=strdup(device);
   data=(struct info_file_struct *)MALLOC(sizeof(*data));
@@ -1435,7 +1468,6 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   disk_car->data=data;
   disk_car->description=file_description;
   disk_car->description_short=file_description_short;
-  disk_car->pread_fast=file_pread_fast;
   disk_car->pread=file_pread;
   disk_car->pwrite=((mode&O_RDWR)==O_RDWR?file_pwrite:file_nopwrite);
   disk_car->sync=file_sync;
@@ -1476,13 +1508,11 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
 #endif
   {
     unsigned char *buffer;
-    struct tdewf_file_header *ewf;
     const uint8_t evf_file_signature[8] = { 'E', 'V', 'F', 0x09, 0x0D, 0x0A, 0xFF, 0x00 };
     if(verbose>1)
       log_verbose("file_test_availability %s is a file\n", device);
     disk_car->sector_size=DEFAULT_SECTOR_SIZE;
     buffer=(unsigned char*)MALLOC(DEFAULT_SECTOR_SIZE);
-    ewf=(struct tdewf_file_header*)buffer;
     if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)<0)
     {
       memset(buffer,0,DEFAULT_SECTOR_SIZE);
@@ -1505,7 +1535,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
       free(disk_car);
       close(hd_h);
 #if defined(HAVE_LIBEWF_H) && defined(HAVE_LIBEWF)
-      return fewf_init(device, arch, testdisk_mode);
+      return fewf_init(device, testdisk_mode);
 #else
       return NULL;
 #endif
@@ -1515,11 +1545,13 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
       disk_car->geom.cylinders=0;
       disk_car->geom.heads_per_cylinder=255;
       disk_car->geom.sectors_per_head=63;
+#if 0
       if((uint64_t)stat_rec.st_size > disk_car->offset)
       {
 	disk_car->disk_real_size=((uint64_t)stat_rec.st_size-disk_car->offset+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
       }
       else
+#endif
       {
 	off_t pos;
 	pos=lseek(hd_h,0,SEEK_END);
@@ -1544,9 +1576,9 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
       char *new_file=(char *)MALLOC(strlen(device)+5);
       sprintf(new_file, "%s.dd", device);
 #ifdef O_LARGEFILE
-      data->handle_clone=open(new_file, O_CREAT|O_EXCL|O_LARGEFILE|O_WRONLY,00600);
+      data->handle_clone=open(new_file, O_CREAT|O_LARGEFILE|O_RDWR,00600);
 #else
-      data->handle_clone=open(new_file, O_CREAT|O_EXCL|O_WRONLY,00600);
+      data->handle_clone=open(new_file, O_CREAT|O_RDWR,00600);
 #endif
       free(new_file);
     }
@@ -1567,7 +1599,7 @@ disk_t *file_test_availability(const char *device, const int verbose, const arch
   return NULL;
 }
 
-void hd_update_geometry(disk_t *disk, const int allow_partial_last_cylinder, const int verbose)
+void hd_update_geometry(disk_t *disk, const int verbose)
 {
   if(disk->autodetect!=0)
   {
@@ -1582,53 +1614,16 @@ void hd_update_geometry(disk_t *disk, const int allow_partial_last_cylinder, con
     }
     free(buffer);
   }
-  if((unsigned int)(disk->geom.cylinders+1)!=0)	/* Avoid to wrap */
-  {
-    uint64_t pos;
-    CHS_t pos_CHS;
-    pos_CHS.cylinder=disk->geom.cylinders-1+1;
-    if(allow_partial_last_cylinder)
-    {
-      pos_CHS.head=0;
-      pos_CHS.sector=1;
-    }
-    else
-    {
-      pos_CHS.head=disk->geom.heads_per_cylinder-1;
-      pos_CHS.sector=disk->geom.sectors_per_head;
-    }
-    pos=CHS2offset(disk,&pos_CHS);
-#ifdef DJGPP
-    if(disk->description==disk_description)
-    {
-      struct info_disk_struct*data=disk->data;
-      data->geo_phys.cylinders=0;
-    }
-#endif
-    {
-      unsigned char *buffer=(unsigned char *)MALLOC(disk->sector_size);
-      if((unsigned)disk->pread(disk, buffer, disk->sector_size, pos) == disk->sector_size)
-      {
-	disk->geom.cylinders++;
-	if(disk->disk_size < (uint64_t)disk->geom.cylinders * disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size)
-	{
-	  disk->disk_size = (uint64_t)disk->geom.cylinders * disk->geom.heads_per_cylinder * disk->geom.sectors_per_head * disk->sector_size;
-	  log_info("Computes LBA from CHS for %s\n",disk->description(disk));
-	}
-      }
-      free(buffer);
-    }
-  }
 #ifdef DJGPP
   if(disk->description==disk_description)
   {
-    struct info_disk_struct*data=disk->data;
+    struct info_disk_struct*data=(struct info_disk_struct*)disk->data;
     data->geo_phys.cylinders=disk->geom.cylinders;
   }
 #endif
 }
 
-void hd_update_all_geometry(const list_disk_t * list_disk, const int allow_partial_last_cylinder, const int verbose)
+void hd_update_all_geometry(const list_disk_t * list_disk, const int verbose)
 {
   const list_disk_t *element_disk;
   if(verbose>1)
@@ -1636,7 +1631,7 @@ void hd_update_all_geometry(const list_disk_t * list_disk, const int allow_parti
     log_trace("hd_update_all_geometry\n");
   }
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
-    hd_update_geometry(element_disk->disk,allow_partial_last_cylinder,verbose);
+    hd_update_geometry(element_disk->disk, verbose);
 }
 
 void init_disk(disk_t *disk)
@@ -1653,6 +1648,8 @@ void init_disk(disk_t *disk)
   disk->rbuffer_size=0;
   disk->wbuffer_size=0;
   disk->model=NULL;
+  disk->serial_no=NULL;
+  disk->fw_rev=NULL;
   disk->write_used=0;
   disk->description_txt[0]='\0';
   disk->unit=UNIT_CHS;
