@@ -88,8 +88,9 @@ int test_HFS(disk_t *disk_car, const hfs_mdb_t *hfs_mdb,partition_t *partition,c
   /* Check for HFS signature */
   if (hfs_mdb->drSigWord!=be16(HFS_SUPER_MAGIC))
     return 1;
-  /* Check for valid blocksize */
-  if(be32(hfs_mdb->drAlBlkSiz)%512!=0 || be32(hfs_mdb->drAlBlkSiz)==0)
+  /* Blocksize must be a multiple of 512 */
+  if(be32(hfs_mdb->drAlBlkSiz)<512 ||
+      ((be32(hfs_mdb->drAlBlkSiz)-1) & be32(hfs_mdb->drAlBlkSiz))!=0)
     return 1;
   /* Check for valid number of allocation blocks */
   if(be16(hfs_mdb->drNmAlBlks)==0)
@@ -123,7 +124,9 @@ int test_HFS(disk_t *disk_car, const hfs_mdb_t *hfs_mdb,partition_t *partition,c
 static int set_HFS_info(partition_t *partition, const hfs_mdb_t *hfs_mdb)
 {
   unsigned int name_size=sizeof(hfs_mdb->drVN)-1;
-  snprintf(partition->info,sizeof(partition->info),"HFS");
+  partition->blocksize=be32(hfs_mdb->drAlBlkSiz);
+  snprintf(partition->info, sizeof(partition->info),
+      "HFS blocksize=%u", partition->blocksize);
   if(name_size>hfs_mdb->drVN[0])
     name_size=hfs_mdb->drVN[0];
   memcpy(partition->fsname,&hfs_mdb->drVN[0]+1,name_size);
