@@ -101,19 +101,6 @@ int screen_buffer_add(const char *_format, ...)
   return 0;
 }
 
-void screen_buffer_to_stdout(void)
-{
-  int i;
-  if(intr_nbr_line<MAX_LINES && intr_buffer_screen[intr_nbr_line][0]!='\0')
-    intr_nbr_line++;
-  /* to log file and stdout */
-  for(i=0;i<intr_nbr_line && i<MAX_LINES;i++)
-  {
-    printf("%s\n",intr_buffer_screen[i]);
-    log_info("%s\n",intr_buffer_screen[i]);
-  }
-}
-
 void screen_buffer_reset(void)
 {
   intr_nbr_line=0;
@@ -130,6 +117,23 @@ void screen_buffer_to_log(void)
     log_info("%s\n",intr_buffer_screen[i]);
 }
 
+int get_partition_status(const partition_t *partition)
+{
+  /* Don't marked as D(eleted) an entry that is not a partition */
+  if(partition->order==NO_ORDER && partition->status==STATUS_DELETED)
+    return ' ';
+  switch(partition->status)
+  {
+    case STATUS_PRIM:           return 'P';
+    case STATUS_PRIM_BOOT:      return '*';
+    case STATUS_EXT:            return 'E';
+    case STATUS_EXT_IN_EXT:     return 'X';
+    case STATUS_LOG:            return 'L';
+    case STATUS_DELETED:        return 'D';
+    default:			return ' ';
+  }
+}
+
 const char *aff_part_aux(const unsigned int newline, const disk_t *disk_car, const partition_t *partition)
 {
   char status=' ';
@@ -138,7 +142,6 @@ const char *aff_part_aux(const unsigned int newline, const disk_t *disk_car, con
   const arch_fnct_t *arch=partition->arch;
   if(arch==NULL)
   {
-    arch=disk_car->arch;
     log_error("BUG: No arch for a partition\n");
     msg[0]='\0';
     return msg;
@@ -146,28 +149,14 @@ const char *aff_part_aux(const unsigned int newline, const disk_t *disk_car, con
   msg[sizeof(msg)-1]=0;
   if((newline&AFF_PART_ORDER)==AFF_PART_ORDER)
   {
-    if((partition->status!=STATUS_EXT_IN_EXT) && (partition->order!=NO_ORDER))
+    if(partition->status!=STATUS_EXT_IN_EXT && partition->order!=NO_ORDER)
       pos+=snprintf(&msg[pos],sizeof(msg)-pos-1,"%2u ", partition->order);
     else
       pos+=snprintf(&msg[pos],sizeof(msg)-pos-1,"   ");
   }
   if((newline&AFF_PART_STATUS)==AFF_PART_STATUS)
   {
-    switch(partition->status)
-    {
-      case STATUS_PRIM:           status='P'; break;
-      case STATUS_PRIM_BOOT:      status='*'; break;
-      case STATUS_EXT:            status='E'; break;
-      case STATUS_EXT_IN_EXT:     status='X'; break;
-      case STATUS_LOG:            status='L'; break;
-      case STATUS_DELETED:        status='D'; break;
-      default:			  status=' '; break;
-    }
-    /* Don't marked as D(eleted) an entry that is not a partition */
-    if((newline&AFF_PART_ORDER)==AFF_PART_ORDER &&
-      partition->order==NO_ORDER &&
-      partition->status==STATUS_DELETED)
-      status=' ';
+    status=get_partition_status(partition);
   }
   pos+=snprintf(&msg[pos],sizeof(msg)-pos-1,"%c", status);
   if(arch->get_partition_typename(partition)!=NULL)
@@ -207,25 +196,6 @@ const char *aff_part_aux(const unsigned int newline, const disk_t *disk_car, con
 #define PATH_DRIVE_LENGTH 9
 #endif
 
-char *td_getcwd(char *buf, unsigned long size)
-{
-  /* buf must non-NULL*/
-#ifdef HAVE_GETCWD
-  if(getcwd(buf, size)!=NULL)
-    return buf;
-#endif
-  buf[0]='.';
-  buf[1]='\0';
-  return buf;
-}
-
-char *get_default_location(void)
-{
-  char dst_directory[4096];
-  td_getcwd(dst_directory, sizeof(dst_directory));
-  return strdup(dst_directory);
-}
-
 unsigned long long int ask_number_cli(char **current_cmd, const unsigned long long int val_cur, const unsigned long long int val_min, const unsigned long long int val_max, const char * _format, ...)
 {
   if(*current_cmd!=NULL)
