@@ -55,9 +55,8 @@ struct info_io_redir
   list_redir_t *list_redir;
 };
 
-static void *io_redir_pread_fast(disk_t *disk_car, void *buffer, const unsigned int count, const uint64_t offset);
 static int io_redir_pread(disk_t *disk_car, void *buffer, const unsigned int count, const uint64_t offset);
-static int io_redir_clean(disk_t *clean);
+static void io_redir_clean(disk_t *clean);
 
 int io_redir_add_redir(disk_t *disk_car, const uint64_t org_offset, const unsigned int size, const uint64_t new_offset, const void *mem)
 {
@@ -79,7 +78,6 @@ int io_redir_add_redir(disk_t *disk_car, const uint64_t org_offset, const unsign
     disk_car->data=data;
     disk_car->description=old_disk_car->description;
     disk_car->pwrite=old_disk_car->pwrite;
-    disk_car->pread_fast=io_redir_pread_fast;
     disk_car->pread=io_redir_pread;
     disk_car->clean=io_redir_clean;
   }
@@ -219,14 +217,7 @@ static int io_redir_pread(disk_t *disk_car, void *buffer, const unsigned int cou
   return count;
 }
 
-static void *io_redir_pread_fast(disk_t *disk, void *buffer, const unsigned int count, const uint64_t offset)
-{
-  if(io_redir_pread(disk, buffer, count, offset)==count)
-    return buffer;
-  return NULL;
-}
-
-static int io_redir_clean(disk_t *disk_car)
+static void io_redir_clean(disk_t *disk_car)
 {
   if(disk_car->data)
   {
@@ -236,6 +227,5 @@ static int io_redir_clean(disk_t *disk_car)
     free(disk_car->data);
     disk_car->data=NULL;
   }
-  return 0;
 }
 
