@@ -23,8 +23,7 @@
 extern "C" {
 #endif
 int check_ISO(disk_t *disk_car, partition_t *partition);
-int recover_ISO(disk_t *disk_car, const struct iso_primary_descriptor *iso, partition_t *partition, const int verbose, const int dump_ind);
-
+int recover_ISO(const struct iso_primary_descriptor *iso, partition_t *partition);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
