@@ -46,18 +46,20 @@
 #define HDPARM_BUF_SIZ 0x1A
 #define MAX_IO_NBR 3
 #define MAX_HD_ERR 100
+
+extern const arch_fnct_t arch_none;
+
 static void free_dos_buffer(void);
 static int alloc_cmd_dos_buffer(void);
 static int hd_identify_enh_bios(disk_t *param_disk,const int verbose);
 static int check_enh_bios(const unsigned int disk, const int verbose);
 static int hd_report_error(disk_t *disk_car, const uint64_t hd_offset, const unsigned int count, const int rc);
 static const char *disk_description_short(disk_t *disk_car);
-static void *disk_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
 static int disk_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t hd_offset);
 static int disk_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t hd_offset);
 static int disk_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int disk_sync(disk_t *disk_car);
-static int disk_clean(disk_t *disk_car);
+static void disk_clean(disk_t *disk_car);
 
 static int cmd_dos_segment = 0;
 static int cmd_dos_selector = 0;
@@ -350,7 +352,7 @@ static int hd_identify_enh_bios(disk_t *disk_car,const int verbose)
   return 0;
 }
 
-disk_t *hd_identify(const int verbose, const unsigned int disk, const arch_fnct_t *arch, const int testdisk_mode)
+disk_t *hd_identify(const int verbose, const unsigned int disk, const int testdisk_mode)
 {
   unsigned char buf[0x200];
   memset(buf,0,sizeof(buf));
@@ -368,7 +370,7 @@ disk_t *hd_identify(const int verbose, const unsigned int disk, const arch_fnct_
     data->disk=disk;
     data->bad_geometry=0;
     data->mode_enh=0;
-    disk_car->arch=arch;
+    disk_car->arch=&arch_none;
     snprintf(device,sizeof(device),"/dev/sda%u",disk);
     disk_car->device=strdup(device);
     disk_car->model=NULL;
@@ -377,7 +379,6 @@ disk_t *hd_identify(const int verbose, const unsigned int disk, const arch_fnct_
     disk_car->sector_size=DEFAULT_SECTOR_SIZE;
     disk_car->description=disk_description;
     disk_car->description_short=disk_description_short;
-    disk_car->pread_fast=disk_pread_fast;
     disk_car->pread=disk_pread;
     disk_car->pwrite=((testdisk_mode&TESTDISK_O_RDWR)==TESTDISK_O_RDWR?disk_pwrite:disk_nopwrite);
     disk_car->sync=disk_sync;
@@ -493,13 +494,6 @@ static int disk_pread(disk_t *disk_car, void *buf, const unsigned int count, con
   return align_pread(&disk_pread_aux, disk_car, buf, count, offset);
 }
 
-static void *disk_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
-{
-  if(disk_pread(disk, buf, count, offset)==offset)
-    return buf;
-  return NULL;
-}
-
 static int disk_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t hd_offset)
 {
   int i=0;
@@ -539,7 +533,7 @@ static int disk_sync(disk_t *disk_car)
   return -1;
 }
 
-static int disk_clean(disk_t *disk_car)
+static void disk_clean(disk_t *disk_car)
 {
   /*
   if(disk_car->data!=NULL)
@@ -547,7 +541,7 @@ static int disk_clean(disk_t *disk_car)
     struct info_disk_struct *data=disk_car->data;
   }
   */
-  return generic_clean(disk_car);
+  generic_clean(disk_car);
 }
 
 static int hd_report_error(disk_t *disk_car, const uint64_t hd_offset, const unsigned int count, const int rc)
