@@ -19,15 +19,22 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-typedef struct disk_netware
+struct disk_netware
 {
   char unknown;
   char magic[12];
   char unknown2[3];
   char unknown3[3]; /* 0x10 */
   int32_t nbr_sectors;
-} disk_netware;
+};
 
-int check_netware(disk_t *disk_car,partition_t *partition,const int verbose);
+int check_netware(disk_t *disk_car, partition_t *partition);
 int recover_netware(disk_t *disk_car, const struct disk_netware *netware_block, partition_t *partition);
+
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
