@@ -113,10 +113,11 @@ int recover_NTFS(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,par
 static int set_NTFS_info(disk_t *disk_car, const struct ntfs_boot_sector*ntfs_header,partition_t *partition,const int verbose)
 {
   partition->fsname[0]='\0';
+  partition->blocksize=ntfs_header->sectors_per_cluster*ntfs_sector_size(ntfs_header);
   if(partition->sb_offset==0)
-    strncpy(partition->info, "NTFS", sizeof(partition->info));
+    snprintf(partition->info, sizeof(partition->info), "NTFS, blocksize=%u", partition->blocksize);
   else
-    strncpy(partition->info, "NTFS found using backup sector!", sizeof(partition->info));
+    snprintf(partition->info, sizeof(partition->info), "NTFS found using backup sector, blocksize=%u", partition->blocksize);
   return ntfs_read_MFT(disk_car, partition, ntfs_header, 0x60, verbose);
 }
 
@@ -144,23 +145,23 @@ int test_NTFS(const disk_t *disk_car,const struct ntfs_boot_sector*ntfs_header,
   }
   if(le16(ntfs_header->heads)!=disk_car->geom.heads_per_cylinder)
   {
-    screen_buffer_add("Warning: Incorrect number of heads/cylinder %u (NTFS) != %u (HD)\n",
+    screen_buffer_add("Warning: number of heads/cylinder mismatches %u (NTFS) != %u (HD)\n",
 	le16(ntfs_header->heads), disk_car->geom.heads_per_cylinder);
     log_warning("heads/cylinder %u (NTFS) != %u (HD)\n",
 	le16(ntfs_header->heads), disk_car->geom.heads_per_cylinder);
   }
   if(le16(ntfs_header->secs_track)!=disk_car->geom.sectors_per_head)
   {
-    screen_buffer_add("Warning: Incorrect number of sectors per track %u (NTFS) != %u (HD)\n",
+    screen_buffer_add("Warning: number of sectors per track mismatches %u (NTFS) != %u (HD)\n",
 	le16(ntfs_header->secs_track), disk_car->geom.sectors_per_head);
     log_warning("sect/track %u (NTFS) != %u (HD)\n",
 	le16(ntfs_header->secs_track), disk_car->geom.sectors_per_head);
   }
   if(ntfs_sector_size(ntfs_header)!=disk_car->sector_size)
   {
-    screen_buffer_add("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",
+    screen_buffer_add("Warning: number of bytes per sector mismatches %u (NTFS) != %u (HD)\n",
 	ntfs_sector_size(ntfs_header), disk_car->sector_size);
-    log_warning("Warning: Incorrect number of bytes per sector %u (NTFS) != %u (HD)\n",
+    log_warning("Warning: number of bytes per sector mismatches %u (NTFS) != %u (HD)\n",
 	ntfs_sector_size(ntfs_header), disk_car->sector_size);
   }
 
@@ -204,9 +205,9 @@ int ntfs_get_attr(const char *mft_record, const int my_type, partition_t *partit
 
 static int ntfs_get_attr_aux(const char *attr_record, const int my_type, partition_t *partition, const char *end, const int verbose, const char*file_name_to_find)
 {
-  int attr_type;
   while(1)
   {
+    int attr_type;
     /* Resident attributes attr_len>=24(0x18), non resident is bigger */
     unsigned int attr_len;
     if(attr_record+0x18>=end)
