@@ -58,7 +58,6 @@ struct s_info_mft
   uint64_t mftmirr_lcn;
 };
 
-static int create_ntfs_boot_sector(disk_t *disk_car, partition_t *partition, const int interface, const unsigned int cluster_size, const uint64_t mft_lcn, const uint64_t mftmirr_lcn, const uint32_t mft_record_size, const uint32_t index_block_size, char**current_cmd);
 #ifdef HAVE_NCURSES
 static int ncurses_ntfs2_info(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2);
 static int ncurses_ntfs_info(const struct ntfs_boot_sector *ntfs_header);
@@ -86,13 +85,16 @@ static void ntfs_dump_ncurses(disk_t *disk_car, const partition_t *partition, co
 } 
 #endif
 
-static void ntfs_dump(disk_t *disk_car, const partition_t *partition, const unsigned char *orgboot, const unsigned char *newboot)
+static void ntfs_dump(disk_t *disk_car, const partition_t *partition, const unsigned char *orgboot, const unsigned char *newboot, char **current_cmd)
 {
   log_info("     Rebuild Boot sector           Boot sector\n");
   dump2_log(newboot, orgboot, NTFS_SECTOR_SIZE);
+  if(*current_cmd==NULL)
+  {
 #ifdef HAVE_NCURSES
-  ntfs_dump_ncurses(disk_car, partition, orgboot, newboot);
+    ntfs_dump_ncurses(disk_car, partition, orgboot, newboot);
 #endif
+  }
 }
 
 static void menu_write_ntfs_boot_sector_cli(disk_t *disk_car, partition_t *partition, const unsigned char *orgboot, const unsigned char *newboot, const int error, char **current_cmd)
@@ -124,7 +126,7 @@ static void menu_write_ntfs_boot_sector_cli(disk_t *disk_car, partition_t *parti
     else if(strncmp(*current_cmd,"dump",4)==0)
     {
       (*current_cmd)+=4;
-      ntfs_dump(disk_car, partition, orgboot, newboot);
+      ntfs_dump(disk_car, partition, orgboot, newboot, current_cmd);
     }
     else if(strncmp(*current_cmd,"noconfirm,",10)==0)
     {
@@ -169,7 +171,7 @@ static void menu_write_ntfs_boot_sector_ncurses(disk_t *disk_car, partition_t *p
 {
   const struct ntfs_boot_sector *org_ntfs_header=(const struct ntfs_boot_sector *)orgboot;
   const struct ntfs_boot_sector *ntfs_header=(const struct ntfs_boot_sector *)newboot;
-  struct MenuItem menuSaveBoot[]=
+  const struct MenuItem menuSaveBoot[]=
   {
     { 'D', "Dump", "Dump sector" },
     { 'L', "List", "List directories and files" },
@@ -178,9 +180,9 @@ static void menu_write_ntfs_boot_sector_ncurses(disk_t *disk_car, partition_t *p
     { 0, NULL, NULL }
   };
   const char *options="DLQ";
-  int command;
   while(1)
   {
+    int command;
     aff_copy(stdscr);
     wmove(stdscr,4,0);
     wprintw(stdscr,"%s",disk_car->description(disk_car));
