@@ -71,6 +71,8 @@
 #include "common.h"
 #include "intrf.h"
 #include "ntfs.h"
+#include "list.h"
+#include "list_sort.h"
 #include "dir.h"
 #include "ntfs_dir.h"
 #include "ntfs_utl.h"
@@ -83,7 +85,6 @@
 #define PATH_SEP      '/'
 #define NTFS_DT_DIR               4
 #define NTFS_DT_REG             8
-#define NTFS_TIME_OFFSET ((s64)(369 * 365 + 89) * 24 * 3600 * 10000000)
 #ifndef FILE_first_user
 #define FILE_first_user 16
 #endif
@@ -102,12 +103,11 @@ extern struct ntfs_device_operations ntfs_device_testdisk_io_ops;
 
 extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
                 void *dirent, ntfs_filldir_t filldir);
-static time_t td_ntfs2utc (s64 ntfstime);
 static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
-static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster);
-static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file);
+static int ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster, file_info_t *dir_list);
+static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
 static void dir_partition_ntfs_close(dir_data_t *dir_data);
 
 /**
@@ -133,20 +133,6 @@ static int index_get_size(ntfs_inode *inode)
 	return iroot->index_block_size;
 }
 
-/**
- * td_ntfs2utc - Convert an NTFS time to Unix time
- * @time:  An NTFS time in 100ns units since 1601
- *
- * NTFS stores times as the number of 100ns intervals since January 1st 1601 at
- * 00:00 UTC.  This system will not suffer from Y2K problems until ~57000AD.
- *
- * Return:  n  A Unix time (number of seconds since 1970)
- */
-static time_t td_ntfs2utc (s64 ntfstime)
-{
-  return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
-}
-
 #ifdef HAVE_ICONV
 static int ntfs_ucstoutf8(iconv_t cd, const ntfschar *ins, int ins_len, char **outs, int outs_len)
 {
@@ -180,9 +166,8 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type)
 {
-  char *filename = NULL;
   int result = 0;
-  filename = (char *)calloc (1, MAX_PATH);
+  char *filename = (char *)calloc (1, MAX_PATH);
   if (!filename)
   {
     log_critical("ntfs_td_list_entry calloc failed\n");
@@ -211,40 +196,33 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
   {
     ntfs_inode *ni;
     ntfs_attr_search_ctx *ctx_si = NULL;
-    file_data_t *new_file=NULL;
+    file_info_t *new_file=NULL;
 
     result = -1;				/* Everything else is bad */
 
     ni = ntfs_inode_open(ls->vol, mref);
     if (!ni)
       goto release;
-    new_file=(file_data_t *)MALLOC(sizeof(*new_file));
+    new_file=(file_info_t*)MALLOC(sizeof(*new_file));
     new_file->status=0;
-    new_file->prev=ls->current_file;
-    new_file->next=NULL;
-    new_file->stat.st_dev=0;
-    new_file->stat.st_ino=MREF(mref);
-    new_file->stat.st_nlink=1;
-    new_file->stat.st_uid=0;
-    new_file->stat.st_gid=0;
-    new_file->stat.st_rdev=0;
-#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-    new_file->stat.st_blksize=DEFAULT_SECTOR_SIZE;
-#endif
+    td_list_add_tail(&new_file->list, &ls->dir_list->list);
+    new_file->st_ino=MREF(mref);
+    new_file->st_uid=0;
+    new_file->st_gid=0;
 
     ctx_si = ntfs_attr_get_search_ctx(ni, ni->mrec);
     if (ctx_si)
     {
-      if (ntfs_attr_lookup(AT_STANDARD_INFORMATION, AT_UNNAMED, 0, 0, 0, NULL, 0, ctx_si)==0)
+      if (ntfs_attr_lookup(AT_STANDARD_INFORMATION, AT_UNNAMED, 0, CASE_SENSITIVE, 0, NULL, 0, ctx_si)==0)
       {
 	const ATTR_RECORD *attr = ctx_si->attr;
-	const STANDARD_INFORMATION *si = (STANDARD_INFORMATION*)((char*)attr +
+	const STANDARD_INFORMATION *si = (const STANDARD_INFORMATION*)((const char*)attr +
 	    le16_to_cpu(attr->value_offset));
 	if(si)
 	{
-	  new_file->stat.st_atime=td_ntfs2utc(sle64_to_cpu(si->last_access_time));
-	  new_file->stat.st_mtime=td_ntfs2utc(sle64_to_cpu(si->last_data_change_time));
-	  new_file->stat.st_ctime=td_ntfs2utc(sle64_to_cpu(si->creation_time));
+	  new_file->td_atime=td_ntfs2utc(sle64_to_cpu(si->last_access_time));
+	  new_file->td_mtime=td_ntfs2utc(sle64_to_cpu(si->last_data_change_time));
+	  new_file->td_ctime=td_ntfs2utc(sle64_to_cpu(si->creation_time));
 	}
       }
       ntfs_attr_put_search_ctx(ctx_si);
@@ -255,22 +233,10 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
       ntfs_attr_search_ctx *ctx = NULL;
       if (dt_type == NTFS_DT_DIR)
       {
-	snprintf(new_file->name, sizeof(new_file->name), "%s", filename);
-	new_file->stat.st_mode = LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO;
-	new_file->stat.st_size=0;
-#ifdef DJGPP
-	new_file->file_size=0;
-#endif
-#if defined(HAVE_STRUCT_STAT_ST_BLOCKS)
-	new_file->stat.st_blocks=0;
-#endif
-	new_file->prev=ls->current_file;
-	new_file->next=NULL;
-	if(ls->current_file!=NULL)
-	  ls->current_file->next=new_file;
-	else
-	  ls->dir_list=new_file;
-	ls->current_file=new_file;
+	new_file->name=strdup(filename);
+	new_file->st_mode = LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO;
+	new_file->st_size=0;
+	td_list_add_tail(&new_file->list, &ls->dir_list->list);
 	first=0;
       }
       ctx = ntfs_attr_get_search_ctx(ni, ni->mrec);
@@ -279,48 +245,39 @@ static int ntfs_td_list_entry(  struct ntfs_dir_struct *ls, const ntfschar *name
       while((rec = find_attribute(AT_DATA, ctx)))
       {
 	const s64 filesize = ntfs_get_attribute_value_length(ctx->attr);
+	if(rec->name_length &&
+	    (ls->dir_data->param & FLAG_LIST_ADS)!=FLAG_LIST_ADS)
+	  continue;
 	if(first==0)
 	{
-	  const file_data_t *old_file=new_file;
-	  new_file=(file_data_t *)MALLOC(sizeof(*new_file));
+	  const file_info_t *old_file=new_file;
+	  new_file=(file_info_t *)MALLOC(sizeof(*new_file));
 	  memcpy(new_file, old_file, sizeof(*new_file));
 	}
-	new_file->stat.st_mode = LINUX_S_IFREG | LINUX_S_IRUGO;
-	new_file->stat.st_size=filesize;
-#ifdef DJGPP
-	new_file->file_size=filesize;
-#endif
-#if defined(HAVE_STRUCT_STAT_ST_BLOCKS) && defined(HAVE_STRUCT_STAT_ST_BLKSIZE)
-	if(new_file->stat.st_blksize!=0)
-	  new_file->stat.st_blocks=(filesize + new_file->stat.st_blksize - 1) / new_file->stat.st_blksize;
-#endif
+	new_file->st_mode = LINUX_S_IFREG | LINUX_S_IRUGO;
+	new_file->st_size=filesize;
 	if (rec->name_length)
 	{
 	  char *stream_name=NULL;
+	  new_file->status=FILE_STATUS_ADS;
+	  new_file->name = (char *)MALLOC(MAX_PATH);
 	  if (ntfs_ucstombs((ntfschar *) ((char *) rec + le16_to_cpu(rec->name_offset)),
 		rec->name_length, &stream_name, 0) < 0)
 	  {
 	    log_error("ERROR: Cannot translate name into current locale.\n");
-	    snprintf(new_file->name, sizeof(new_file->name), "%s:???", filename);
+	    snprintf(new_file->name, MAX_PATH, "%s:???", filename);
 	  }
 	  else
 	  {
-	    snprintf(new_file->name, sizeof(new_file->name), "%s:%s", filename, stream_name);
+	    snprintf(new_file->name, MAX_PATH, "%s:%s", filename, stream_name);
 	  }
 	  free(stream_name);
 	}
 	else
 	{
-	  snprintf(new_file->name,sizeof(new_file->name), "%s", filename);
+	  new_file->name=strdup(filename);
 	}
-	new_file->prev=ls->current_file;
-	new_file->next=NULL;
-	if(ls->current_file!=NULL)
-	  ls->current_file->next=new_file;
-	else
-	  ls->dir_list=new_file;
-	ls->current_file=new_file;
-
+	td_list_add_tail(&new_file->list, &ls->dir_list->list);
 	first=0;
       }
       ntfs_attr_put_search_ctx(ctx);
@@ -337,18 +294,17 @@ free:
   return result;
 }
 
-static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster)
+static int ntfs_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int cluster, file_info_t *dir_list)
 {
   ntfs_inode *inode;
   s64 pos;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
-  ls->dir_list=NULL;
-  ls->current_file=NULL;
+  ls->dir_list=dir_list;
 
   inode = ntfs_inode_open (ls->vol, cluster);
   if (!inode) {
     log_error("ntfs_dir: ntfs_inode_open failed\n");
-    return NULL;
+    return -1;
   }
 
   /*
@@ -359,23 +315,26 @@ static file_data_t *ntfs_dir(disk_t *disk_car, const partition_t *partition, dir
   if (inode->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
     if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)ntfs_td_list_entry)<0)
     {
-      log_error("ntfs_readdir failed\n");
+      log_error("ntfs_readdir failed for cluster %lu\n", cluster);
     }
   }
   else
     log_critical("ntfs_readdir BUG not MFT_RECORD_IS_DIRECTORY\n");
   /* Finished with the inode; release it. */
   ntfs_inode_close(inode);
-  return ls->dir_list;
+  td_list_sort(&dir_list->list, filesort);
+  return 0;
 }
 
 enum { bufsize = 4096 };
 
-static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_data_t *file)
+static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file)
 {
-  const unsigned long int first_inode=file->stat.st_ino;
+  const unsigned long int first_inode=file->st_ino;
   ntfs_inode *inode;
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct*)dir_data->private_dir_data;
+  char *stream_name;
+  stream_name=strrchr(dir_data->current_directory, ':');
   inode = ntfs_inode_open (ls->vol, first_inode);
   if (!inode) {
     log_error("ntfs_copy: ntfs_inode_open failed\n");
@@ -386,17 +345,14 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     char *new_file;
     ntfs_attr *attr=NULL;
     FILE *f_out;
-    s64 bytes_read, written;
     s64 offset;
     u32 block_size;
-    const char *stream_name=NULL;
     buffer = (char *)MALLOC(bufsize);
     if (!buffer)
     {
       ntfs_inode_close(inode);
       return -2;
     }
-    stream_name=strrchr(dir_data->current_directory, ':');
     if(stream_name)
       stream_name++;
     if(stream_name != NULL)
@@ -427,7 +383,20 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       block_size = index_get_size(inode);
     else
       block_size = 0;
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+    if(stream_name)
+    {
+      /* fopen() create normal files instead of ADS with ':' replaced by an UTF char
+       * replace ':' by '_' instead */
+      stream_name--;
+      *stream_name='_';
+      f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
+    }
+    else
+      f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
+#else
     f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
+#endif
     if(!f_out)
     {
       log_critical("Can't create file %s: %s\n",new_file, strerror(errno));
@@ -440,6 +409,7 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
     offset = 0;
     for (;;)
     {
+      s64 bytes_read, written;
       if (block_size > 0) {
 	// These types have fixup
 	bytes_read = ntfs_attr_mst_pread(attr, offset, 1, block_size, buffer);
@@ -464,7 +434,7 @@ static int ntfs_copy(disk_t *disk_car, const partition_t *partition, dir_data_t
       offset += bytes_read;
     }
     fclose(f_out);
-    set_date(new_file, file->stat.st_atime, file->stat.st_mtime);
+    set_date(new_file, file->td_atime, file->td_mtime);
     free(new_file);
     ntfs_attr_close(attr);
     free(buffer);
@@ -488,7 +458,7 @@ static void dir_partition_ntfs_close(dir_data_t *dir_data)
 }
 #endif
 
-int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
+dir_partition_t dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const int verbose)
 {
 #if defined(HAVE_LIBNTFS) || defined(HAVE_LIBNTFS3G)
   struct ntfs_device *dev;
@@ -530,7 +500,7 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
   if (!vol) {
     free(my_data);
     ntfs_device_free(dev);
-    return -1;
+    return DIR_PART_EIO;
   }
   if (vol->flags & VOLUME_IS_DIRTY) {
     log_warning("NTFS Volume is dirty.\n");
@@ -538,9 +508,9 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
   {
     struct ntfs_dir_struct *ls=(struct ntfs_dir_struct *)MALLOC(sizeof(*ls));
     ls->dir_list=NULL;
-    ls->current_file=NULL;
     ls->vol=vol;
     ls->my_data=my_data;
+    ls->dir_data=dir_data;
 #ifdef HAVE_ICONV
     if ((ls->cd = iconv_open("UTF-8", "UTF-16LE")) == (iconv_t)(-1))
     {
@@ -549,18 +519,18 @@ int dir_partition_ntfs_init(disk_t *disk_car, const partition_t *partition, dir_
 #endif
     strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
     dir_data->current_inode=FILE_root;
-    dir_data->param=0;
+    dir_data->param=FLAG_LIST_ADS;
     dir_data->verbose=verbose;
-    dir_data->capabilities=0;
+    dir_data->capabilities=CAPA_LIST_ADS;
     dir_data->get_dir=ntfs_dir;
     dir_data->copy_file=ntfs_copy;
     dir_data->close=&dir_partition_ntfs_close;
     dir_data->local_dir=NULL;
     dir_data->private_dir_data=ls;
   }
-  return 0;
+  return DIR_PART_OK;
 #else
-  return -2;
+  return DIR_PART_ENOSYS;
 #endif
 }
 
