@@ -166,20 +166,6 @@ struct ufile {
 	char		 padding[4];	/* Unused: padding to 64 bit. */
 };
 
-/**
- * td_ntfs2utc - Convert an NTFS time to Unix time
- * @time:  An NTFS time in 100ns units since 1601
- *
- * NTFS stores times as the number of 100ns intervals since January 1st 1601 at
- * 00:00 UTC.  This system will not suffer from Y2K problems until ~57000AD.
- *
- * Return:  n  A Unix time (number of seconds since 1970)
- */
-static time_t td_ntfs2utc (s64 ntfstime)
-{
-  return (ntfstime - (NTFS_TIME_OFFSET)) / 10000000;
-}
-
 static const char *UNKNOWN   = "unknown";
 static struct options opts;
 
@@ -306,13 +292,13 @@ static void get_parent_name(struct filename* name, ntfs_volume* vol)
   }
 
   {
-    FILE_NAME_ATTR* filename_attr;
     long long inode_num;
     int ok;
     inode_num = MREF(name->parent_mref);
     name->parent_name = NULL;
     do
     {
+      FILE_NAME_ATTR* filename_attr;
       ok=0;
       if (ntfs_attr_pread(mft_data, vol->mft_record_size * inode_num, vol->mft_record_size, rec) < 1)
       {
@@ -628,12 +614,7 @@ static struct ufile * read_record(ntfs_volume *vol, long long record)
  */
 static int calc_percentage(struct ufile *file, ntfs_volume *vol)
 {
-	runlist_element *rl = NULL;
 	struct td_list_head *pos;
-	struct data *data;
-	long long i, j;
-	long long start, end;
-	int clusters_inuse, clusters_free;
 	int percent = 0;
 
 	if (!file || !vol)
@@ -648,6 +629,11 @@ static int calc_percentage(struct ufile *file, ntfs_volume *vol)
 	}
 
 	td_list_for_each(pos, &file->data) {
+		runlist_element *rl = NULL;
+		long long i;
+		long long start, end;
+	  	int clusters_inuse, clusters_free;
+		struct data *data;
 		data  = td_list_entry(pos, struct data, list);
 		clusters_inuse = 0;
 		clusters_free  = 0;
@@ -686,6 +672,7 @@ static int calc_percentage(struct ufile *file, ntfs_volume *vol)
 		}
 
 		for (i = 0; rl[i].length > 0; i++) {
+			long long j;
 			if (rl[i].lcn == LCN_RL_NOT_MAPPED) {
 				log_debug("Missing segment at end, %lld clusters\n",
 						(long long)rl[i].length);
@@ -1097,29 +1084,14 @@ static file_info_t *ufile_to_file_data(const struct ufile *file, const struct da
       (file->pref_name?file->pref_name:inode_name),
       (d->name?":":""),
       (d->name?d->name:""));
-  new_file->stat.st_dev=0;
-  new_file->stat.st_ino=file->inode;
-  new_file->stat.st_mode = (file->directory ?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
-  new_file->stat.st_nlink=0;
-  new_file->stat.st_uid=0;
-  new_file->stat.st_gid=0;
-  new_file->stat.st_rdev=0;
+  new_file->st_ino=file->inode;
+  new_file->st_mode = (file->directory ?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
+  new_file->st_uid=0;
+  new_file->st_gid=0;
 
-  new_file->stat.st_size=max(d->size_init, d->size_data);
-#ifdef DJGPP
-  new_file->file_size=max(d->size_init, d->size_data);
-#endif
-#ifdef HAVE_STRUCT_STAT_ST_BLKSIZE
-  new_file->stat.st_blksize=512;	/* FIXME cluster_size */
-#ifdef HAVE_STRUCT_STAT_ST_BLOCKS
-  if(new_file->stat.st_blksize!=0)
-  {
-    new_file->stat.st_blocks=(file->max_size + new_file->stat.st_blksize - 1) / new_file->stat.st_blksize;
-  }
-#endif
-#endif
-  new_file->stat.st_atime=new_file->stat.st_ctime=new_file->stat.st_mtime=file->date;
-  new_file->status=FILE_STATUS_DELETED;
+  new_file->st_size=max(d->size_init, d->size_data);
+  new_file->td_atime=new_file->td_ctime=new_file->td_mtime=file->date;
+  new_file->status=0;
   return new_file;
 }
 
@@ -1216,7 +1188,35 @@ done:
 #ifdef HAVE_NCURSES
 #define INTER_DIR (LINES-25+16)
 
-static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir_list)
+static struct td_list_head *ntfs_next_non_deleted(struct td_list_head *current_file, const struct td_list_head *end)
+{
+  struct td_list_head *walker=current_file;
+  while(walker->next!=end)
+  {
+    const file_info_t *file_info;
+    walker=walker->next;
+    file_info=td_list_entry(walker, file_info_t, list);
+    if((file_info->status&FILE_STATUS_DELETED)==0)
+      return walker;
+  }
+  return current_file;
+}
+
+static struct td_list_head *ntfs_prev_non_deleted(struct td_list_head *current_file, const struct td_list_head *start)
+{
+  struct td_list_head *walker=current_file;
+  while(walker->prev!=start)
+  {
+    const file_info_t *file_info;
+    walker=walker->prev;
+    file_info=td_list_entry(walker, file_info_t, list);
+    if((file_info->status&FILE_STATUS_DELETED)==0)
+      return walker;
+  }
+  return current_file;
+}
+
+static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, file_info_t *dir_list)
 {
   struct ntfs_dir_struct *ls=(struct ntfs_dir_struct *)dir_data->private_dir_data;
   WINDOW *window=(WINDOW*)dir_data->display;
@@ -1234,60 +1234,61 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
     do
     {
       struct td_list_head *file_walker = NULL;
-      int i=0;
+      int i;
       int car;
+      for(i=5; i<=6+INTER_DIR; i++)
+      {
+	wmove(window, i, 0);
+	wclrtoeol(window);	/* before addstr for BSD compatibility */
+      }
+      i=0;
       td_list_for_each(file_walker,&dir_list->list)
       {
+	char		datestr[80];
+	file_info_t *file_info;
+	file_info=td_list_entry(file_walker, file_info_t, list);
+	if((file_info->status&FILE_STATUS_DELETED)!=0)
+	  continue;
 	if(i++<offset)
 	  continue;
+	wmove(window, 6-1+i-offset, 0);
+	wclrtoeol(window);	/* before addstr for BSD compatibility */
+	if(file_walker==current_file)
 	{
-	  file_info_t *file_info;
-	  struct tm		*tm_p;
-	  char		datestr[80];
-	  file_info=td_list_entry(file_walker, file_info_t, list);
-	  wmove(window, 6-1+i-offset, 0);
-	  wclrtoeol(window);	/* before addstr for BSD compatibility */
-	  if(file_walker==current_file)
-	  {
-	    wattrset(window, A_REVERSE);
-	    waddstr(window, ">");
-	  }
-	  else
-	    waddstr(window, " ");
-	  if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
-	    wbkgdset(window,' ' | COLOR_PAIR(2));
-	  if(file_info->stat.st_mtime!=0)
-	  {
-	    tm_p = localtime(&file_info->stat.st_mtime);
-	    snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
-		tm_p->tm_mday, monstr[tm_p->tm_mon],
-		1900 + tm_p->tm_year, tm_p->tm_hour,
-		tm_p->tm_min);
-	    /* May have to use %d instead of %e */
-	  } else {
-	    strncpy(datestr, "                 ",sizeof(datestr));
-	  }
-	  if(COLS <= 1+17+1+9+1)
-	    wprintw(window, "%s", file_info->name);
-	  else
-	  {
-	    const unsigned int nbr=COLS - (1+17+1+9+1);
-	    if(strlen(file_info->name) < nbr)
-	      wprintw(window, "%-*s", nbr, file_info->name);
-	    else
-	      wprintw(window, "%-*s", nbr, &file_info->name[strlen(file_info->name) - nbr]);
-	  }
-	  wprintw(window, " %s ", datestr);
-#ifdef DJGPP
-	  wprintw(window, "%9llu", (long long unsigned int)file_info->file_size);
-#else
-	  wprintw(window, "%9llu", (long long unsigned int)file_info->stat.st_size);
-#endif
-	  if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
-	    wbkgdset(window,' ' | COLOR_PAIR(0));
-	  if(file_walker==current_file)
-	    wattroff(window, A_REVERSE);
+	  wattrset(window, A_REVERSE);
+	  waddstr(window, ">");
 	}
+	else
+	  waddstr(window, " ");
+	if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
+	  wbkgdset(window,' ' | COLOR_PAIR(2));
+	if(file_info->td_mtime!=0)
+	{
+	  const struct tm *tm_p= localtime(&file_info->td_mtime);
+	  snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
+	      tm_p->tm_mday, monstr[tm_p->tm_mon],
+	      1900 + tm_p->tm_year, tm_p->tm_hour,
+	      tm_p->tm_min);
+	  /* May have to use %d instead of %e */
+	} else {
+	  strncpy(datestr, "                 ",sizeof(datestr));
+	}
+	if(COLS <= 1+17+1+9+1)
+	  wprintw(window, "%s", file_info->name);
+	else
+	{
+	  const unsigned int nbr=COLS - (1+17+1+9+1);
+	  if(strlen(file_info->name) < nbr)
+	    wprintw(window, "%-*s", nbr, file_info->name);
+	  else
+	    wprintw(window, "%-*s", nbr, &file_info->name[strlen(file_info->name) - nbr]);
+	}
+	wprintw(window, " %s ", datestr);
+	wprintw(window, "%9llu", (long long unsigned int)file_info->st_size);
+	if((file_info->status&FILE_STATUS_MARKED)!=0 && has_colors())
+	  wbkgdset(window,' ' | COLOR_PAIR(0));
+	if(file_walker==current_file)
+	  wattroff(window, A_REVERSE);
 	if(offset+INTER_DIR<=i)
 	  break;
       }
@@ -1359,56 +1360,118 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
       {
 	case KEY_UP:
 	case '8':
-	  if(current_file->prev!=&dir_list->list)
+	  file_walker=ntfs_prev_non_deleted(current_file, &dir_list->list);
+	  if(current_file!=file_walker)
 	  {
-	    current_file=current_file->prev;
+	    current_file=file_walker;
 	    pos_num--;
 	  }
 	  break;
 	case KEY_DOWN:
 	case '2':
-	  if(current_file->next!=&dir_list->list)
+	  file_walker=ntfs_next_non_deleted(current_file, &dir_list->list);
+	  if(current_file!=file_walker)
 	  {
-	    current_file=current_file->next;
+	    current_file=file_walker;
 	    pos_num++;
 	  }
 	  break;
 	case KEY_PPAGE:
-	  for(i=0; i<INTER_DIR-1 && current_file->prev!=&dir_list->list; i++)
+	  for(i=0; i<INTER_DIR-1; i++)
 	  {
-	    current_file=current_file->prev;
-	    pos_num--;
+	    file_walker=ntfs_prev_non_deleted(current_file, &dir_list->list);
+	    if(current_file!=file_walker)
+	    {
+	      current_file=file_walker;
+	      pos_num--;
+	    }
+	    else
+	      i=INTER_DIR;
 	  }
 	  break;
 	case KEY_NPAGE:
-	  for(i=0; i<INTER_DIR-1 && current_file->next!=&dir_list->list; i++)
+	  for(i=0; i<INTER_DIR-1;  i++)
 	  {
-	    current_file=current_file->next;
-	    pos_num++;
+	    file_walker=ntfs_next_non_deleted(current_file, &dir_list->list);
+	    if(current_file!=file_walker)
+	    {
+	      current_file=file_walker;
+	      pos_num++;
+	    }
+	    else
+	      i=INTER_DIR;
 	  }
 	  break;
 	case 'a':
 	  {
-	    int status;
+	    unsigned int status;
 	    file_info_t *file_info;
 	    file_info=td_list_entry(current_file, file_info_t, list);
 	    status=(file_info->status^FILE_STATUS_MARKED)&FILE_STATUS_MARKED;
 	    td_list_for_each(file_walker,&dir_list->list)
 	    {
 	      file_info=td_list_entry(file_walker, file_info_t, list);
-	      if((file_info->status & FILE_STATUS_MARKED)!=status)
+	      if((file_info->status&FILE_STATUS_DELETED)==0 &&
+		  (file_info->status & FILE_STATUS_MARKED)!=status)
 		file_info->status^=FILE_STATUS_MARKED;
 	    }
 	  }
 	  break;
+	case 'f':
+	  {
+	    const char *needle=ask_string_ncurses("Filename filter ");
+	    if(needle!=NULL && needle[0]!='\0')
+	    {
+	      td_list_for_each(file_walker,&dir_list->list)
+	      {
+		file_info_t *file_info;
+		file_info=td_list_entry(file_walker, file_info_t, list);
+		if((file_info->status&FILE_STATUS_DELETED)==0 &&
+		    strcasestr(file_info->name, needle)==NULL)
+		  file_info->status|=FILE_STATUS_DELETED;
+	      }
+	      pos_num=0;
+	      current_file=ntfs_next_non_deleted(&dir_list->list, &dir_list->list);
+	    }
+	  }
+	  break;
+	case 'r':
+	  td_list_for_each(file_walker,&dir_list->list)
+	  {
+	    file_info_t *file_info;
+	    file_info=td_list_entry(file_walker, file_info_t, list);
+	    file_info->status&=~FILE_STATUS_DELETED;
+	  }
+	  pos_num=0;
+	  current_file=dir_list->list.next;
+	  break;
+	case 's':
+	  {
+	    uint64_t min_size=ask_int_ncurses("Minimum file size ");
+	    if(min_size>0)
+	    {
+	      td_list_for_each(file_walker,&dir_list->list)
+	      {
+		file_info_t *file_info;
+		file_info=td_list_entry(file_walker, file_info_t, list);
+		if((file_info->status&FILE_STATUS_DELETED)==0 &&
+		    file_info->st_size < min_size)
+		  file_info->status|=FILE_STATUS_DELETED;
+	      }
+	      pos_num=0;
+	      current_file=ntfs_next_non_deleted(&dir_list->list, &dir_list->list);
+	    }
+	  }
+	  break;
 	case ':':
 	  {
 	    file_info_t *file_info;
 	    file_info=td_list_entry(current_file, file_info_t, list);
 	    file_info->status^=FILE_STATUS_MARKED;
-	    if(current_file->next!=&dir_list->list)
+	    file_walker=ntfs_next_non_deleted(current_file, &dir_list->list);
+	    if(current_file!=file_walker)
 	    {
-	      current_file=current_file->next;
+	      current_file=file_walker;
 	      pos_num++;
 	    }
 	  }
@@ -1418,12 +1481,12 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 	    file_info_t *file_info;
 	    file_info=td_list_entry(current_file, file_info_t, list);
 	    if(current_file!=&dir_list->list &&
-		LINUX_S_ISDIR(file_info->stat.st_mode)==0)
+		LINUX_S_ISDIR(file_info->st_mode)==0)
 	    {
 	      if(dir_data->local_dir==NULL)
 	      {
 		char *res;
-		if(LINUX_S_ISDIR(file_info->stat.st_mode)!=0)
+		if(LINUX_S_ISDIR(file_info->st_mode)!=0)
 		  res=ask_location("Please select a destination where %s and any files below will be copied.",
 		      file_info->name, NULL);
 		else
@@ -1443,7 +1506,7 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 		if(has_colors())
 		  wbkgdset(window,' ' | COLOR_PAIR(0));
 		wrefresh(window);
-		res=undelete_file(ls->vol, file_info->stat.st_ino);
+		res=undelete_file(ls->vol, file_info->st_ino);
 		wmove(window,5,0);
 		wclrtoeol(window);
 		if(res < -1)
@@ -1491,7 +1554,7 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 	      file_info=td_list_entry(file_walker, file_info_t, list);
 	      if((file_info->status&FILE_STATUS_MARKED)!=0)
 	      {
-		if(undelete_file(ls->vol, file_info->stat.st_ino) < 0)
+		if(undelete_file(ls->vol, file_info->st_ino) < 0)
 		  file_bad++;
 		else
 		{
@@ -1534,7 +1597,7 @@ static void ntfs_undelete_menu_ncurses(disk_t *disk_car, const partition_t *part
 }
 #endif
 
-static void ntfs_undelete_menu(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir_list, char**current_cmd)
+static void ntfs_undelete_menu(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, file_info_t *dir_list, char**current_cmd)
 {
   log_list_file(disk_car, partition, dir_data, dir_list);
   if(*current_cmd!=NULL)
@@ -1599,14 +1662,14 @@ int ntfs_undelete_part(disk_t *disk_car, const partition_t *partition, const int
       break;
     default:
       {
-	static file_info_t dir_list = {
+	file_info_t dir_list = {
 	  .list = TD_LIST_HEAD_INIT(dir_list.list),
 	  .name = NULL
 	};
 	struct ntfs_dir_struct *ls=(struct ntfs_dir_struct *)dir_data.private_dir_data;
 	scan_disk(ls->vol, &dir_list);
 	ntfs_undelete_menu(disk_car, partition, &dir_data, &dir_list, current_cmd);
-	delete_list_file_info(&dir_list.list);
+	delete_list_file(&dir_list);
 	dir_data.close(&dir_data);
       }
       break;
