@@ -1293,22 +1293,35 @@ list_part_t *add_partition_i386_cli(disk_t *disk_car, list_part_t *list_part, ch
 
 static void set_next_status_i386(const disk_t *disk_car, partition_t *partition)
 {
-  if(partition->status==STATUS_LOG)
-    partition->status=STATUS_DELETED;
-  else
-    partition->status++;
-  if(partition->status==STATUS_LOG && parti386_can_be_ext(disk_car,partition)==0)
-    partition->status=STATUS_DELETED;
+  /* STATUS_DELETED, STATUS_PRIM, STATUS_PRIM_BOOT, STATUS_LOG */
+  switch(partition->status)
+  {
+    case STATUS_PRIM_BOOT:
+      if(parti386_can_be_ext(disk_car,partition)!=0)
+	partition->status=STATUS_LOG;
+      else
+	partition->status=STATUS_DELETED;
+      break;
+    case STATUS_LOG:		partition->status=STATUS_DELETED; break;
+    case STATUS_DELETED:	partition->status=STATUS_PRIM; break;
+    default:			partition->status=STATUS_PRIM_BOOT; break;
+  }
 }
 
 static void set_prev_status_i386(const disk_t *disk_car, partition_t *partition)
 {
-  if(partition->status==STATUS_DELETED)
-    partition->status=STATUS_LOG;
-  else
-    partition->status--;
-  if(partition->status==STATUS_LOG && parti386_can_be_ext(disk_car,partition)==0)
-    partition->status--;
+  switch(partition->status)
+  {
+    case STATUS_DELETED:
+      if(parti386_can_be_ext(disk_car,partition)!=0)
+	partition->status=STATUS_LOG;
+      else
+	partition->status=STATUS_PRIM_BOOT;
+      break;
+    case STATUS_LOG:		partition->status=STATUS_PRIM_BOOT; break;
+    case STATUS_PRIM_BOOT:	partition->status=STATUS_PRIM; break;
+    default:			partition->status=STATUS_DELETED; break;
+  }
 }
 
 static int set_part_type_i386(partition_t *partition, unsigned int part_type)
@@ -1328,7 +1341,6 @@ static int is_part_known_i386(const partition_t *partition)
 
 static void init_structure_i386(const disk_t *disk_car,list_part_t *list_part, const int verbose)
 {
-  unsigned int nbr_prim=0, nbr_log_block=0;
   unsigned int vista_partition=0;
   list_part_t *element;
   list_part_t *new_list_part=NULL;
@@ -1365,7 +1377,9 @@ static void init_structure_i386(const disk_t *disk_car,list_part_t *list_part, c
   {
     /* log_block_size must be 0 or 1 for a valid partition table */
     unsigned int log_block_size=0;
+    unsigned int nbr_log_block=0;
     unsigned int biggest_log_block_size=0;
+    unsigned int nbr_prim=0;
     list_part_t *end_log_block=NULL;
     list_part_t *end_biggest_log_block=NULL;
     /* Verify */
