@@ -52,7 +52,8 @@
 #include "hidden.h"
 #include "hiddenn.h"
 #include "nodisk.h"
-#include "chgtypen.h"
+#include "chgarch.h"
+#include "chgarchn.h"
 
 #ifdef HAVE_NCURSES
 #define NBR_DISK_MAX 		(LINES-6-8)
@@ -61,6 +62,8 @@
 #define INTER_NOTE_Y		(LINES-4)
 #endif
 
+extern const arch_fnct_t arch_none;
+
 static int photorec_disk_selection_cli(struct ph_param *params, struct ph_options *options, const list_disk_t *list_disk, alloc_data_t *list_search_space)
 {
   const list_disk_t *element_disk;
@@ -90,10 +93,16 @@ static int photorec_disk_selection_cli(struct ph_param *params, struct ph_option
       current_search_space->end=current_search_space->end*disk->sector_size+disk->sector_size-1;
     }
   }
-  autodetect_arch(disk);
+  autodetect_arch(disk, &arch_none);
   params->disk=disk;
-  if(interface_partition_type(disk, options->verbose, &params->cmd_run)==0)
+  if(change_arch_type_cli(disk, options->verbose, &params->cmd_run)==0
+#ifdef HAVE_NCURSES
+      || change_arch_type_ncurses(disk, options->verbose)==0
+#endif
+    )
+  {
     menu_photorec(params, options, list_search_space);
+  }
   return 0;
 }
 
@@ -266,11 +275,11 @@ static int photorec_disk_selection_ncurses(struct ph_param *params, struct ph_op
 	{
 	  disk_t *disk=current_disk->disk;
 	  const int hpa_dco=is_hpa_or_dco(disk);
-	  autodetect_arch(disk);
+	  autodetect_arch(disk, &arch_none);
 	  params->disk=disk;
 	  if((hpa_dco==0 || interface_check_hidden_ncurses(disk, hpa_dco)==0) &&
 	      (options->expert == 0 ||
-	       interface_partition_type(disk, options->verbose, &params->cmd_run)==0))
+	       change_arch_type_ncurses(disk, options->verbose)==0))
 	    menu_photorec(params, options, list_search_space);
 	}
 	break;
@@ -294,13 +303,16 @@ int do_curses_photorec(struct ph_param *params, struct ph_options *options, cons
   static alloc_data_t list_search_space={
     .list = TD_LIST_HEAD_INIT(list_search_space.list)
   };
-#ifdef HAVE_NCURSES
   if(params->cmd_device==NULL)
   {
     char *saved_device=NULL;
     char *saved_cmd=NULL;
     session_load(&saved_device, &saved_cmd,&list_search_space);
-    if(saved_device!=NULL && saved_cmd!=NULL && !td_list_empty(&list_search_space.list) && ask_confirmation("Continue previous session ? (Y/N)")!=0)
+    if(saved_device!=NULL && saved_cmd!=NULL && !td_list_empty(&list_search_space.list)
+#ifdef HAVE_NCURSES
+	&& ask_confirmation("Continue previous session ? (Y/N)")!=0
+#endif
+      )
     {
       /* yes */
       params->cmd_run=saved_cmd;
@@ -313,7 +325,6 @@ int do_curses_photorec(struct ph_param *params, struct ph_options *options, cons
       free_list_search_space(&list_search_space);
     }
   }
-#endif
   if(params->cmd_device!=NULL && params->cmd_run!=NULL)
     return photorec_disk_selection_cli(params, options, list_disk, &list_search_space);
 #ifdef HAVE_NCURSES
