@@ -35,6 +35,12 @@
 #include <string.h>
 #endif
 #include <errno.h>
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -108,6 +114,7 @@ static void list_free_add(const file_recovery_t *file_recovery, alloc_data_t *li
       new_free_space->start=file_recovery->location.start;
       new_free_space->end=current_search_space->end;
       new_free_space->file_stat=NULL;
+      new_free_space->data=1;
       current_search_space->end=file_recovery->location.start-1;
       td_list_add(&new_free_space->list, search_walker);
     }
@@ -257,6 +264,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
       new_free_space->start=start;
       new_free_space->end=current_search_space->end;
       new_free_space->file_stat=NULL;
+      new_free_space->data=1;
       current_search_space->end=start-1;
       td_list_add(&new_free_space->list,search_walker);
       if(offset!=NULL && new_current_search_space!=NULL &&
@@ -281,6 +289,7 @@ void init_search_space(alloc_data_t *list_search_space, const disk_t *disk_car,
   if(new_sp->end > disk_car->disk_real_size-1)
     new_sp->end = disk_car->disk_real_size-1;
   new_sp->file_stat=NULL;
+  new_sp->data=1;
   new_sp->list.prev=&new_sp->list;
   new_sp->list.next=&new_sp->list;
   td_list_add_tail(&new_sp->list, &list_search_space->list);
@@ -477,31 +486,6 @@ int sorfile_stat_ts(const void *p1, const void *p2)
   return 0;
 }
 
-void write_stats_stdout(const file_stat_t *file_stats)
-{
-  int i;
-  unsigned int file_nbr=0;
-  for(i=0;file_stats[i].file_hint!=NULL;i++)
-  {
-    if(file_stats[i].recovered+file_stats[i].not_recovered>0)
-    {
-      file_nbr+=file_stats[i].recovered;
-      printf("%s: %u/%u recovered\n",
-          (file_stats[i].file_hint->extension!=NULL?
-           file_stats[i].file_hint->extension:""),
-          file_stats[i].recovered, file_stats[i].recovered+file_stats[i].not_recovered);
-    }
-  }
-  if(file_nbr>1)
-  {
-    printf("Total: %u files found\n\n",file_nbr);
-  }
-  else
-  {
-    printf("Total: %u file found\n\n",file_nbr);
-  }
-}
-
 partition_t *new_whole_disk(const disk_t *disk_car)
 {
   partition_t *fake_partition;
@@ -526,8 +510,7 @@ unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int
     run_again=0;
     td_list_for_each(search_walker, &list_search_space->list)
     {
-      alloc_data_t *tmp;
-      tmp=td_list_entry(search_walker, alloc_data_t, list);
+      const alloc_data_t *tmp=td_list_entry(search_walker, alloc_data_t, list);
       if(tmp->file_stat!=NULL)
       {
 	if(tmp->start%blocksize!=*offset && blocksize>default_blocksize)
@@ -548,10 +531,14 @@ void update_blocksize(unsigned int blocksize, alloc_data_t *list_search_space, c
   struct td_list_head *search_walker_next = NULL;
   td_list_for_each_safe(search_walker,search_walker_next,&list_search_space->list)
   {
+    uint64_t old_start;
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    old_start=current_search_space->start;
     current_search_space->start=(current_search_space->start-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize;
-    if(current_search_space->start>current_search_space->end)
+    if(current_search_space->start!=old_start)
+      current_search_space->file_stat=NULL;
+    if(current_search_space->start>=current_search_space->end)
     {
       td_list_del(search_walker);
       free(current_search_space);
@@ -574,6 +561,66 @@ static void free_list_allocation(alloc_list_t *list_allocation)
     free(allocated_space);
   }
 }
+/* file_finish_aux()
+    @param file_recovery - handle!=NULL
+    @param struct ph_param *params
+*/
+
+static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *params, const int paranoid)
+{
+  if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
+      params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
+  {
+    if(file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && paranoid>0)
+    { /* Check if recovered file is valid */
+      file_recovery->file_check(file_recovery);
+    }
+    /* FIXME: need to adapt read_size to volume size to avoid this */
+    if(file_recovery->file_size > params->disk->disk_size)
+      file_recovery->file_size = params->disk->disk_size;
+    if(file_recovery->file_size > params->disk->disk_real_size)
+      file_recovery->file_size = params->disk->disk_real_size;
+    if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
+	file_recovery->file_size < file_recovery->min_filesize)
+    { 
+      log_info("File too small ( %llu < %llu), reject it\n",
+	  (long long unsigned) file_recovery->file_size,
+	  (long long unsigned) file_recovery->min_filesize);
+      file_recovery->file_size=0;
+      file_recovery->file_size_on_disk=0;
+    }
+  }
+  if(file_recovery->file_size==0)
+  {
+    fclose(file_recovery->handle);
+    file_recovery->handle=NULL;
+    /* File is zero-length; erase it */
+    unlink(file_recovery->filename);
+  }
+  else
+  {
+#ifdef HAVE_FTRUNCATE
+    fflush(file_recovery->handle);
+    if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
+    {
+      log_critical("ftruncate failed.\n");
+    }
+#endif
+    fclose(file_recovery->handle);
+    file_recovery->handle=NULL;
+    if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
+      set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
+    if(file_recovery->file_rename!=NULL)
+      file_recovery->file_rename(file_recovery->filename);
+    if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
+    {
+      params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
+    }
+    if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
+	params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
+      file_recovery->file_stat->recovered++;
+  }
+}
 
 /** file_finish()
     @param file_recovery - 
@@ -601,51 +648,7 @@ int file_finish(file_recovery_t *file_recovery, struct ph_param *params,
   info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
 #endif
   if(file_recovery->handle)
-  {
-    if(file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL)
-    { /* Check if recovered file is valid */
-      file_recovery->file_check(file_recovery);
-    }
-    /* FIXME: need to adapt read_size to volume size to avoid this */
-    if(file_recovery->file_size > params->disk->disk_size)
-      file_recovery->file_size = params->disk->disk_size;
-    if(file_recovery->file_size > params->disk->disk_real_size)
-      file_recovery->file_size = params->disk->disk_real_size;
-    if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
-	file_recovery->file_size < file_recovery->min_filesize)
-    { 
-      log_info("File too small ( %llu < %llu), reject it\n",
-	  (long long unsigned) file_recovery->file_size,
-	  (long long unsigned) file_recovery->min_filesize);
-      file_recovery->file_size=0;
-      file_recovery->file_size_on_disk=0;
-    }
-#ifdef HAVE_FTRUNCATE
-    fflush(file_recovery->handle);
-    if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
-    {
-      log_critical("ftruncate failed.\n");
-    }
-#endif
-    fclose(file_recovery->handle);
-    file_recovery->handle=NULL;
-    //    log_debug("%s %llu\n",file_recovery->filename,(long long unsigned)file_recovery->file_size);
-    if(file_recovery->file_size==0)
-    {
-      /* File is zero-length; erase it */
-      unlink(file_recovery->filename);
-    }
-    else
-    {
-      if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
-	set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
-      if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
-      {
-        params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
-      }
-      file_recovery->file_stat->recovered++;
-    }
-  }
+    file_finish_aux(file_recovery, params, 1);
   if(file_recovery->file_stat!=NULL)
   {
     list_truncate(&file_recovery->location,file_recovery->file_size);
@@ -703,60 +706,7 @@ int file_finish2(file_recovery_t *file_recovery, struct ph_param *params, const
   info_list_search_space(list_search_space, NULL, DEFAULT_SECTOR_SIZE, 0, 1);
 #endif
   if(file_recovery->handle)
-  {
-    if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
-	params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
-    {
-      if(file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && options->paranoid>0)
-      { /* Check if recovered file is valid */
-        file_recovery->file_check(file_recovery);
-      }
-      /* FIXME: need to adapt read_size to volume size to avoid this */
-      if(file_recovery->file_size > params->disk->disk_size)
-        file_recovery->file_size = params->disk->disk_size;
-      if(file_recovery->file_size > params->disk->disk_real_size)
-        file_recovery->file_size = params->disk->disk_real_size;
-
-      if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
-          file_recovery->file_size < file_recovery->min_filesize)
-      { 
-        log_info("File too small ( %llu < %llu), reject it\n",
-            (long long unsigned) file_recovery->file_size,
-            (long long unsigned) file_recovery->min_filesize);
-        file_recovery->file_size=0;
-        file_recovery->file_size_on_disk=0;
-      }
-#ifdef HAVE_FTRUNCATE
-      fflush(file_recovery->handle);
-      if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
-      {
-        log_critical("ftruncate failed.\n");
-      }
-#endif
-    }
-    fclose(file_recovery->handle);
-    file_recovery->handle=NULL;
-    if(file_recovery->file_size==0)
-    {
-      unlink(file_recovery->filename);
-    }
-    else
-    {
-      if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
-	set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
-      if(file_recovery->file_rename!=NULL)
-	file_recovery->file_rename(file_recovery->filename);
-      if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
-      {
-        params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
-      }
-      if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
-	  params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING)
-      {
-        file_recovery->file_stat->recovered++;
-      }
-    }
-  }
+    file_finish_aux(file_recovery, params, options->paranoid);
   if(file_recovery->file_stat!=NULL)
   {
     if(file_recovery->file_size==0)
@@ -919,6 +869,7 @@ static alloc_data_t *file_error_aux(alloc_data_t *space, alloc_data_t *file, con
 	  memcpy(new_element, element, sizeof(*new_element));
 	  new_element->start+=file_size_on_disk - size;
 	  new_element->file_stat=NULL;
+	  new_element->data=1;
 	  td_list_add(&new_element->list, &element->list);
 	  element->end=new_element->start - 1;
 	  return new_element;
@@ -981,7 +932,8 @@ static void set_search_start_aux(alloc_data_t **new_current_search_space, alloc_
     }
   }
   /* not found */
-  *new_current_search_space=list_search_space;
+  search_walker=list_search_space->list.next;
+  *new_current_search_space=td_list_entry(search_walker, alloc_data_t, list);
 }
 
 uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_search_space, alloc_data_t *list_search_space)
@@ -1008,4 +960,101 @@ uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_se
   return offset;
 }
 
+void params_reset(struct ph_param *params, const struct ph_options *options)
+{
+  params->file_nbr=0;
+  params->status=STATUS_FIND_OFFSET;
+  params->real_start_time=time(NULL);
+  params->dir_num=1;
+  params->file_stats=init_file_stats(options->list_file_format);
+  params->offset=-1;
+  if(params->blocksize==0)
+    params->blocksize=params->disk->sector_size;
+}
 
+const char *status_to_name(const photorec_status_t status)
+{
+  switch(status)
+  {
+    case STATUS_UNFORMAT:
+      return "STATUS_UNFORMAT";
+    case STATUS_FIND_OFFSET:
+      return "STATUS_FIND_OFFSET";
+    case STATUS_EXT2_ON:
+      return "STATUS_EXT2_ON";
+    case STATUS_EXT2_ON_BF:			
+      return "STATUS_EXT2_ON_BF";
+    case STATUS_EXT2_OFF:
+      return "STATUS_EXT2_OFF";
+    case STATUS_EXT2_OFF_BF:
+      return "STATUS_EXT2_OFF_BF";
+    case STATUS_EXT2_ON_SAVE_EVERYTHING:
+      return "STATUS_EXT2_ON_SAVE_EVERYTHING";
+    case STATUS_EXT2_OFF_SAVE_EVERYTHING:
+      return "STATUS_EXT2_OFF_SAVE_EVERYTHING";
+    case STATUS_QUIT :
+    default:
+      return "STATUS_QUIT";
+  }
+}
+
+void status_inc(struct ph_param *params, const struct ph_options *options)
+{
+  params->offset=-1;
+  switch(params->status)
+  {
+    case STATUS_UNFORMAT:
+      params->status=STATUS_FIND_OFFSET;
+      break;
+    case STATUS_FIND_OFFSET:
+      params->status=(options->mode_ext2>0?STATUS_EXT2_ON:STATUS_EXT2_OFF);
+      params->file_nbr=0;
+      break;
+    case STATUS_EXT2_ON:
+      if(options->paranoid>1)
+	params->status=STATUS_EXT2_ON_BF;
+      else if(options->paranoid==1 && options->keep_corrupted_file>0)
+	params->status=STATUS_EXT2_ON_SAVE_EVERYTHING;
+      else
+	params->status=STATUS_QUIT;
+      break;
+    case STATUS_EXT2_ON_BF:
+      if(options->keep_corrupted_file>0)
+	params->status=STATUS_EXT2_ON_SAVE_EVERYTHING;
+      else
+	params->status=STATUS_QUIT;
+      break;
+    case STATUS_EXT2_OFF:
+      if(options->paranoid>1)
+	params->status=STATUS_EXT2_OFF_BF;
+      else if(options->paranoid==1 && options->keep_corrupted_file>0)
+	params->status=STATUS_EXT2_OFF_SAVE_EVERYTHING;
+      else
+	params->status=STATUS_QUIT;
+      break;
+    case STATUS_EXT2_OFF_BF:
+      if(options->keep_corrupted_file>0)
+	params->status=STATUS_EXT2_OFF_SAVE_EVERYTHING;
+      else
+	params->status=STATUS_QUIT;
+      break;
+    default:
+      params->status=STATUS_QUIT;
+      break;
+  }
+}
+
+list_part_t *init_list_part(disk_t *disk, const struct ph_options *options)
+{
+  int insert_error=0;
+  list_part_t *list_part;
+  partition_t *partition_wd;
+  list_part=disk->arch->read_part(disk, (options!=NULL?options->verbose:0), 0);
+  partition_wd=new_whole_disk(disk);
+  list_part=insert_new_partition(list_part, partition_wd, 0, &insert_error);
+  if(insert_error>0)
+  {
+    free(partition_wd);
+  }
+  return list_part;
+}
