@@ -50,17 +50,19 @@
 #include "ppartsel.h"
 #include "askloc.h"
 #include "geometry.h"
-#include "addpart.h"
+#include "geometryn.h"
+#include "addpartn.h"
 #include "intrfn.h"
+#include "poptions.h"
 
 extern const arch_fnct_t arch_none;
 
-enum { INIT_SPACE_WHOLE, INIT_SPACE_PREINIT, INIT_SPACE_EXT2_GROUP, INIT_SPACE_EXT2_INODE };
+typedef enum { INIT_SPACE_WHOLE, INIT_SPACE_PREINIT, INIT_SPACE_EXT2_GROUP, INIT_SPACE_EXT2_INODE } init_mode_t;
 
 static int spacerange_cmp(const struct td_list_head *a, const struct td_list_head *b)
 {
-  const alloc_data_t *space_a=td_list_entry(a, const alloc_data_t, list);
-  const alloc_data_t *space_b=td_list_entry(b, const alloc_data_t, list);
+  const alloc_data_t *space_a=td_list_entry_const(a, const alloc_data_t, list);
+  const alloc_data_t *space_b=td_list_entry_const(b, const alloc_data_t, list);
   if(space_a->start < space_b->start)
     return -1;
   if(space_a->start > space_b->start)
@@ -76,13 +78,12 @@ static int spacerange_cmp(const struct td_list_head *a, const struct td_list_hea
 
 void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_data_t*list_search_space)
 {
-  int insert_error=0;
   list_part_t *list_part;
   list_part_t *current_element;
   unsigned int current_element_num;
-  unsigned int carve_free_space_only=0;
+  unsigned int user_blocksize=0;
   int done=0;
-  int mode_init_space=(td_list_empty(&list_search_space->list)?INIT_SPACE_WHOLE:INIT_SPACE_PREINIT);
+  init_mode_t mode_init_space=(td_list_empty(&list_search_space->list)?INIT_SPACE_WHOLE:INIT_SPACE_PREINIT);
 #ifdef HAVE_NCURSES
   int command;
   unsigned int offset=0;
@@ -98,25 +99,10 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
   };
 #endif
   params->blocksize=0;
-  list_part=params->disk->arch->read_part(params->disk,options->verbose,0);
-  {
-    partition_t *partition_wd;
-    partition_wd=new_whole_disk(params->disk);
-    list_part=insert_new_partition(list_part, partition_wd, 0, &insert_error);
-    if(insert_error>0)
-    {
-      free(partition_wd);
-    }
-  }
+  list_part=init_list_part(params->disk, options);
   if(list_part==NULL)
     return;
-  {
-    list_part_t *element;
-    for(element=list_part;element!=NULL;element=element->next)
-    {
-      log_partition(params->disk,element->part);
-    }
-  }
+  log_all_partitions(params->disk, list_part);
   if(list_part->next!=NULL)
   {
     current_element_num=1;
@@ -177,20 +163,19 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
 	  {
 	    init_search_space(list_search_space, params->disk, params->partition);
 	  }
-	  if(carve_free_space_only>0)
+	  if(params->carve_free_space_only>0)
 	  {
 	    params->blocksize=remove_used_space(params->disk, params->partition, list_search_space);
 	  }
-	  photorec(params, options, list_search_space, carve_free_space_only);
+	  if(user_blocksize > 0)
+	    params->blocksize=user_blocksize;
+	  photorec(params, options, list_search_space);
 	}
       }
       else if(strncmp(params->cmd_run,"options",7)==0)
       {
-	const int old_allow_partial_last_cylinder=options->allow_partial_last_cylinder;
 	params->cmd_run+=7;
-	interface_options_photorec(options, &params->cmd_run);
-	if(old_allow_partial_last_cylinder!=options->allow_partial_last_cylinder)
-	  hd_update_geometry(params->disk, options->allow_partial_last_cylinder, options->verbose);
+	interface_options_photorec_cli(options, &params->cmd_run);
       }
       else if(strncmp(params->cmd_run,"fileopt",7)==0)
       {
@@ -200,14 +185,14 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
       else if(strncmp(params->cmd_run,"blocksize,",10)==0)
       {
 	params->cmd_run+=10;
-	params->blocksize=atoi(params->cmd_run);
+	user_blocksize=atoi(params->cmd_run);
 	while(params->cmd_run[0]!=',' && params->cmd_run[0]!='\0')
 	  params->cmd_run++;
       }
       else if(strncmp(params->cmd_run,"geometry,",9)==0)
       {
 	params->cmd_run+=9;
-	change_geometry(params->disk, &params->cmd_run);
+	change_geometry_cli(params->disk, &params->cmd_run);
       }
       else if(strncmp(params->cmd_run,"inter",5)==0)
       {	/* Start interactive mode */
@@ -216,12 +201,12 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
       else if(strncmp(params->cmd_run,"wholespace",10)==0)
       {
 	params->cmd_run+=10;
-	carve_free_space_only=0;
+	params->carve_free_space_only=0;
       }
       else if(strncmp(params->cmd_run,"freespace",9)==0)
       {
 	params->cmd_run+=9;
-	carve_free_space_only=1;
+	params->carve_free_space_only=1;
       }
       else if(strncmp(params->cmd_run,"ext2_group,",11)==0)
       {
@@ -241,6 +226,7 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
           new_free_space->start=groupnr;
           new_free_space->end=groupnr;
           new_free_space->file_stat=NULL;
+	  new_free_space->data=1;
           if(td_list_add_sorted_uniq(&new_free_space->list, &list_search_space->list, spacerange_cmp))
 	    free(new_free_space);
         }
@@ -263,6 +249,7 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
           new_free_space->start=inodenr;
           new_free_space->end=inodenr;
           new_free_space->file_stat=NULL;
+	  new_free_space->data=1;
           if(td_list_add_sorted_uniq(&new_free_space->list, &list_search_space->list, spacerange_cmp))
 	    free(new_free_space);
         }
@@ -387,7 +374,7 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
 	  if(current_element!=NULL)
 	  {
 	    params->partition=current_element->part;
-	    ask_mode_ext2(params->disk, params->partition, &options->mode_ext2, &carve_free_space_only);
+	    ask_mode_ext2(params->disk, params->partition, &options->mode_ext2, &params->carve_free_space_only);
 	    menu=0;
 	    if(params->recup_dir==NULL)
 	    {
@@ -408,7 +395,7 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
 	      {
 		init_search_space(list_search_space, params->disk, params->partition);
 	      }
-	      if(carve_free_space_only>0)
+	      if(params->carve_free_space_only>0)
 	      {
 		aff_copy(stdscr);
 		wmove(stdscr,5,0);
@@ -419,17 +406,14 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
 		 * To carve the whole space, need to quit and reselect the params->partition */
 		done = 1;
 	      }
-	      photorec(params, options, list_search_space, carve_free_space_only);
+	      photorec(params, options, list_search_space);
 	    }
 	  }
 	  break;
 	case 'o':
 	case 'O':
 	  {
-	    const int old_allow_partial_last_cylinder=options->allow_partial_last_cylinder;
-	    interface_options_photorec(options, &params->cmd_run);
-	    if(old_allow_partial_last_cylinder!=options->allow_partial_last_cylinder)
-	      hd_update_geometry(params->disk, options->allow_partial_last_cylinder, options->verbose);
+	    interface_options_photorec_ncurses(options);
 	    menu=1;
 	  }
 	  break;
@@ -441,14 +425,14 @@ void menu_photorec(struct ph_param *params, struct ph_options *options, alloc_da
 	case 'g':
 	case 'G':
 	  if(options->expert!=0)
-	    if(change_geometry(params->disk, &params->cmd_run))
+	    if(change_geometry_ncurses(params->disk))
 	      done=1;
 	  break;
       case 'a':
       case 'A':
 	if(params->disk->arch != &arch_none)
 	{
-	  list_part=add_partition(params->disk, list_part, &params->cmd_run);
+	  list_part=add_partition_ncurses(params->disk, list_part);
 	  current_element=list_part;
 	  current_element_num=0;
 	}
