@@ -46,6 +46,7 @@
 #include "common.h"
 #include "intrf.h"
 #include "filegen.h"
+#include "photorec.h"
 #include "sessionp.h"
 #include "log.h"
 
@@ -60,13 +61,13 @@ int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_
   int taille;
   struct stat stat_rec;
   unsigned int buffer_size;
-  time_t my_time;
+//  time_t my_time;
   char *info=NULL;
   f_session=fopen(SESSION_FILENAME,"rb");
   if(!f_session)
   {
     log_info("Can't open photorec.ses file: %s\n",strerror(errno));
-    session_save(NULL, NULL, NULL, NULL, 0, 0, 0, 0, 0, 0, 0, 0);
+    session_save(NULL, NULL, NULL);
     return -1;
   }
   if(fstat(fileno(f_session), &stat_rec)<0)
@@ -85,7 +86,7 @@ int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_
   }
   pos++;
   /* load time */
-  my_time=strtol(pos,&pos,10);
+  strtol(pos,&pos,10); 	// my_time=strtol(pos,&pos,10);
   if(pos==NULL)
   {
     free(buffer);
@@ -147,6 +148,7 @@ int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_
       new_free_space->start=start;
       new_free_space->end=end;
       new_free_space->file_stat=NULL;
+      new_free_space->data=1;
       td_list_add_tail(&new_free_space->list, &list_free_space->list);
 #ifdef DEBUG
       log_trace(">%lu-%lu<\n", start, end);
@@ -157,64 +159,146 @@ int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_
   }
 }
 
-int session_save(alloc_data_t *list_free_space, disk_t *disk_car, const partition_t *partition, const file_enable_t *files_enable, const unsigned int blocksize, const unsigned int paranoid, const unsigned int keep_corrupted_file, const unsigned int mode_ext2, const unsigned int expert, const unsigned int lowmem, const unsigned int carve_free_space_only, const int verbose)
+int session_save(alloc_data_t *list_free_space, struct ph_param *params,  const struct ph_options *options)
 {
   FILE *f_session;
-  if(verbose>1)
-  {
-    log_trace("session_save\n");
-  }
+  if(params!=NULL && params->status==STATUS_QUIT)
+    return 0;
   f_session=fopen(SESSION_FILENAME,"wb");
   if(!f_session)
   {
     log_critical("Can't create photorec.ses file: %s\n",strerror(errno));
     return -1;
   }
-  if(disk_car!=NULL)
+  if(params!=NULL)
   {
     struct td_list_head *free_walker = NULL;
     unsigned int i;
-    fprintf(f_session,"#%u\n%s %s,%u,blocksize,%u,fileopt,",
-	(unsigned int)time(NULL), disk_car->device, disk_car->arch->part_name_option, partition->order, blocksize);
+    const file_enable_t *files_enable=options->list_file_format;
+    unsigned int disable=0;
+    unsigned int enable=0;
+    unsigned int enable_by_default=0;
+    if(options->verbose>1)
+    {
+      log_trace("session_save\n");
+    }
+    fprintf(f_session,"#%u\n%s %s,%u",
+	(unsigned int)time(NULL), params->disk->device, params->disk->arch->part_name_option, params->partition->order);
+    if(params->blocksize>0)
+      fprintf(f_session,"blocksize,%u,", params->blocksize);
+    fprintf(f_session,"fileopt,");
     for(i=0;files_enable[i].file_hint!=NULL;i++)
     {
-      if(files_enable[i].file_hint->extension!=NULL && files_enable[i].file_hint->extension[0]!='\0')
+      if(files_enable[i].enable==0)
+	disable++;
+      else
+	enable++;
+      if(files_enable[i].enable==files_enable[i].file_hint->enable_by_default)
+	enable_by_default++;
+    }
+    if(enable_by_default >= disable && enable_by_default >= enable)
+    {
+      for(i=0;files_enable[i].file_hint!=NULL;i++)
       {
-	fprintf(f_session,"%s,%s,",files_enable[i].file_hint->extension,(files_enable[i].enable!=0?"enable":"disable"));
+	if(files_enable[i].enable!=files_enable[i].file_hint->enable_by_default &&
+	      files_enable[i].file_hint->extension!=NULL &&
+	      files_enable[i].file_hint->extension[0]!='\0')
+	{
+	  fprintf(f_session,"%s,%s,", files_enable[i].file_hint->extension,
+	      (files_enable[i].enable!=0?"enable":"disable"));
+	}
+      }
+    }
+    else if(enable > disable)
+    {
+      fprintf(f_session,"everything,enable,");
+      for(i=0;files_enable[i].file_hint!=NULL;i++)
+      {
+	if(files_enable[i].enable==0 &&
+	      files_enable[i].file_hint->extension!=NULL &&
+	      files_enable[i].file_hint->extension[0]!='\0')
+	{
+	  fprintf(f_session,"%s,disable,", files_enable[i].file_hint->extension);
+	}
+      }
+    }
+    else
+    {
+      fprintf(f_session,"everything,disable,");
+      for(i=0;files_enable[i].file_hint!=NULL;i++)
+      {
+	if(files_enable[i].enable!=0 &&
+	      files_enable[i].file_hint->extension!=NULL &&
+	      files_enable[i].file_hint->extension[0]!='\0')
+	{
+	  fprintf(f_session,"%s,enable,", files_enable[i].file_hint->extension);
+	}
       }
     }
     /* Save options */
     fprintf(f_session, "options,");
-    if(paranoid==0)
+    if(options->paranoid==0)
       fprintf(f_session, "paranoid_no,");
-    else if(paranoid==1)
+    else if(options->paranoid==1)
       fprintf(f_session, "paranoid,");
     else
       fprintf(f_session, "paranoid_bf,");
-    /* TODO: allow_partial_last_cylinder ? */
-    if(keep_corrupted_file>0)
+    if(options->keep_corrupted_file>0)
       fprintf(f_session, "keep_corrupted_file,");
     else
       fprintf(f_session, "keep_corrupted_file_no,");
-    if(mode_ext2>0)
+    if(options->mode_ext2>0)
       fprintf(f_session, "mode_ext2,");
-    if(expert>0)
+    if(options->expert>0)
       fprintf(f_session, "expert,");
-    if(lowmem>0)
+    if(options->lowmem>0)
       fprintf(f_session, "lowmem,");
     /* Save options - End */
-    if(carve_free_space_only>0)
+    if(params->carve_free_space_only>0)
       fprintf(f_session,"freespace,");
     else
       fprintf(f_session,"wholespace,");
-    fprintf(f_session,"search,inter\n");
+    fprintf(f_session,"search,");
+    switch(params->status)
+    {
+      case STATUS_UNFORMAT:
+        fprintf(f_session, "status=unformat,");
+	break;
+      case STATUS_FIND_OFFSET:
+        fprintf(f_session, "status=find_offset,");
+	break;
+      case STATUS_EXT2_ON_BF:
+	fprintf(f_session, "status=ext2_on_bf,");
+	break;
+      case STATUS_EXT2_ON_SAVE_EVERYTHING:
+	fprintf(f_session, "status=ext2_on_save_everything,");
+	break;
+      case STATUS_EXT2_ON:
+	fprintf(f_session, "status=ext2_on,");
+	break;
+      case STATUS_EXT2_OFF_SAVE_EVERYTHING:
+	fprintf(f_session, "status=ext2_off_save_everything,");
+	break;
+      case STATUS_EXT2_OFF_BF:
+	fprintf(f_session, "status=ext2_off_bf,");
+	break;
+      case STATUS_EXT2_OFF:
+	fprintf(f_session, "status=ext2_off,");
+	break;
+      case STATUS_QUIT:
+        break;
+    }
+    if(params->status!=STATUS_FIND_OFFSET && params->offset!=-1)
+      fprintf(f_session, "%llu,",
+	  (long long unsigned)(params->offset/params->disk->sector_size));
+    fprintf(f_session,"inter\n");
     td_list_for_each(free_walker, &list_free_space->list)
     {
       alloc_data_t *current_free_space;
       current_free_space=td_list_entry(free_walker, alloc_data_t, list);
       fprintf(f_session,"%llu-%llu\n",
-	  (long long unsigned)(current_free_space->start/disk_car->sector_size),
-	  (long long unsigned)(current_free_space->end/disk_car->sector_size));
+	  (long long unsigned)(current_free_space->start/params->disk->sector_size),
+	  (long long unsigned)(current_free_space->end/params->disk->sector_size));
     }
   }
   { /* Reserve some space */
@@ -233,5 +317,3 @@ int session_save(alloc_data_t *list_free_space, disk_t *disk_car, const partitio
   fclose(f_session);
   return 0;
 }
-
-
