@@ -24,7 +24,7 @@ extern "C" {
 #endif
 
 int session_load(char **cmd_device, char **current_cmd, alloc_data_t *list_free_space);
-int session_save(alloc_data_t *list_free_space, struct ph_param *params, const struct ph_options *options, const unsigned int carve_free_space_only);
+int session_save(alloc_data_t *list_free_space, struct ph_param *params, const struct ph_options *options);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
