@@ -45,21 +45,19 @@
 #include "tdelete.h"
 #include "tdiskop.h"
 #include "geometry.h"
+#include "geometryn.h"
 
 extern const arch_fnct_t arch_i386;
 extern const arch_fnct_t arch_none;
 #define INTER_DISK_X	0
-#define INTER_DISK_Y	7
+#define INTER_DISK_Y	8
 
 
 static int menu_disk_cli(disk_t *disk_car, const int verbose,int dump_ind, const int saveheader, char **current_cmd)
 {
-  int align=2;
-  int allow_partial_last_cylinder=0;
+  int align=1;
   int ask_part_order=0;
   unsigned int expert=0;
-  char options[16];
-  strcpy(options, "AGOPTQ");
   while(1)
   {
     while(*current_cmd[0]==',')
@@ -68,40 +66,16 @@ static int menu_disk_cli(disk_t *disk_car, const int verbose,int dump_ind, const
     {
       (*current_cmd)+=7;
       {
-	int search_vista_part=0;
 	list_part_t *list_part;
 	list_part=interface_analyse(disk_car, verbose, saveheader, current_cmd);
-	if(disk_car->arch==&arch_i386)
-	{
-	  const int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
-	  const list_part_t *element;
-	  for(element=list_part;element!=NULL;element=element->next)
-	  {
-	    if(element->part->part_offset%(2048*512)==0 && element->part->part_size%(2048*512)==0)
-	      search_vista_part=1;
-	  }
-	  while(*current_cmd[0]==',')
-	    (*current_cmd)++;
-	  if(strncmp(*current_cmd,"mode_vista",10)==0)
-	  {
-	    (*current_cmd)+=10;
-	    search_vista_part=1;
-	  }
-	  if(search_vista_part==1)
-	    allow_partial_last_cylinder=1;
-	  if(old_allow_partial_last_cylinder!=allow_partial_last_cylinder)
-	    hd_update_geometry(disk_car,allow_partial_last_cylinder,verbose);
-	  log_info("Allow partial last cylinder : %s\n", allow_partial_last_cylinder>0?"Yes":"No");
-	  log_info("search_vista_part: %d\n", search_vista_part);
-	}
-	interface_recovery(disk_car, list_part, verbose, dump_ind, align, ask_part_order, expert, search_vista_part, current_cmd);
+	interface_recovery(disk_car, list_part, verbose, dump_ind, align, ask_part_order, expert, current_cmd);
 	part_free_list(list_part);
       }
     }
     else if(strncmp(*current_cmd,"geometry,",9)==0)
     {
       (*current_cmd)+=9;
-      change_geometry(disk_car, current_cmd);
+      change_geometry_cli(disk_car, current_cmd);
     }
     else if(strncmp(*current_cmd,"advanced",8)==0)
     {
@@ -110,11 +84,8 @@ static int menu_disk_cli(disk_t *disk_car, const int verbose,int dump_ind, const
     }
     else if(strncmp(*current_cmd,"options,",8)==0)
     {
-      const int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
       (*current_cmd)+=8;
-      interface_options(&dump_ind, &align,&allow_partial_last_cylinder,&expert,current_cmd);
-      if(old_allow_partial_last_cylinder!=allow_partial_last_cylinder)
-	hd_update_geometry(disk_car,allow_partial_last_cylinder,verbose);
+      interface_options(&dump_ind, &align, &expert,current_cmd);
     }
     else if(strncmp(*current_cmd,"delete",6)==0)
     {
@@ -134,16 +105,13 @@ static int menu_disk_cli(disk_t *disk_car, const int verbose,int dump_ind, const
 }
 
 #ifdef HAVE_NCURSES
-static int menu_disk_ncurses(disk_t *disk_car, const int verbose,int dump_ind, const int saveheader, char **current_cmd)
+static int menu_disk_ncurses(disk_t *disk, const int verbose,int dump_ind, const int saveheader, char **current_cmd)
 {
-  int align=2;
-  int allow_partial_last_cylinder=0;
+  int align=1;
   int ask_part_order=0;
-  int command;
-  int real_key;
   unsigned int expert=0;
   char options[16];
-  static struct MenuItem menuMain[]=
+  static const struct MenuItem menuMain[]=
   {
 	{'A',"Analyse","Analyse current partition structure and search for lost partitions"},
 	{'T',"Advanced","Filesystem Utils"},
@@ -155,17 +123,26 @@ static int menu_disk_ncurses(disk_t *disk_car, const int verbose,int dump_ind, c
 	{'E',"Editor","Basic disk editor"},
 	{0,NULL,NULL}
   };
-  unsigned int menu=(disk_car->arch == &arch_none ? 1 : 0);
+  unsigned int menu=(disk->arch == &arch_none ? 1 : 0);
   strcpy(options, "AGOPTQ");
-  if(disk_car->arch->write_MBR_code!=NULL)
+  if(disk->arch->write_MBR_code!=NULL)
     strcat(options,"C");
-  if(disk_car->arch->erase_list_part!=NULL)
+  if(disk->arch->erase_list_part!=NULL)
     strcat(options,"D");
   while(1)
   {
+    int real_key;
+    int command;
     aff_copy(stdscr);
     wmove(stdscr,5,0);
-    wprintw(stdscr,"%s\n",disk_car->description(disk_car));
+    wprintw(stdscr, "%s\n", disk->description_short(disk));
+    wmove(stdscr,6,0);
+    if(disk->geom.heads_per_cylinder == 1 && disk->geom.sectors_per_head == 1)
+      wprintw(stdscr, "     %lu sectors", disk->geom.cylinders);
+    else
+      wprintw(stdscr, "     CHS %lu %u %u",
+	  disk->geom.cylinders, disk->geom.heads_per_cylinder, disk->geom.sectors_per_head);
+    wprintw(stdscr, " - sector size=%u", disk->sector_size);
     wmove(stdscr,20,0);
     wprintw(stdscr,"Note: Correct disk geometry is required for a successful recovery. 'Analyse'");
     wmove(stdscr,21,0);
@@ -178,63 +155,37 @@ static int menu_disk_ncurses(disk_t *disk_car, const int verbose,int dump_ind, c
       case 'a':
       case 'A':
 	{
-	  int search_vista_part=0;
 	  list_part_t *list_part;
-	  list_part=interface_analyse(disk_car, verbose, saveheader, current_cmd);
-	  if(disk_car->arch==&arch_i386)
-	  {
-	    const int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
-	    const list_part_t *element;
-	    for(element=list_part;element!=NULL;element=element->next)
-	    {
-	      if(element->part->part_offset%(2048*512)==0 && element->part->part_size%(2048*512)==0)
-		search_vista_part=1;
-	    }
-	    if(search_vista_part==0)
-	    {
-	      log_info("Ask the user for vista mode\n");
-	      if(ask_confirmation("Should TestDisk search for partition created under Vista or later ? [Y/N] (answer Yes if unsure)")!=0)
-		search_vista_part=1;
-	    }
-	    if(search_vista_part==1)
-	      allow_partial_last_cylinder=1;
-	    if(old_allow_partial_last_cylinder!=allow_partial_last_cylinder)
-	      hd_update_geometry(disk_car,allow_partial_last_cylinder,verbose);
-	    log_info("Allow partial last cylinder : %s\n", allow_partial_last_cylinder>0?"Yes":"No");
-	    log_info("search_vista_part: %d\n", search_vista_part);
-	  }
-	  interface_recovery(disk_car, list_part, verbose, dump_ind, align, ask_part_order, expert, search_vista_part, current_cmd);
+	  list_part=interface_analyse(disk, verbose, saveheader, current_cmd);
+	  interface_recovery(disk, list_part, verbose, dump_ind, align, ask_part_order, expert, current_cmd);
 	  part_free_list(list_part);
 	}
 	break;
       case 'd':
       case 'D':
-	write_clean_table(disk_car);
+	write_clean_table(disk);
 	break;
       case 'c':
       case 'C':
-	write_MBR_code(disk_car);
+	write_MBR_code(disk);
 	break;
       case 'g':
       case 'G':
-	change_geometry(disk_car, current_cmd);
+	change_geometry_ncurses(disk);
 	break;
       case 'o':
       case 'O':
 	{
-	  const int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
-	  interface_options(&dump_ind, &align,&allow_partial_last_cylinder,&expert, current_cmd);
-	  if(old_allow_partial_last_cylinder!=allow_partial_last_cylinder)
-	    hd_update_geometry(disk_car,allow_partial_last_cylinder,verbose);
+	  interface_options(&dump_ind, &align, &expert, current_cmd);
 	}
 	break;
       case 't':
       case 'T':
-	interface_adv(disk_car, verbose, dump_ind, expert, current_cmd);
+	interface_adv(disk, verbose, dump_ind, expert, current_cmd);
 	break;
       case 'e':
       case 'E':
-	interface_editor(disk_car);
+	interface_editor(disk);
 	break;
       case 'q':
       case 'Q':
