@@ -49,6 +49,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <errno.h>
 #include "intrface.h"
 #include "fnctdsk.h"
 #include "dir.h"
@@ -65,10 +66,7 @@
 #include "tdisksel.h"
 #include "tlog.h"
 #include "autoset.h"
-
-extern const arch_fnct_t arch_i386;
-extern const arch_fnct_t arch_mac;
-extern const arch_fnct_t arch_sun;
+#include "hidden.h"
 
 #ifdef HAVE_SIGACTION
 static struct sigaction action;
@@ -107,14 +105,8 @@ int main( int argc, char **argv )
   const char *cmd_device=NULL;
   char *cmd_run=NULL;
   const char *logfile="testdisk.log";
-#ifdef TARGET_SOLARIS
-  const arch_fnct_t *arch=&arch_sun;
-#elif defined __APPLE__
-  const arch_fnct_t *arch=&arch_mac;
-#else
-  const arch_fnct_t *arch=&arch_i386;
-#endif
   FILE *log_handle=NULL;
+  int log_errno=0;
   /* srand needed for GPT creation (weak is ok) */
   srand(time(NULL));
 #ifdef HAVE_SIGACTION
@@ -151,12 +143,16 @@ int main( int argc, char **argv )
     {
       if(create_log==TD_LOG_NONE)
         create_log=TD_LOG_APPEND;
+      if(log_handle==NULL)
+	log_handle=log_open(logfile, create_log, &log_errno);
     }
     else if((strcmp(argv[i],"/debug")==0) || (strcmp(argv[i],"-debug")==0))
     {
       verbose++;
       if(create_log==TD_LOG_NONE)
         create_log=TD_LOG_APPEND;
+      if(log_handle==NULL)
+	log_handle=log_open(logfile, create_log, &log_errno);
     }
     else if((strcmp(argv[i],"/all")==0) || (strcmp(argv[i],"-all")==0))
       testdisk_mode|=TESTDISK_O_ALL;
@@ -195,7 +191,7 @@ int main( int argc, char **argv )
 	disk_t *disk_car;
 	cmd_device=argv[++i];
 	cmd_run=argv[++i];
-	disk_car=file_test_availability(cmd_device,verbose,arch,testdisk_mode);
+	disk_car=file_test_availability(cmd_device, verbose, testdisk_mode);
 	if(disk_car==NULL)
 	{
 	  printf("\nUnable to open file or device %s\n",cmd_device);
@@ -207,7 +203,7 @@ int main( int argc, char **argv )
     }
     else
     {
-      disk_t *disk_car=file_test_availability(argv[i],verbose,arch,testdisk_mode);
+      disk_t *disk_car=file_test_availability(argv[i], verbose, testdisk_mode);
       if(disk_car==NULL)
       {
 	printf("\nUnable to open file or device %s\n",argv[i]);
@@ -243,12 +239,16 @@ int main( int argc, char **argv )
 	"It works with :\n" \
 	"- BeFS (BeOS)                           - BSD disklabel (Free/Open/Net BSD)\n" \
 	"- CramFS, Compressed File System        - DOS/Windows FAT12, FAT16 and FAT32\n" \
+	"- XBox FATX                             - Windows exFAT\n" \
 	"- HFS, HFS+, Hierarchical File System   - JFS, IBM's Journaled File System\n" \
-	"- Linux Ext2 and Ext3                   - Linux Raid\n" \
-	"- Linux Swap                            - LVM, LVM2, Logical Volume Manager\n" \
-	"- Netware NSS                           - NTFS (Windows NT/2K/XP/2003)\n" \
-	"- ReiserFS 3.5, 3.6 and 4               - Sun Solaris i386 disklabel\n" \
-	"- UFS and UFS2 (Sun/BSD/...)            - XFS, SGI's Journaled File System\n" \
+	"- Linux btrfs                           - Linux ext2, ext3 and ext4\n" \
+	"- Linux GFS2                            - Linux LUKS\n" \
+	"- Linux Raid                            - Linux Swap\n" \
+	"- LVM, LVM2, Logical Volume Manager     - Netware NSS\n" \
+	"- Windows NTFS                          - ReiserFS 3.5, 3.6 and 4\n" \
+	"- Sun Solaris i386 disklabel            - UFS and UFS2 (Sun/BSD/...)\n" \
+	"- XFS, SGI's Journaled File System      - Wii WBFS\n" \
+	"- Sun ZFS\n" \
 	"\n" \
 	"If you have problems with TestDisk or bug reports, please contact me.\n");
     return 0;
@@ -259,38 +259,57 @@ int main( int argc, char **argv )
     printf("Please wait...\n");
     /* Scan for available device only if no device or image has been supplied in parameter */
     if(list_disk==NULL)
-      list_disk=hd_parse(list_disk,verbose,arch,testdisk_mode);
+      list_disk=hd_parse(list_disk, verbose, testdisk_mode);
     /* Activate the cache */
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
       element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
-#ifdef DJGPP
-    for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
-    {
-      printf("%s\n",element_disk->disk->description(element_disk->disk));
-    }
-#endif
     if(safe==0)
-      hd_update_all_geometry(list_disk,0,verbose);
+      hd_update_all_geometry(list_disk, verbose);
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
-      printf("%s, sector size=%u\n",element_disk->disk->description(element_disk->disk),element_disk->disk->sector_size);
+      disk_t *disk=element_disk->disk;
+      const int hpa_dco=is_hpa_or_dco(disk);
+      printf("%s\n", disk->description(disk));
+      printf("Sector size:%u\n", disk->sector_size);
+      if(disk->model!=NULL)
+	printf("Model: %s", disk->model);
+      if(disk->serial_no!=NULL)
+	printf(", S/N:%s", disk->serial_no);
+      if(disk->fw_rev!=NULL)
+	printf(", FW:%s", disk->fw_rev);
+      printf("\n");
+      if(hpa_dco!=0)
+      {
+	if(disk->sector_size!=0)
+	  printf("size       %llu sectors\n", (long long unsigned)(disk->disk_real_size/disk->sector_size));
+	if(disk->user_max!=0)
+	  printf("user_max   %llu sectors\n", (long long unsigned)disk->user_max);
+	if(disk->native_max!=0)
+	  printf("native_max %llu sectors\n", (long long unsigned)(disk->native_max+1));
+	if(disk->dco!=0)
+	  printf("dco        %llu sectors\n", (long long unsigned)(disk->dco+1));
+	if(hpa_dco&1)
+	  printf("Host Protected Area (HPA) present.\n");
+	if(hpa_dco&2)
+	  printf("Device Configuration Overlay (DCO) present.\n");
+      }
+      printf("\n");
     }
-    printf("\n");
 
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
-      autodetect_arch(element_disk->disk);
+      disk_t *disk=element_disk->disk;
+      autodetect_arch(disk, NULL);
       if(unit==UNIT_DEFAULT)
-	autoset_unit(element_disk->disk);
+	autoset_unit(disk);
       else
-	element_disk->disk->unit=unit;
-      interface_list(element_disk->disk, verbose, saveheader, create_backup);
+	disk->unit=unit;
+      interface_list(disk, verbose, saveheader, create_backup);
+      printf("\n");
     }
     delete_list_disk(list_disk);
     return 0;
   }
-  if(create_log!=TD_LOG_NONE)
-    log_handle=log_open(logfile, create_log);
 #ifdef HAVE_SETLOCALE
   if(run_setlocale>0)
   {
@@ -305,7 +324,7 @@ int main( int argc, char **argv )
   }
 #endif
   if(create_log!=TD_LOG_NONE && log_handle==NULL)
-    log_handle=log_open_default(logfile, create_log);
+    log_handle=log_open_default(logfile, create_log, &log_errno);
 #ifdef HAVE_NCURSES
   /* ncurses need locale for correct unicode support */
   if(start_ncurses("TestDisk",argv[0]))
@@ -315,15 +334,15 @@ int main( int argc, char **argv )
     verbose=1;
     create_log=ask_testdisk_log_creation();
     if(create_log==TD_LOG_CREATE || create_log==TD_LOG_APPEND)
-      log_handle=log_open(logfile, create_log);
+      log_handle=log_open(logfile, create_log, &log_errno);
   }
   {
     const char*filename=logfile;
     while(create_log!=TD_LOG_NONE && log_handle==NULL)
     {
-      filename=ask_log_location(filename);
+      filename=ask_log_location(filename, log_errno);
       if(filename!=NULL)
-	log_handle=log_open(filename, create_log);
+	log_handle=log_open(filename, create_log, &log_errno);
       else
 	create_log=TD_LOG_NONE;
     }
@@ -366,7 +385,7 @@ int main( int argc, char **argv )
 #endif
   /* Scan for available device only if no device or image has been supplied in parameter */
   if(list_disk==NULL)
-    list_disk=hd_parse(list_disk,verbose,arch,testdisk_mode);
+    list_disk=hd_parse(list_disk, verbose, testdisk_mode);
   /* Activate the cache */
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     element_disk->disk=new_diskcache(element_disk->disk,testdisk_mode);
@@ -379,7 +398,7 @@ int main( int argc, char **argv )
   wrefresh(stdscr);
 #endif
   if(safe==0)
-    hd_update_all_geometry(list_disk,0,verbose);
+    hd_update_all_geometry(list_disk, verbose);
   /* save disk parameters to rapport */
   log_info("Hard disk list\n");
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
@@ -395,7 +414,20 @@ int main( int argc, char **argv )
     log_info("\n");
   }
   log_info("\n");
-  use_sudo=do_curses_testdisk(verbose,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
+#ifdef SUDO_BIN
+  if(list_disk==NULL)
+  {
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(DJGPP)
+#else
+#ifdef HAVE_GETEUID
+    if(geteuid()!=0)
+      use_sudo=2;
+#endif
+#endif
+  }
+#endif
+  if(use_sudo==0)
+    use_sudo=do_curses_testdisk(verbose,dump_ind,list_disk,saveheader,cmd_device,&cmd_run);
 #ifdef HAVE_NCURSES
   end_ncurses();
 #endif
@@ -438,17 +470,21 @@ int main( int argc, char **argv )
   {
     printf("TestDisk: Log file corrupted!\n");
   }
-  else
-  {
-    printf("TestDisk exited normally.\n");
-  }
   if(write_used!=0)
   {
     printf("You have to reboot for the change to take effect.\n");
   }
 #ifdef SUDO_BIN
   if(use_sudo>0)
+  {
+    printf("\n");
+    if(use_sudo>1)
+      printf("No disk found.\n");
+    printf("TestDisk will try to restart itself using the sudo command to get\n");
+    printf("root (superuser) privileges.\n");
+    printf("\n");
     run_sudo(argc, argv);
+  }
 #endif
   return 0;
 }
