@@ -24,6 +24,7 @@
 #endif
  
 #include <stdio.h>
+#include <string.h>
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
@@ -34,9 +35,9 @@
 #include "toptions.h"
 
 #ifdef HAVE_NCURSES
-static void interface_options_ncurses(int *dump_ind, int *align, int *allow_partial_last_cylinder, unsigned int *expert)
+static void interface_options_ncurses(int *dump_ind, int *align, unsigned int *expert)
 {
-  unsigned int menu = 4;
+  unsigned int menu = 3;
   /* ncurses interface */
   while (1)
   {
@@ -45,29 +46,16 @@ static void interface_options_ncurses(int *dump_ind, int *align, int *allow_part
     struct MenuItem menuOptions[]=
     {
       { 'E',NULL,"Expert mode adds some functionalities"},
-      { 'C',NULL,"Partitions are aligned on cylinder/head boundaries" },
-      { 'A',NULL,""},
+      { 'C',NULL,"Align partitions to cylinder or 1MiB boundaries" },
       { 'D',NULL,"Dump essential sectors" },
       { 'Q',"[ Ok ]","Done with changing options"},
       { 0, NULL, NULL }
     };
     menuOptions[0].name=*expert?"Expert mode : Yes":"Expert mode : No";
-    switch(*align)
-    {
-      case 0:
-	menuOptions[1].name="Cylinder boundary : No";
-	break;
-      case 1:
-	menuOptions[1].name="Cylinder boundary : Head boundary only";
-	break;
-      case 2:
-	menuOptions[1].name="Cylinder boundary : Yes";
-	break;
-    }
-    menuOptions[2].name=*allow_partial_last_cylinder?"Allow partial last cylinder : Yes":"Allow partial last cylinder : No";
-    menuOptions[3].name=*dump_ind?"Dump : Yes":"Dump : No";
+    menuOptions[1].name=*align?"Align partition: Yes":"Align partition: No";
+    menuOptions[2].name=*dump_ind?"Dump : Yes":"Dump : No";
     aff_copy(stdscr);
-    car=wmenuSelect_ext(stdscr, 23, INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "ECADQ", MENU_VERT|MENU_VERT_ARROW2VALID, &menu,&real_key);
+    car=wmenuSelect_ext(stdscr, 23, INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "ECDQ", MENU_VERT|MENU_VERT_ARROW2VALID, &menu,&real_key);
     switch(car)
     {
       case 'd':
@@ -76,14 +64,7 @@ static void interface_options_ncurses(int *dump_ind, int *align, int *allow_part
 	break;
       case 'c':
       case 'C':
-	if(*align<2)
-	  (*align)++;
-	else
-	  *align=0;
-	break;
-      case 'a':
-      case 'A':
-	*allow_partial_last_cylinder=!*allow_partial_last_cylinder;
+	*align=!*align;
 	break;
       case 'e':
       case 'E':
@@ -98,29 +79,43 @@ static void interface_options_ncurses(int *dump_ind, int *align, int *allow_part
 }
 #endif
 
-void interface_options(int *dump_ind, int *align, int *allow_partial_last_cylinder, unsigned int *expert, char**current_cmd)
+void interface_options(int *dump_ind, int *align, unsigned int *expert, char**current_cmd)
 {
-  if(*current_cmd==NULL)
+  if(*current_cmd!=NULL)
+  {
+    int keep_asking=1;
+    do
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"dump",4)==0)
+      {
+	(*current_cmd)+=4;
+	*dump_ind=1;
+      }
+      else if(strncmp(*current_cmd,"align",5)==0)
+      {
+	(*current_cmd)+=5;
+	*align=1;
+      }
+      else if(strncmp(*current_cmd,"expert",6)==0)
+      {
+	(*current_cmd)+=6;
+	*expert=1;
+      }
+      else
+	keep_asking=0;
+    } while(keep_asking>0);
+  }
+  else
   {
 #ifdef HAVE_NCURSES
-    interface_options_ncurses(dump_ind, align, allow_partial_last_cylinder, expert);
+    interface_options_ncurses(dump_ind, align, expert);
 #endif
   }
   /* write new options to log file */
-  log_info("New options :\n Dump : %s\n ", (*dump_ind?"Yes":"No"));
-  switch(*align)
-  {
-    case 0:
-      log_info("Cylinder boundary : No");
-      break;
-    case 1:
-      log_info("Cylinder boundary : Head boundary only");
-      break;
-    case 2:
-      log_info("Cylinder boundary : Yes");
-      break;
-  }
-  log_info("\n Allow partial last cylinder : %s\n Expert mode : %s\n",
-      *allow_partial_last_cylinder?"Yes":"No",
-      *expert?"Yes":"No");
+  log_info("New options :\n");
+  log_info(" Dump : %s\n", (*dump_ind?"Yes":"No"));
+  log_info(" Align partition: %s\n", (*align?"Yes":"No"));
+  log_info(" Expert mode : %s\n", (*expert?"Yes":"No"));
 }
