@@ -53,12 +53,13 @@
 #include "hdaccess.h"
 #include "alignio.h"
 
+extern const arch_fnct_t arch_none;
+
 static unsigned int file_win32_compute_sector_size(HANDLE handle);
 static uint64_t filewin32_getfilesize(HANDLE handle, const char *device);
 static const char *file_win32_description(disk_t *disk_car);
 static const char *file_win32_description_short(disk_t *disk_car);
-static int file_win32_clean(disk_t *disk_car);
-static void *file_win32_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset);
+static void file_win32_clean(disk_t *disk_car);
 static int file_win32_pread(disk_t *disk_car, void *buf, const unsigned int count, const uint64_t offset);
 static int file_win32_pwrite(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset);
 static int file_win32_nopwrite(disk_t *disk_car, const void *buf, const unsigned int count,  const uint64_t offset);
@@ -246,7 +247,7 @@ static uint64_t filewin32_setfilepointer(HANDLE handle, const char *device)
   return disk_size;
 }
 
-disk_t *file_test_availability_win32(const char *device, const int verbose, const arch_fnct_t *arch, int testdisk_mode)
+disk_t *file_test_availability_win32(const char *device, const int verbose, int testdisk_mode)
 {
   disk_t *disk_car=NULL;
   HANDLE handle=INVALID_HANDLE_VALUE;
@@ -314,7 +315,7 @@ disk_t *file_test_availability_win32(const char *device, const int verbose, cons
     struct info_file_win32_struct *data;
     disk_car=(disk_t *)MALLOC(sizeof(*disk_car));
     init_disk(disk_car);
-    disk_car->arch=arch;
+    disk_car->arch=&arch_none;
     disk_car->device=strdup(device);
     data=(struct info_file_win32_struct *)MALLOC(sizeof(*data));
     data->handle=handle;
@@ -322,7 +323,6 @@ disk_t *file_test_availability_win32(const char *device, const int verbose, cons
     disk_car->data=data;
     disk_car->description=file_win32_description;
     disk_car->description_short=file_win32_description_short;
-    disk_car->pread_fast=file_win32_pread_fast;
     disk_car->pread=file_win32_pread;
     disk_car->pwrite=((data->mode&FILE_WRITE_DATA)==FILE_WRITE_DATA?file_win32_pwrite:file_win32_nopwrite);
     disk_car->sync=file_win32_sync;
@@ -336,11 +336,7 @@ disk_t *file_test_availability_win32(const char *device, const int verbose, cons
     if(disk_car->disk_real_size!=0)
       return disk_car;
     log_warning("Warning: can't get size for %s\n",device);
-    free(data);
-    free(disk_car->device);
-    free(disk_car->model);
-    free(disk_car);
-    CloseHandle(handle);
+    file_win32_clean(disk_car);
   }
   return NULL;
 }
@@ -399,14 +395,14 @@ static const char *file_win32_description_short(disk_t *disk_car)
   return disk_car->description_short_txt;
 }
 
-static int file_win32_clean(disk_t *disk_car)
+static void file_win32_clean(disk_t *disk)
 {
-  if(disk_car->data!=NULL)
+  if(disk->data!=NULL)
   {
-    struct info_file_win32_struct *data=(struct info_file_win32_struct *)disk_car->data;
+    struct info_file_win32_struct *data=(struct info_file_win32_struct *)disk->data;
     CloseHandle(data->handle);
   }
-  return generic_clean(disk_car);
+  generic_clean(disk);
 }
 
 static unsigned int file_win32_compute_sector_size(HANDLE handle)
@@ -498,13 +494,6 @@ static int file_win32_pread(disk_t *disk_car, void *buf, const unsigned int coun
   return align_pread(&file_win32_pread_aux, disk_car, buf, count, offset);
 }
 
-static void *file_win32_pread_fast(disk_t *disk, void *buf, const unsigned int count, const uint64_t offset)
-{
-  if(file_win32_pread(disk, buf, count, offset)==offset)
-    return buf;
-  return NULL;
-}
-
 static int file_win32_pwrite_aux(disk_t *disk_car, const void *buf, const unsigned int count, const uint64_t offset)
 {
   long int ret;
